@@ -1,20 +1,25 @@
 #!/usr/bin/env python
 # Copyright 2004-2005 Joe Wreschnig, Niklas Janlert
+#           2012 Christoph Reiter
 # <quod-libet-development@googlegroups.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
+0 <> 0 # Python 3.x not supported! Use 2.6+ instead.
+
 import os
 import sys
 
-def main():
-    import quodlibet
-    from quodlibet import util
-    from quodlibet import const
-    import gobject
+import quodlibet
+from quodlibet import app
+from quodlibet import util
+from quodlibet import const
+from quodlibet import config
 
+
+def main():
     quodlibet._init_signal()
 
     opts = util.OptionParser(
@@ -25,32 +30,29 @@ def main():
     opts, args = opts.parse()
     args[0] = os.path.realpath(args[0])
 
-    from quodlibet import config
     config.init(const.CONFIG)
 
-    library = quodlibet.init(icon="exfalso",
-                             name="Ex Falso",
-                             title=const.PROCESS_TITLE_EF)
-
-    player = quodlibet.init_backend("nullbe", library.librarian)
+    app.library = quodlibet.init(icon="exfalso",
+                                 name="Ex Falso",
+                                 title=const.PROCESS_TITLE_EF)
+    app.player = quodlibet.init_backend("nullbe", app.librarian)
+    pm = quodlibet.init_plugins()
+    pm.rescan()
 
     from quodlibet.qltk.exfalsowindow import ExFalsoWindow
-    window = ExFalsoWindow(library, args[0])
-
-    quodlibet.enable_periodic_save(save_library=False)
+    app.window = ExFalsoWindow(app.library, args[0])
+    app.window.init_plugins()
 
     from quodlibet.qltk import session
-    session.init("exfalso", window)
+    session.init("exfalso")
 
-    from quodlibet import widgets
-    widgets.main = window
-    widgets.watcher = library.librarian
-
-    quodlibet.main(window)
+    quodlibet.enable_periodic_save(save_library=False)
+    quodlibet.main(app.window)
 
     config.save(const.CONFIG)
 
     print_d("Finished shutdown.")
 
+
 if __name__ == "__main__":
     main()
