@@ -13,13 +13,71 @@ gettext message catalogs.
 
 import os
 import glob
+from subprocess import Popen, PIPE
 
 from distutils.dep_util import newer
 from distutils.util import change_root
 from distutils.spawn import find_executable
-from gdist.core import GCommand
+from distutils.core import Command
 
-class build_mo(GCommand):
+class po_stats(Command):
+    description = "Show translation statistics"
+    user_options = []
+    build_base = None
+
+    def initialize_options(self):
+        pass
+
+    def finalize_options(self):
+        self.po_package = self.distribution.po_package
+        self.po_directory = self.distribution.po_directory
+        self.po_files = glob.glob(os.path.join(self.po_directory, "*.po"))
+        self.set_undefined_options('build', ('build_base', 'build_base'))
+
+    def run(self):
+        self.run_command("build_mo")
+        res = []
+        for po in self.po_files:
+            language = os.path.basename(po).split(".")[0]
+            p = Popen(["msgfmt", "--statistics", po], stdout=PIPE, stderr=PIPE)
+            output = p.communicate()[1]
+            res.append((language, output))
+
+        stats = []
+        for po, r in res:
+            nums = [int(p.split()[0]) for p in r.split(",")]
+            while len(nums) < 3:
+                nums.append(0)
+            trans, fuzzy, untrans = nums
+            stats.append((po, trans, fuzzy, untrans))
+
+        stats.sort(key=lambda x: x[1], reverse=True)
+        print "#"*30
+        for po, trans, fuzzy, untrans in stats:
+            all_ = float(trans + fuzzy + untrans)/100
+            print "%5s: %3d%% (+%2d%% fuzzy)" % (po, trans/all_, fuzzy/all_)
+
+
+class check_pot(Command):
+    description = "check for missing files in POTFILES.in"
+    user_options = []
+
+    def initialize_options(self):
+        pass
+
+    def finalize_options(self):
+        self.po_directory = self.distribution.po_directory
+        self.po_package = self.distribution.po_package
+
+    def run(self):
+        oldpath = os.getcwd()
+        os.chdir(self.po_directory)
+        self.spawn(["intltool-update", "--maintain",
+                    "--gettext-package", self.po_package])
+        os.chdir(oldpath)
+
+
+class build_mo(Command):
     """build message catalog files
 
     Build message catalog (.mo) files from .po files using xgettext
@@ -33,8 +91,11 @@ class build_mo(GCommand):
     po_files = None
     pot_file = None
 
+    def initialize_options(self):
+        pass
+
     def finalize_options(self):
-        GCommand.finalize_options(self)
+        self.po_directory = self.distribution.po_directory
         self.shortcuts = self.distribution.shortcuts
         self.po_package = self.distribution.po_package
         self.set_undefined_options('build', ('build_base', 'build_base'))
@@ -52,6 +113,7 @@ class build_mo(GCommand):
         infiles = file(infilename).read().splitlines()
         pot_name = os.path.join(
             self.po_directory, self.po_package + ".pot")
+
         for filename in infiles:
             if newer(filename, pot_name):
                 oldpath = os.getcwd()
@@ -71,7 +133,7 @@ class build_mo(GCommand):
                 self.mkpath(fullpath)
                 self.spawn(["msgfmt", "-o", destpath, po])
 
-class install_mo(GCommand):
+class install_mo(Command):
     """install message catalog files
 
     Copy compiled message catalog files into their installation
@@ -79,14 +141,17 @@ class install_mo(GCommand):
     """
 
     description = "install message catalog files"
+    user_options = []
 
     skip_build = None
     build_base = None
     install_base = None
     root = None
 
+    def initialize_options(self):
+        pass
+
     def finalize_options(self):
-        GCommand.finalize_options(self)
         self.set_undefined_options('build', ('build_base', 'build_base'))
         self.set_undefined_options(
             'install',
@@ -103,4 +168,4 @@ class install_mo(GCommand):
             dest = change_root(self.root, dest)
         self.copy_tree(src, dest)
 
-__all__ = ["build_mo", "install_mo"]
+__all__ = ["build_mo", "install_mo", "po_stats", "check_pot"]
