@@ -2,28 +2,42 @@
 # -*- coding: utf-8 -*-
 
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
+#           2012, 2013 Christoph Reiter
 # <quod-libet-development@googlegroups.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
+0 <> 0 # Python 3.x not supported! Use 2.6+ instead.
+
 import os
 import signal
 import sys
 import tempfile
-
-import quodlibet
-
-from quodlibet import config
-from quodlibet import const
-from quodlibet import util
-from quodlibet.util.uri import URI
+import stat
 
 play = False
+no_plugins = False
+
 
 def main():
+    process_arguments()
+
+    from quodlibet import const
+    if isrunning() and not const.DEBUG:
+        print_(_("Quod Libet is already running."))
+        control('focus')
+
+    import quodlibet
+    quodlibet._init_signal()
+
     import quodlibet.player
+    from quodlibet import app
+    from quodlibet import config
+    from quodlibet import browsers
+    from quodlibet import const
+    from quodlibet import util
 
     config.init(const.CONFIG)
 
@@ -31,47 +45,92 @@ def main():
                              icon="quodlibet",
                              name="Quod Libet",
                              title=const.PROCESS_TITLE_QL)
+    app.library = library
+
+    from quodlibet.player import PlayerError
+    for backend in [config.get("player", "backend"), "nullbe"]:
+        try:
+            player = quodlibet.init_backend(backend, app.librarian)
+        except PlayerError, error:
+            print_e("%s. %s" % (error.short_desc, error.long_desc))
+        else:
+            break
+    app.player = player
 
     os.environ["PULSE_PROP_media.role"] = "music"
     os.environ["PULSE_PROP_application.icon_name"] = "quodlibet"
 
-    for backend in [config.get("player", "backend"), "nullbe"]:
-        try:
-            player = quodlibet.init_backend(backend, library.librarian)
-        except quodlibet.player.error, error:
-            print_e("%s. %s" % (error.short_desc, error.long_desc))
-        else:
-            break
+    browsers.init()
 
-    from quodlibet import browsers
     from quodlibet.qltk.songlist import SongList
 
-    try: ratings = config.getint("settings", "ratings")
-    except (ValueError, TypeError): pass
-    else: util.RATING_PRECISION = 1.0/ratings
+    try:
+        ratings = config.getint("settings", "ratings")
+    except (ValueError, TypeError):
+        pass
+    else:
+        util.RATING_PRECISION = 1.0 / ratings
 
-    try: default_rating = config.getfloat("settings", "default_rating")
-    except (ValueError, TypeError): pass
-    else: const.DEFAULT_RATING = default_rating
+    try:
+        default_rating = config.getfloat("settings", "default_rating")
+    except (ValueError, TypeError):
+        pass
+    else:
+        const.DEFAULT_RATING = default_rating
 
-    if config.get("settings", "headers").split() == []:
-       config.set("settings", "headers", "title")
-    headers = config.get("settings", "headers").split()
+    try:
+        symbol = config.get("settings", "rating_symbol_full").decode("utf-8")
+    except UnicodeDecodeError:
+        pass
+    else:
+        util.RATING_SYMBOL = symbol
+
+    try:
+        symbol = config.get("settings", "rating_symbol_blank").decode("utf-8")
+    except UnicodeDecodeError:
+        pass
+    else:
+        util.RATING_SYMBOL_BLANK = symbol
+
+    from quodlibet.util.collection import Album
+    try:
+        cover_size = config.getint("browsers", "cover_size")
+    except config.Error:
+        pass
+    else:
+        if cover_size > 0:
+            Album.COVER_SIZE = cover_size
+
+    headers = config.get_columns()
     SongList.set_all_column_headers(headers)
 
     for opt in config.options("header_maps"):
         val = config.get("header_maps", opt)
         util.tags.add(opt, val)
 
-    in_all =("~filename ~uri ~#lastplayed ~#rating ~#playcount ~#skipcount "
-             "~#added ~#bitrate ~current ~#laststarted ~basename "
-             "~dirname").split()
+    in_all = ("~filename ~uri ~#lastplayed ~#rating ~#playcount ~#skipcount "
+              "~#added ~#bitrate ~current ~#laststarted ~basename "
+              "~dirname").split()
     for Kind in browsers.browsers:
-        if Kind.headers is not None: Kind.headers.extend(in_all)
+        if Kind.headers is not None:
+            Kind.headers.extend(in_all)
         Kind.init(library)
 
-    window = init_plugins(player, library)
+    pm = quodlibet.init_plugins(no_plugins)
 
+    if hasattr(player, "init_plugins"):
+        player.init_plugins()
+
+    from quodlibet.qltk.songsmenu import SongsMenu
+    SongsMenu.init_plugins()
+
+    from quodlibet.qltk.quodlibetwindow import QuodLibetWindow
+    app.window = window = QuodLibetWindow(library, player)
+
+    from quodlibet.plugins.events import EventPluginHandler
+    pm.register_handler(EventPluginHandler(library.librarian, player))
+
+    from quodlibet.qltk import mmkeys_ as mmkeys
     from quodlibet.qltk.remote import FSInterface, FIFOControl
     from quodlibet.qltk.tracker import SongTracker
     try:
@@ -79,72 +138,37 @@ def main():
     except ImportError:
         DBusHandler = lambda player, library: None
 
+    mmkeys.init(window, player)
     FSInterface(player)
     FIFOControl(library, window, player)
     DBusHandler(player, library)
     SongTracker(library.librarian, player, window.playlist)
 
-    if play:
-        player.paused = False
+    from quodlibet.qltk import session
+    session.init("quodlibet")
 
     quodlibet.enable_periodic_save(save_library=True)
 
-    from quodlibet.qltk import session
-    session.init("quodlibet", window)
+    if play:
+        player.paused = False
 
     quodlibet.main(window)
 
     print_d("Shutting down player device %r." % player.version_info)
-    quodlibet.player.quit(player)
-
+    player.destroy()
     quodlibet.library.save(force=True)
+
     config.save(const.CONFIG)
 
     print_d("Finished shutdown.")
 
-def init_plugins(player, library):
-    from quodlibet.plugins.editing import EditingPlugins
-    from quodlibet.plugins.songsmenu import SongsMenuPlugins
-    from quodlibet.plugins.events import EventPlugins
-    from quodlibet.plugins.playorder import PlayOrderPlugins
-    from quodlibet.qltk.songsmenu import SongsMenu
-    from quodlibet.qltk.properties import SongProperties
-    from quodlibet import widgets
-
-    widgets.watcher = library.librarian
-
-    SongsMenu.plugins = SongsMenuPlugins(
-        [os.path.join(const.BASEDIR, "plugins", "songsmenu"),
-         os.path.join(const.USERDIR, "plugins", "songsmenu")], "songsmenu")
-    SongsMenu.plugins.rescan()
-
-    SongProperties.plugins = EditingPlugins(
-        [os.path.join(const.BASEDIR, "plugins", "editing"),
-         os.path.join(const.USERDIR, "plugins", "editing")], "editing")
-
-    playorder = PlayOrderPlugins(
-        [os.path.join(const.BASEDIR, "plugins", "playorder"),
-         os.path.join(const.USERDIR, "plugins", "playorder")], "playorder")
-    playorder.rescan()
-
-    # main window
-    from quodlibet.qltk.quodlibetwindow import QuodLibetWindow
-    window = QuodLibetWindow(library, player)
-    widgets.main = window
-
-    events = EventPlugins(library.librarian, player, [
-        os.path.join(const.BASEDIR, "plugins", "events"),
-        os.path.join(const.USERDIR, "plugins", "events")], "events")
-    events.rescan()
-
-    for p in [playorder, SongsMenu.plugins, SongProperties.plugins, events]:
-        window.connect('destroy', p.destroy)
-
-    return window
 
 def print_fifo(command):
+    import quodlibet
+    from quodlibet import const
+
     if not os.path.exists(const.CURRENT):
-        raise SystemExit("not-running")
+        quodlibet.exit("not-running")
     else:
         fd, filename = tempfile.mkstemp()
         try:
@@ -161,48 +185,78 @@ def print_fifo(command):
 
             f = file(filename, "r")
             sys.stdout.write(f.read())
-            try: os.unlink(filename)
-            except EnvironmentError: pass
+            try:
+                os.unlink(filename)
+            except EnvironmentError:
+                pass
             f.close()
-            raise SystemExit
+            quodlibet.exit()
         except TypeError:
-            try: os.unlink(filename)
-            except EnvironmentError: pass
-            raise SystemExit("not-running")
+            try:
+                os.unlink(filename)
+            except EnvironmentError:
+                pass
+            quodlibet.exit("not-running")
+
 
 def print_playing(fstring="<artist~album~tracknumber~title>"):
+    import quodlibet
     from quodlibet.formats._audio import AudioFile
     from quodlibet.parse import Pattern
+    from quodlibet import const
 
     try:
         text = open(const.CURRENT, "rb").read()
         song = AudioFile()
         song.from_dump(text)
         print_(Pattern(fstring).format(song))
-        raise SystemExit
+        quodlibet.exit()
     except (OSError, IOError):
         print_(_("No song is currently playing."))
-        raise SystemExit(True)
+        quodlibet.exit(True)
+
 
 def print_query(query):
     '''Queries library, dumping filenames of matches to stdout
 
         See Issue 716
     '''
-    print_d("Querying library for %r" %query)
+
+    import quodlibet
     import quodlibet.library
+    from quodlibet import const
+
+    print_d("Querying library for %r" % query)
+
     library = quodlibet.library.init(const.LIBRARY)
     songs = library.query(query)
     #songs.sort()
     sys.stdout.write("\n".join([song("~filename") for song in songs]) + "\n")
-    raise SystemExit
+    quodlibet.exit()
+
 
 def isrunning():
+    from quodlibet import const
+
+    # http://code.google.com/p/quodlibet/issues/detail?id=1131
+    # FIXME: There is a race where control() creates a new file
+    # instead of writing to the fifo, confusing the next QL instance.
+    # Remove non-fifos here for now.
+    try:
+        if not stat.S_ISFIFO(os.stat(const.CONTROL).st_mode):
+            print_d("%r not a FIFO. Remove it." % const.CONTROL)
+            os.remove(const.CONTROL)
+    except OSError:
+        pass
     return os.path.exists(const.CONTROL)
 
+
 def control(c):
+    import quodlibet
+    from quodlibet import const
+
     if not isrunning():
-        raise SystemExit(_("Quod Libet is not running."))
+        quodlibet.exit(_("Quod Libet is not running."))
     else:
         try:
             # This is a total abuse of Python! Hooray!
@@ -214,15 +268,23 @@ def control(c):
             f.close()
         except (OSError, IOError, TypeError):
             print_w(_("Unable to write to %s. Removing it.") % const.CONTROL)
-            try: os.unlink(const.CONTROL)
-            except OSError: pass
+            try:
+                os.unlink(const.CONTROL)
+            except OSError:
+                pass
             if c != 'focus':
-                raise SystemExit(True)
+                raise quodlibet.exit(True)
         else:
-            raise SystemExit
+            quodlibet.exit()
+
 
 def process_arguments():
-    controls = ["next", "previous", "play", "pause", "play-pause",
+    import quodlibet
+    from quodlibet.util.uri import URI
+    from quodlibet import util
+    from quodlibet import const
+
+    controls = ["next", "previous", "play", "pause", "play-pause", "stop",
                 "hide-window", "show-window", "toggle-window",
                 "focus", "quit", "unfilter", "refresh", "force-previous"]
     controls_opt = ["seek", "order", "repeat", "query", "volume", "filter",
@@ -245,6 +307,7 @@ def process_arguments():
         ("play", _("Start playback")),
         ("pause", _("Pause playback")),
         ("play-pause", _("Toggle play/pause mode")),
+        ("stop", _("Stop playback")),
         ("volume-up", _("Turn up volume")),
         ("volume-down", _("Turn down volume")),
         ("status", _("Print player status")),
@@ -256,8 +319,10 @@ def process_arguments():
         ("refresh", _("Refresh and rescan library")),
         ("print-playlist", _("Print the current playlist")),
         ("print-queue", _("Print the contents of the queue")),
+        ("no-plugins", _("Start without plugins")),
         ("quit", _("Exit Quod Libet")),
-        ]: options.add(opt, help=help)
+            ]:
+        options.add(opt, help=help)
 
     for opt, help, arg in [
         ("seek", _("Seek within the playing song"), _("[+|-][HH:]MM:SS")),
@@ -278,11 +343,12 @@ def process_arguments():
             Q_("command|filename"), _("query"))),
         ("enqueue-files", _("Enqueue comma-separated files"), "%s[,%s..]" % (
             _("filename"), _("filename"))),
-        ("print-query", _("Print filenames of results of query to stdout"), 
+        ("print-query", _("Print filenames of results of query to stdout"),
             _("query")),
         ("unqueue", _("Unqueue a file or query"), "%s|%s" % (
             Q_("command|filename"), _("query"))),
-        ]: options.add(opt, help=help, arg=arg)
+            ]:
+        options.add(opt, help=help, arg=arg)
 
     options.add("sm-config-prefix", arg="dummy")
     options.add("sm-client-id", arg="prefix")
@@ -290,19 +356,29 @@ def process_arguments():
 
     def is_vol(str):
         if str[0] in '+-':
-            if len(str) == 1: return True
+            if len(str) == 1:
+                return True
             str = str[1:]
         return str.isdigit()
+
     def is_time(str):
-        if str[0] not in "+-0123456789": return False
-        elif str[0] in "+-": str = str[1:]
+        if str[0] not in "+-0123456789":
+            return False
+        elif str[0] in "+-":
+            str = str[1:]
         parts = str.split(":")
-        if len(parts) > 3: return False
-        else: return not (False in [p.isdigit() for p in parts])
+        if len(parts) > 3:
+            return False
+        else:
+            return not (False in [p.isdigit() for p in parts])
+
     def is_float(str):
-        try: float(str)
-        except ValueError: return False
-        else: return True
+        try:
+            float(str)
+        except ValueError:
+            return False
+        else:
+            return True
 
     validators = {
         "order": ["0", "1", "t", "toggle", "inorder", "shuffle",
@@ -315,18 +391,25 @@ def process_arguments():
 
     opts, args = options.parse()
     for command, arg in opts.items():
-        if command in controls: control(command)
+        if command in controls:
+            control(command)
         elif command in controls_opt:
             if command in validators and not validators[command](arg):
                 print_e(_("Invalid argument for '%s'.") % command)
                 print_e(_("Try %s --help.") % sys.argv[0])
-                raise SystemExit(True)
-            else: control(command + " " + arg)
-        elif command == "status": print_fifo("status")
-        elif command == "print-playlist": print_fifo("dump-playlist")
-        elif command == "print-queue": print_fifo("dump-queue")
-        elif command == "volume-up": control("volume +")
-        elif command == "volume-down": control("volume -")
+                quodlibet.exit(True)
+            else:
+                control(command + " " + arg)
+        elif command == "status":
+            print_fifo("status")
+        elif command == "print-playlist":
+            print_fifo("dump-playlist")
+        elif command == "print-queue":
+            print_fifo("dump-queue")
+        elif command == "volume-up":
+            control("volume +")
+        elif command == "volume-down":
+            control("volume -")
         elif command == "enqueue" or command == "unqueue":
             try:
                 filename = URI(arg).filename
@@ -340,22 +423,23 @@ def process_arguments():
                 filename = URI(arg).filename
             except ValueError:
                 filename = os.path.abspath(util.expanduser(arg))
-            if os.path.isdir(filename): control("add-directory " + filename)
-            else: control("add-file " + filename)
+            if os.path.isdir(filename):
+                control("add-directory " + filename)
+            else:
+                control("add-file " + filename)
         elif command == "print-playing":
-            try: print_playing(args[0])
-            except IndexError: print_playing()
+            try:
+                print_playing(args[0])
+            except IndexError:
+                print_playing()
         elif command == "print-query":
             print_query(arg)
         elif command == "start-playing":
             global play
             play = True
+        elif command == "no-plugins":
+            global no_plugins
+            no_plugins = True
 
 if __name__ == "__main__":
-    quodlibet._init_signal()
-
-    process_arguments()
-    if isrunning() and not quodlibet.const.DEBUG:
-        print_(_("Quod Libet is already running."))
-        control('focus')
     main()
