@@ -8,9 +8,7 @@
 
 import os
 
-import gobject
-import gtk
-import pango
+from gi.repository import Gtk, Pango, Gdk, GLib
 
 from quodlibet import config
 from quodlibet import const
@@ -24,11 +22,13 @@ from quodlibet.qltk.completion import EntryWordCompletion
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.textedit import PatternEditBox
 from quodlibet.qltk.views import AllTreeView
-from quodlibet.qltk.x import MenuItem
+from quodlibet.qltk.x import MenuItem, Alignment, ScrolledWindow, RadioMenuItem
+from quodlibet.qltk.x import SymbolicIconImage, SeparatorMenuItem
 from quodlibet.qltk.searchbar import SearchBarBox
 from quodlibet.qltk.menubutton import MenuButton
 from quodlibet.util import copool, gobject_weak, thumbnails
 from quodlibet.util.library import background_filter
+from quodlibet.util.collection import Album
 
 EMPTY = _("Songs not in an album")
 PATTERN = r"""\<b\><album|\<i\><album>\</i\>|%s>\</b\><date| (<date>)>
@@ -37,6 +37,7 @@ PATTERN = r"""\<b\><album|\<i\><album>\</i\>|%s>\</b\><date| (<date>)>
 PATTERN_FN = os.path.join(const.USERDIR, "album_pattern")
 ALBUM_QUERIES = os.path.join(const.USERDIR, "lists", "album_queries")
 
+
 class FakeAlbum(dict):
     def get(self, key, default="", connector=" - "):
         if key[:1] == "~" and '~' in key[1:]:
@@ -53,15 +54,18 @@ class FakeAlbum(dict):
 
     def comma(self, key):
         value = self.get(key)
-        if isinstance(value, (int, float)): return value
+        if isinstance(value, (int, float)):
+            return value
         return value.replace("\n", ", ")
 
+
 class AlbumTagCompletion(EntryWordCompletion):
     def __init__(self):
         super(AlbumTagCompletion, self).__init__()
-        try: model = self.__model
+        try:
+            model = self.__model
         except AttributeError:
-            model = type(self).__model = gtk.ListStore(str)
+            model = type(self).__model = Gtk.ListStore(str)
             self.__refreshmodel()
         self.set_model(model)
         self.set_text_column(0)
@@ -75,31 +79,34 @@ class AlbumTagCompletion(EntryWordCompletion):
             for suffix in ["avg", "max", "min", "sum"]:
                 self.__model.append(row=["#(%s:%s" % (tag, suffix)])
 
+
 class Preferences(qltk.UniqueWindow):
     def __init__(self, parent):
-        if self.is_not_unique(): return
+        if self.is_not_unique():
+            return
         super(Preferences, self).__init__()
         self.set_border_width(12)
         self.set_title(_("Album List Preferences") + " - Quod Libet")
         self.set_default_size(400, 270)
         self.set_transient_for(qltk.get_top_parent(parent))
 
-        box = gtk.VBox(spacing=6)
+        box = Gtk.VBox(spacing=6)
 
         cb = ConfigCheckButton(
             _("Show album _covers"), "browsers", "album_covers")
         cb.set_active(config.getboolean("browsers", "album_covers"))
-        gobject_weak(cb.connect, 'toggled', lambda s: AlbumList.toggle_covers())
-        box.pack_start(cb, expand=False)
+        gobject_weak(cb.connect, 'toggled',
+                     lambda s: AlbumList.toggle_covers())
+        box.pack_start(cb, False, True, 0)
 
         cb = ConfigCheckButton(
             _("Inline _search includes people"),
             "browsers", "album_substrings")
         cb.set_active(config.getboolean("browsers", "album_substrings"))
-        box.pack_start(cb, expand=False)
+        box.pack_start(cb, False, True, 0)
 
-        vbox = gtk.VBox(spacing=6)
-        label = gtk.Label()
+        vbox = Gtk.VBox(spacing=6)
+        label = Gtk.Label()
         label.set_alignment(0.0, 0.5)
         edit = PatternEditBox(PATTERN)
         edit.text = AlbumList._pattern_text
@@ -107,21 +114,21 @@ class Preferences(qltk.UniqueWindow):
         gobject_weak(edit.buffer.connect_object, 'changed',
             self.__preview_pattern, edit, label, parent=edit)
 
-        vbox.pack_start(label, expand=False)
-        vbox.pack_start(edit)
+        vbox.pack_start(label, False, True, 0)
+        vbox.pack_start(edit, True, True, 0)
         self.__preview_pattern(edit, label)
         f = qltk.Frame(_("Album Display"), child=vbox)
-        box.pack_start(f)
+        box.pack_start(f, True, True, 0)
 
-        main_box = gtk.VBox(spacing=12)
-        close = gtk.Button(stock=gtk.STOCK_CLOSE)
+        main_box = Gtk.VBox(spacing=12)
+        close = Gtk.Button(stock=Gtk.STOCK_CLOSE)
         close.connect('clicked', lambda *x: self.destroy())
-        b = gtk.HButtonBox()
-        b.set_layout(gtk.BUTTONBOX_END)
-        b.pack_start(close)
+        b = Gtk.HButtonBox()
+        b.set_layout(Gtk.ButtonBoxStyle.END)
+        b.pack_start(close, True, True, 0)
 
-        main_box.pack_start(box)
-        main_box.pack_start(b, expand=False)
+        main_box.pack_start(box, True, True, 0)
+        main_box.pack_start(b, False, True, 0)
         self.add(main_box)
 
         close.grab_focus()
@@ -133,31 +140,39 @@ class Preferences(qltk.UniqueWindow):
     def __preview_pattern(self, edit, label):
         people = "\n".join(
             [util.tag("artist"), util.tag("performer"), util.tag("arranger")])
-        album = FakeAlbum({"date": "2004-10-31",
+        album = FakeAlbum({
+            "date": "2004-10-31",
             "~length": util.format_time(6319),
             "~long-length": util.format_time_long(6319),
             "~tracks": ngettext("%d track", "%d tracks", 5) % 5,
             "~discs": ngettext("%d disc", "%d discs", 2) % 2,
             "~people": people})
 
-        try: text = XMLFromPattern(edit.text) % album
+        try:
+            text = XMLFromPattern(edit.text) % album
         except:
             text = _("Invalid pattern")
             edit.apply.set_sensitive(False)
-        try: pango.parse_markup(text, u"\u0000")
-        except gobject.GError:
+        try:
+            Pango.parse_markup(text, -1, u"\u0000")
+        except GLib.GError:
             text = _("Invalid pattern")
             edit.apply.set_sensitive(False)
-        else: edit.apply.set_sensitive(True)
+        else:
+            edit.apply.set_sensitive(True)
         label.set_markup(text)
 
+
 def cmpa(a, b):
     """Like cmp but treats values that evaluate to false as inf"""
-    if not a and b: return 1
-    if not b and a: return -1
+    if not a and b:
+        return 1
+    if not b and a:
+        return -1
     return cmp(a, b)
 
-class PreferencesButton(gtk.HBox):
+
+class PreferencesButton(Gtk.HBox):
     def __init__(self, model):
         super(PreferencesButton, self).__init__()
 
@@ -166,21 +181,23 @@ class PreferencesButton(gtk.HBox):
             (_("_Artist"), self.__compare_artist),
             (_("_Date"), self.__compare_date),
             (_("_Genre"), self.__compare_genre),
-            ]
+            (_("_Rating"), self.__compare_rating),
+        ]
 
-        menu = gtk.Menu()
+        menu = Gtk.Menu()
 
-        sort_item = gtk.MenuItem(_("Sort _by..."))
-        sort_menu = gtk.Menu()
+        sort_item = Gtk.MenuItem(_("Sort _by..."), use_underline=True)
+        sort_menu = Gtk.Menu()
 
         active = config.getint('browsers', 'album_sort', 1)
 
         item = None
         for i, (label, func) in enumerate(sort_orders):
-            item = gtk.RadioMenuItem(item, label)
+            item = RadioMenuItem(group=item, label=label,
+                                 use_underline=True)
             model.set_sort_func(100 + i, func)
             if i == active:
-                model.set_sort_column_id(100 + i, gtk.SORT_ASCENDING)
+                model.set_sort_column_id(100 + i, Gtk.SortType.ASCENDING)
                 item.set_active(True)
             gobject_weak(item.connect, "toggled",
                          util.DeferredSignal(self.__sort_toggled_cb),
@@ -190,64 +207,84 @@ class PreferencesButton(gtk.HBox):
         sort_item.set_submenu(sort_menu)
         menu.append(sort_item)
 
-        pref_item = MenuItem(_("_Preferences"), gtk.STOCK_PREFERENCES)
+        pref_item = MenuItem(_("_Preferences"), Gtk.STOCK_PREFERENCES)
         menu.append(pref_item)
-        gobject_weak(pref_item.connect, "activate", Preferences)
+        gobject_weak(pref_item.connect_object, "activate", Preferences, self)
 
         menu.show_all()
 
         button = MenuButton(
-            gtk.image_new_from_stock(
-                gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_MENU),
-                arrow=False)
+                SymbolicIconImage("emblem-system", Gtk.IconSize.MENU),
+                arrow=True)
         button.set_menu(menu)
-        self.pack_start(button)
+        self.pack_start(button, True, True, 0)
 
     def __sort_toggled_cb(self, item, model, num):
         if item.get_active():
             config.set("browsers", "album_sort", str(num))
-            model.set_sort_column_id(100 + num, gtk.SORT_ASCENDING)
+            model.set_sort_column_id(100 + num, Gtk.SortType.ASCENDING)
 
-    def __compare_title(self, model, i1, i2):
+    def __compare_title(self, model, i1, i2, data):
         a1, a2 = model[i1][0], model[i2][0]
         # all albums has to stay at the top
-        if (a1 and a2) is None: return cmp(a1, a2)
+        if (a1 and a2) is None:
+            return cmp(a1, a2)
         # move album without a title to the bottom
-        if not a1.title: return 1
-        if not a2.title: return -1
+        if not a1.title:
+            return 1
+        if not a2.title:
+            return -1
         return (cmpa(a1.sort, a2.sort) or
                 cmp(a1.key, a2.key))
 
-    def __compare_artist(self, model, i1, i2):
+    def __compare_artist(self, model, i1, i2, data):
         a1, a2 = model[i1][0], model[i2][0]
-        if (a1 and a2) is None: return cmp(a1, a2)
-        if not a1.title: return 1
-        if not a2.title: return -1
+        if (a1 and a2) is None:
+            return cmp(a1, a2)
+        if not a1.title:
+            return 1
+        if not a2.title:
+            return -1
         return (cmpa(a1.peoplesort, a2.peoplesort) or
                 cmpa(a1.date, a2.date) or
                 cmpa(a1.sort, a2.sort) or
                 cmp(a1.key, a2.key))
 
-    def __compare_date(self, model, i1, i2):
+    def __compare_date(self, model, i1, i2, data):
         a1, a2 = model[i1][0], model[i2][0]
-        if (a1 and a2) is None: return cmp(a1, a2)
-        if not a1.title: return 1
-        if not a2.title: return -1
+        if (a1 and a2) is None:
+            return cmp(a1, a2)
+        if not a1.title:
+            return 1
+        if not a2.title:
+            return -1
         return (cmpa(a1.date, a2.date) or
                 cmpa(a1.sort, a2.sort) or
                 cmp(a1.key, a2.key))
 
-    def __compare_genre(self, model, i1, i2):
+    def __compare_genre(self, model, i1, i2, data):
         a1, a2 = model[i1][0], model[i2][0]
-        if (a1 and a2) is None: return cmp(a1, a2)
-        if not a1.title: return 1
-        if not a2.title: return -1
+        if (a1 and a2) is None:
+            return cmp(a1, a2)
+        if not a1.title:
+            return 1
+        if not a2.title:
+            return -1
         return (cmpa(a1.genre, a2.genre) or
                 cmpa(a1.peoplesort, a2.peoplesort) or
                 cmpa(a1.date, a2.date) or
                 cmpa(a1.sort, a2.sort) or
                 cmp(a1.key, a2.key))
 
+    def __compare_rating(self, model, i2, i1, data):
+        a1, a2 = model[i1][0], model[i2][0]
+        if (a1 and a2) is None:
+            return cmp(a1, a2)
+        return (cmpa(a1("~#rating"), a2("~#rating")) or
+                cmpa(a1.date, a2.date) or
+                cmpa(a1.sort, a2.sort) or
+                cmp(a1.key, a2.key))
+
 
 class VisibleUpdate(object):
 
@@ -256,11 +293,11 @@ class VisibleUpdate(object):
     PRELOAD_COUNT = 35
 
     def enable_row_update(self, view, sw, column):
-        gobject_weak(view.connect_object, 'expose-event',
+        gobject_weak(view.connect_object, 'draw',
                      self.__update_visibility, view)
 
         gobject_weak(sw.get_vadjustment().connect, "value-changed",
-                     self.__stop_update ,view)
+                     self.__stop_update, view, parent=view)
 
         self.__pending_paths = []
         self.__scan_timeout = None
@@ -269,7 +306,7 @@ class VisibleUpdate(object):
 
     def disable_row_update(self):
         if self.__scan_timeout:
-            gobject.source_remove(self.__scan_timeout)
+            GLib.source_remove(self.__scan_timeout)
             self.__scan_timeout = None
 
         if self.__pending_paths:
@@ -300,13 +337,14 @@ class VisibleUpdate(object):
         if self.__first_expose:
             self.__first_expose = False
             self.__update_visible_rows(view, 0)
-            for i in self.__scan_paths(): pass
+            for i in self.__scan_paths():
+                pass
 
         if self.__scan_timeout:
-            gobject.source_remove(self.__scan_timeout)
+            GLib.source_remove(self.__scan_timeout)
             self.__scan_timeout = None
 
-        self.__scan_timeout = gobject.timeout_add(
+        self.__scan_timeout = GLib.timeout_add(
             50, self.__update_visible_rows, view, self.PRELOAD_COUNT)
 
     def __scan_paths(self):
@@ -315,14 +353,16 @@ class VisibleUpdate(object):
             try:
                 row = model[path]
             # row could have gone away by now
-            except IndexError: pass
+            except IndexError:
+                pass
             else:
                 self._update_row(row)
                 yield True
 
     def __update_visible_rows(self, view, preload):
         vrange = view.get_visible_range()
-        if vrange is None: return
+        if vrange is None:
+            return
 
         model_filter = view.get_model()
         model = model_filter.get_model()
@@ -335,20 +375,24 @@ class VisibleUpdate(object):
         if not start or not end:
             return
 
-        start = start[0] - preload - 1
-        end = end[0] + preload
+        start = start.get_indices()[0] - preload - 1
+        end = end.get_indices()[0] + preload
 
         vlist = range(end, start, -1)
-        top = vlist[:len(vlist)/2]
-        bottom = vlist[len(vlist)/2:]
+        top = vlist[:len(vlist) / 2]
+        bottom = vlist[len(vlist) / 2:]
         top.reverse()
 
         vlist_new = []
         for i in vlist:
-            if top: vlist_new.append(top.pop())
-            if bottom: vlist_new.append(bottom.pop())
+            if top:
+                vlist_new.append(top.pop())
+            if bottom:
+                vlist_new.append(bottom.pop())
         vlist_new = filter(lambda s: s >= 0, vlist_new)
 
+        vlist_new = map(Gtk.TreePath, vlist_new)
+
         visible_paths = []
         for path in vlist_new:
             model_path = model_filter.convert_path_to_child_path(path)
@@ -365,8 +409,7 @@ class VisibleUpdate(object):
         self.__pending_paths = visible_paths
 
 
-class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
-    expand = qltk.RHPaned
+class AlbumList(Browser, Gtk.VBox, util.InstanceTracker, VisibleUpdate):
     __gsignals__ = Browser.__gsignals__
     __model = None
     __no_cover = None
@@ -377,6 +420,16 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
     accelerated_name = _("_Album List")
     priority = 4
 
+    def pack(self, songpane):
+        container = qltk.RHPaned()
+        container.pack1(self, True, False)
+        container.pack2(songpane, True, False)
+        return container
+
+    def unpack(self, container, songpane):
+        container.remove(songpane)
+        container.remove(self)
+
     @classmethod
     def init(klass, library):
         try:
@@ -389,14 +442,16 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
         except EnvironmentError:
             klass._pattern_text = PATTERN
 
-        theme = gtk.icon_theme_get_default()
+        theme = Gtk.IconTheme.get_default()
+        cover_size = Album.COVER_SIZE
         try:
             klass.__no_cover = theme.load_icon(
-                "quodlibet-missing-cover", 48, 0)
-        except gobject.GError: pass
+                "quodlibet-missing-cover", cover_size, 0)
+        except GLib.GError:
+            pass
         else:
             klass.__no_cover = thumbnails.scale(
-                klass.__no_cover, (48, 48))
+                klass.__no_cover, (cover_size, cover_size))
 
         klass._pattern = XMLFromPattern(klass._pattern_text)
 
@@ -418,19 +473,20 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
 
     @classmethod
     def refresh_pattern(klass, pattern_text):
-        if pattern_text == klass._pattern_text: return
+        if pattern_text == klass._pattern_text:
+            return
         klass._pattern_text = pattern_text
         klass._pattern = XMLFromPattern(pattern_text)
         for row in klass.__model:
             klass.__model.row_changed(row.path, row.iter)
         pattern_fn = os.path.join(const.USERDIR, "album_pattern")
         f = file(pattern_fn, "w")
-        f.write(pattern_text  + "\n")
+        f.write(pattern_text + "\n")
         f.close()
 
     @classmethod
     def _init_model(klass, library):
-        klass.__model = model = gtk.ListStore(object)
+        klass.__model = model = Gtk.ListStore(object)
         klass.__library = library
         library.albums.load()
         klass.__sigs = [
@@ -460,7 +516,8 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
             if row[0] and row[0] in removed_albums:
                 removed_albums.remove(row[0])
                 model.remove(row.iter)
-                if not removed_albums: break
+                if not removed_albums:
+                    break
 
     @classmethod
     def _change_albums(klass, library, changed, model):
@@ -470,49 +527,55 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
             if row[0] and row[0] in changed_albums:
                 changed_albums.remove(row[0])
                 model.row_changed(row.path, row.iter)
-                if not changed_albums: break
+                if not changed_albums:
+                    break
 
-    def __init__(self, library, player):
+    def __init__(self, library, main):
         super(AlbumList, self).__init__(spacing=6)
         self._register_instance()
         if self.__model is None:
             self._init_model(library)
 
-        sw = gtk.ScrolledWindow()
-        sw.set_shadow_type(gtk.SHADOW_IN)
+        sw = ScrolledWindow()
+        sw.set_shadow_type(Gtk.ShadowType.IN)
         self.view = view = AllTreeView()
         view.set_headers_visible(False)
-        model_sort = gtk.TreeModelSort(self.__model)
+        model_sort = Gtk.TreeModelSort(model=self.__model)
         model_filter = model_sort.filter_new()
 
         self.__bg_filter = background_filter()
         self.__filter = None
         model_filter.set_visible_func(self.__parse_query)
 
-        render = gtk.CellRendererPixbuf()
-        self.__cover_column = column = gtk.TreeViewColumn("covers", render)
+        render = Gtk.CellRendererPixbuf()
+        self.__cover_column = column = Gtk.TreeViewColumn("covers", render)
         column.set_visible(config.getboolean("browsers", "album_covers"))
-        column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
-        column.set_fixed_width(60)
-        render.set_property('height', 56)
+        column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
+        column.set_fixed_width(Album.COVER_SIZE + 12)
+        render.set_property('height', Album.COVER_SIZE + 8)
 
         def cell_data_pb(column, cell, model, iter, no_cover):
             album = model[iter][0]
-            if album is None: pixbuf = None
-            elif album.cover: pixbuf = album.cover
-            else: pixbuf = no_cover
-            if self.__last_render_pb == pixbuf: return
+            if album is None:
+                pixbuf = None
+            elif album.cover:
+                pixbuf = album.cover
+            else:
+                pixbuf = no_cover
+            if self.__last_render_pb == pixbuf:
+                return
             self.__last_render_pb = pixbuf
             cell.set_property('pixbuf', pixbuf)
 
         column.set_cell_data_func(render, cell_data_pb, self.__no_cover)
         view.append_column(column)
 
-        render = gtk.CellRendererText()
-        column = gtk.TreeViewColumn("albums", render)
-        render.set_property('ellipsize', pango.ELLIPSIZE_END)
+        render = Gtk.CellRendererText()
+        column = Gtk.TreeViewColumn("albums", render)
+        column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
+        render.set_property('ellipsize', Pango.EllipsizeMode.END)
 
-        def cell_data(column, cell, model, iter):
+        def cell_data(column, cell, model, iter, data):
             album = model[iter][0]
             if album is None:
                 text = "<b>%s</b>" % _("All Albums")
@@ -522,7 +585,8 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
             else:
                 markup = AlbumList._pattern % album
 
-            if self.__last_render == markup: return
+            if self.__last_render == markup:
+                return
             self.__last_render = markup
             cell.markup = markup
             cell.set_property('markup', markup)
@@ -530,31 +594,34 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
         column.set_cell_data_func(render, cell_data)
         view.append_column(column)
 
-        view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+        view.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
         view.set_rules_hint(True)
-        view.set_search_equal_func(self.__search_func)
+        view.set_search_equal_func(self.__search_func, None)
         view.set_search_column(0)
         view.set_model(model_filter)
-        sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        sw.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         sw.add(view)
 
-        if player:
+        if main:
             gobject_weak(view.connect, 'row-activated',
-                self.__play_selection, player)
+                         self.__play_selection)
 
-        self.__sig = gobject_weak(view.get_selection().connect, 'changed',
+        self.__sig = gobject_weak(
+            view.get_selection().connect, 'changed',
             util.DeferredSignal(self.__update_songs), parent=view)
 
-        targets = [("text/x-quodlibet-songs", gtk.TARGET_SAME_APP, 1),
+        targets = [("text/x-quodlibet-songs", Gtk.TargetFlags.SAME_APP, 1),
                    ("text/uri-list", 0, 2)]
+        targets = [Gtk.TargetEntry.new(*t) for t in targets]
+
         view.drag_source_set(
-            gtk.gdk.BUTTON1_MASK, targets, gtk.gdk.ACTION_COPY)
+            Gdk.ModifierType.BUTTON1_MASK, targets, Gdk.DragAction.COPY)
         gobject_weak(view.connect, "drag-data-get", self.__drag_data_get)
         gobject_weak(view.connect_object, 'popup-menu',
-            self.__popup, view, library)
+                     self.__popup, view, library)
 
-        self.accelerators = gtk.AccelGroup()
-        search = SearchBarBox(button=False, completion=AlbumTagCompletion(),
+        self.accelerators = Gtk.AccelGroup()
+        search = SearchBarBox(completion=AlbumTagCompletion(),
                               accel_group=self.accelerators)
         gobject_weak(search.connect, 'query-changed', self.__update_filter)
         gobject_weak(search.connect_object,
@@ -562,9 +629,13 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
         self.__search = search
 
         prefs = PreferencesButton(model_sort)
-        search.pack_start(prefs, expand=False)
-        self.pack_start(search, expand=False)
-        self.pack_start(sw, expand=True)
+        search.pack_start(prefs, False, True, 0)
+        if main:
+            self.pack_start(Alignment(search, left=3, top=3), False, True, 0)
+        else:
+            self.pack_start(search, False, True, 0)
+
+        self.pack_start(sw, True, True, 0)
 
         self.connect("destroy", self.__destroy)
 
@@ -584,22 +655,14 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
     def __destroy(self, browser):
         self.disable_row_update()
 
-        # https://bugzilla.gnome.org/show_bug.cgi?id=624112
-        # filter model keeps its filter function reference.
-        # at least try to get rid of as much data as possible.
         self.__inhibit()
-        model = self.view.get_model()
         self.view.set_model(None)
-        model.clear_cache()
-        model = model.get_model()
-        model.clear_cache()
-        self.__dict__.clear()
 
         klass = type(browser)
         if not klass.instances():
             klass._destroy_model()
 
-    def __update_filter(self, entry, text, restore=False):
+    def __update_filter(self, entry, text, scroll_up=True, restore=False):
         model = self.view.get_model()
 
         self.__filter = None
@@ -613,7 +676,7 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
         # but that introduces lots of wild scrolling. Feel free to change it.
         # Without scrolling the TV trys to stay at the same position (40% down)
         # which makes no sence so always go to the top.
-        if not restore:
+        if scroll_up:
             self.view.scroll_to_point(0, 0)
 
         # don't filter on restore if there is nothing to filter
@@ -622,20 +685,23 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
 
         self.__uninhibit()
 
-    def __parse_query(self, model, iter):
-        # leaked filter models try to refilter on model changes
-        if not self.__dict__: return
-
+    def __parse_query(self, model, iter, data):
         f, b = self.__filter, self.__bg_filter
-        if f is None and b is None: return True
-        elif model[iter][0] is None: return True
-        elif b is None: return f(model[iter][0])
-        elif f is None: return b(model[iter][0])
-        else: return b(model[iter][0]) and f(model[iter][0])
+        if f is None and b is None:
+            return True
+        elif model[iter][0] is None:
+            return True
+        elif b is None:
+            return f(model[iter][0])
+        elif f is None:
+            return b(model[iter][0])
+        else:
+            return b(model[iter][0]) and f(model[iter][0])
 
-    def __search_func(self, model, column, key, iter):
+    def __search_func(self, model, column, key, iter, data):
         album = model[iter][0]
-        if album is None: return True
+        if album is None:
+            return True
         key = key.decode('utf-8').lower()
         title = album.title.lower()
         if key in title:
@@ -657,28 +723,28 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
             num = len(albums)
             button = MenuItem(
                 ngettext("Reload album _cover", "Reload album _covers", num),
-                gtk.STOCK_REFRESH)
+                Gtk.STOCK_REFRESH)
             gobject_weak(button.connect, 'activate',
                 self.__refresh_album, view)
-            menu.prepend(gtk.SeparatorMenuItem())
+            menu.prepend(SeparatorMenuItem())
             menu.prepend(button)
 
         menu.show_all()
-        return view.popup_menu(menu, 0, gtk.get_current_event_time())
+        return view.popup_menu(menu, 0, Gtk.get_current_event_time())
 
     def __refresh_album(self, menuitem, view):
         selection = view.get_selection()
         albums = self.__get_selected_albums(selection)
         for album in albums:
-            album.scanned = False
-            album.scan_cover()
+            album.scan_cover(True)
         self._refresh_albums(albums)
 
     def __get_selected_albums(self, selection):
         if not selection:
             return []
         model, rows = selection.get_selected_rows()
-        if not model or not rows: return []
+        if not model or not rows:
+            return []
         if rows and model[rows[0]][0] is None:
             return [row[0] for row in model if row[0]]
         return [model[row][0] for row in rows]
@@ -703,66 +769,62 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
         songs = self.__get_selected_songs(view.get_selection())
         if tid == 1:
             filenames = [song("~filename") for song in songs]
-            sel.set("text/x-quodlibet-songs", 8, "\x00".join(filenames))
-        else: sel.set_uris([song("~uri") for song in songs])
+            type_ = Gdk.atom_intern("text/x-quodlibet-songs", True)
+            sel.set(type_, 8, "\x00".join(filenames))
+        else:
+            sel.set_uris([song("~uri") for song in songs])
 
-    def __play_selection(self, view, indices, col, player):
-        player.reset()
+    def __play_selection(self, view, indices, col):
+        self.emit("activated")
 
     def active_filter(self, song):
         selection = self.view.get_selection()
-        if not selection:
-            return
-
-        model, rows = selection.get_selected_rows()
-        if not model or not rows:
-            return
-
-        if rows and model[rows[0]][0] is None:
-            return True
-
-        key = song.album_key
-        for album in [model[row][0] for row in rows]:
-            if key == album.key:
+        for album in self.__get_selected_albums(selection):
+            if song in album.songs:
                 return True
-
         return False
 
-    def filter(self, key, values):
-        assert(key == "album")
-        if not values: values = [""]
+    def can_filter_text(self):
+        return True
+
+    def filter_text(self, text):
+        self.__search.set_text(text)
+        if Query.is_parsable(text):
+            self.__update_filter(self.__search, text)
+            self.__inhibit()
+            self.view.set_cursor((0,))
+            self.__uninhibit()
+            self.activate()
+
+    def can_filter(self, key):
+        # numerics are different for collections, and title
+        # works, but not of much use here
+        if key is not None and (key.startswith("~#") or key == "title"):
+            return False
+        return super(AlbumList, self).can_filter(key)
+
+    def can_filter_albums(self):
+        return True
+
+    def list_albums(self):
+        model = self.view.get_model()
+        return [row[0].key for row in model if row[0]]
+
+    def filter_albums(self, values):
         view = self.view
         self.__inhibit()
-        selection = view.get_selection()
-        model = view.get_model()
-        first = True
-        for row in model:
-            if row[0] is not None and row[0].title in values:
-                if first:
-                    view.scroll_to_cell(row.path[0],
-                        use_align=True, row_align=0.5)
-                    view.set_cursor(row.path)
-                    first = False
-                else:
-                    selection.select_path(row.path)
+        changed = view.select_by_func(lambda r: r[0] and r[0].key in values)
         self.__uninhibit()
-        if not first:
-            selection.emit('changed')
+        if changed:
+            self.activate()
 
     def unfilter(self):
+        self.filter_text("")
         self.view.set_cursor((0,))
 
     def activate(self):
         self.view.get_selection().emit('changed')
 
-    def can_filter(self, key):
-        return (key == "album")
-
-    def list(self, key):
-        assert (key == "album")
-        model = self.view.get_model()
-        return [row[0].title for row in model if row[0]]
-
     def __inhibit(self):
         self.view.get_selection().handler_block(self.__sig)
 
@@ -776,49 +838,36 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
 
         # update_filter expects a parsable query
         if Query.is_parsable(text):
-            self.__update_filter(entry, text, restore=True)
+            self.__update_filter(entry, text, scroll_up=False, restore=True)
 
         albums = config.get("browsers", "albums").split("\n")
-        view = self.view
-        selection = view.get_selection()
+
         # FIXME: If albums is "" then it could be either all albums or
         # no albums. If it's "" and some other stuff, assume no albums,
         # otherwise all albums.
         self.__inhibit()
         if albums == [""]:
-            selection.unselect_all()
-            selection.select_path((0,))
+            self.view.set_cursor((0,))
         else:
-            model = selection.get_tree_view().get_model()
-            first = True
-            for row in model:
-                if row[0] is not None and row[0].title in albums:
-                    if first:
-                        view.scroll_to_cell(row.path, use_align=True,
-                                            row_align=0.5)
-                        view.set_cursor(row.path)
-                        first = False
-                    else:
-                        selection.select_path(row.path)
+            select = lambda r: r[0] and r[0].title in albums
+            self.view.select_by_func(select)
         self.__uninhibit()
 
     def scroll(self, song):
-        view = self.view
-        model = view.get_model()
         album_key = song.album_key
-        for row in model:
-            if row[0] is not None and row[0].key == album_key:
-                view.scroll_to_cell(row.path[0], use_align=True, row_align=0.5)
-                view.set_cursor(row.path)
-                break
+        select = lambda r: r[0] and r[0].key == album_key
+        self.view.select_by_func(select, one=True)
 
     def __get_config_string(self, selection):
-        if not selection: return ""
+        if not selection:
+            return ""
         model, rows = selection.get_selected_rows()
-        if not model or not rows: return ""
+        if not model or not rows:
+            return ""
 
         # All is selected
-        if rows and model[rows[0]][0] is None: return ""
+        if rows and model[rows[0]][0] is None:
+            return ""
 
         # All selected albums
         albums = (model[row][0] for row in rows)
@@ -837,7 +886,6 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
         config.set("browsers", "query_text", text)
 
     def __update_songs(self, selection):
-        if not self.__dict__: return
         songs = self.__get_selected_songs(selection, False)
         self.emit('songs-selected', songs, None)
 
