@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
+#                2012 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -10,12 +11,10 @@
 # needs them to be there.
 
 import os
-import sys
 
-import gtk
+from gi.repository import Gtk, Gdk
 
 from quodlibet import config
-from quodlibet import const
 from quodlibet import formats
 from quodlibet import qltk
 
@@ -24,20 +23,36 @@ from quodlibet.browsers._base import Browser
 from quodlibet.library import SongFileLibrary
 from quodlibet.qltk.filesel import DirectoryTree
 from quodlibet.qltk.songsmenu import SongsMenu
+from quodlibet.qltk.x import ScrolledWindow
 from quodlibet.util import copool, split_scan_dirs
+from quodlibet.util.dprint import print_d
+from quodlibet.util.uri import URI
 
-class FileSystem(Browser, gtk.ScrolledWindow):
+
+class FileSystem(Browser, Gtk.HBox):
     __gsignals__ = Browser.__gsignals__
-    expand = qltk.RHPaned
+
     __library = None
 
     name = _("File System")
     accelerated_name = _("_File System")
     priority = 10
 
+    TARGET_QL, TARGET_EXT = range(1, 3)
+
+    def pack(self, songpane):
+        container = qltk.RHPaned()
+        container.pack1(self, True, False)
+        container.pack2(songpane, True, False)
+        return container
+
+    def unpack(self, container, songpane):
+        container.remove(songpane)
+        container.remove(self)
+
+    @classmethod
     def __added(klass, library, songs):
         klass.__library.remove(songs)
-    __added = classmethod(__added)
 
     @classmethod
     def init(klass, library):
@@ -50,31 +65,43 @@ class FileSystem(Browser, gtk.ScrolledWindow):
         songs = filter(klass.__library.__contains__, songs)
         klass.__library.remove(songs)
 
-    def __init__(self, library, player):
+    def __init__(self, library, main):
         super(FileSystem, self).__init__()
-        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.set_shadow_type(gtk.SHADOW_IN)
+        sw = ScrolledWindow()
+        sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+        sw.set_shadow_type(Gtk.ShadowType.IN)
 
         folders = filter(None, split_scan_dirs(config.get("settings", "scan")))
 
         dt = DirectoryTree(folders=folders)
-        targets = [("text/x-quodlibet-songs", gtk.TARGET_SAME_APP, 1),
-                   ("text/uri-list", 0, 2)]
-        dt.drag_source_set(gtk.gdk.BUTTON1_MASK, targets, gtk.gdk.ACTION_COPY)
+        targets = [("text/x-quodlibet-songs", Gtk.TargetFlags.SAME_APP,
+                    self.TARGET_QL),
+                   ("text/uri-list", 0, self.TARGET_EXT)]
+        targets = [Gtk.TargetEntry.new(*t) for t in targets]
+
+        dt.drag_source_set(Gdk.ModifierType.BUTTON1_MASK,
+                           targets, Gdk.DragAction.COPY)
         dt.connect('drag-data-get', self.__drag_data_get)
 
         sel = dt.get_selection()
         sel.unselect_all()
         sel.connect_object('changed', copool.add, self.__songs_selected, dt)
-        if player:
-            dt.connect('row-activated', lambda *a: player.reset())
-        self.add(dt)
+        if main:
+            dt.connect('row-activated', lambda *a: self.emit("activated"))
+        sw.add(dt)
+        self.pack_start(sw, True, True, 0)
+
         self.show_all()
 
+    def get_child(self):
+        return self.get_children()[0].get_child()
+
     def __drag_data_get(self, view, ctx, sel, tid, etime):
+        model, rows = view.get_selection().get_selected_rows()
+        dirs = [model[row][0] for row in rows]
         for songs in self.__find_songs(view.get_selection()):
             pass
-        if tid == 1:
+        if tid == self.TARGET_QL:
             cant_add = filter(lambda s: not s.can_add, songs)
             if cant_add:
                 qltk.ErrorMessage(
@@ -86,17 +113,21 @@ class FileSystem(Browser, gtk.ScrolledWindow):
             to_add = filter(self.__library.__contains__, songs)
             self.__add_songs(view, to_add)
             filenames = [song("~filename") for song in songs]
-            sel.set("text/x-quodlibet-songs", 8, "\x00".join(filenames))
+            type_ = Gdk.atom_intern("text/x-quodlibet-songs", True)
+            sel.set(type_, 8, "\x00".join(filenames))
         else:
-            uris = [song("~uri") for song in songs]
+            # External target (app) is delivered a list of URIS of songs
+            uris = list(set([URI.frompath(dir) for dir in dirs]))
+            print_d("Directories to drop: %s" % [u.filename for u in uris])
             sel.set_uris(uris)
 
-    def can_filter(self, key):
-        return (key == "~dirname")
+    def can_filter_tag(self, key):
+        return key == "~dirname"
 
     def filter(self, key, values):
-        self.child.get_selection().unselect_all()
-        for v in values: self.child.go_to(v)
+        self.get_child().get_selection().unselect_all()
+        for v in values:
+            self.get_child().go_to(v)
 
     def scroll(self, song):
         self.__select_paths([song("~dirname")])
@@ -104,7 +135,7 @@ class FileSystem(Browser, gtk.ScrolledWindow):
     def restore(self):
         try:
             paths = config.get("browsers", "filesystem").split("\n")
-        except config.error:
+        except config.Error:
             pass
         else:
             self.__select_paths(paths)
@@ -112,33 +143,35 @@ class FileSystem(Browser, gtk.ScrolledWindow):
     def __select_paths(self, paths):
         def select(model, path, iter, (paths, first)):
             if model[iter][0] in paths:
-                self.child.get_selection().select_path(path)
+                self.get_child().get_selection().select_path(path)
                 paths.remove(model[iter][0])
                 if not first:
-                    self.child.set_cursor(path)
-                    first.append(path)
+                    self.get_child().set_cursor(path)
+                    # copy treepath, gets invalid after the callback
+                    first.append(path.copy())
             else:
                 for fpath in paths:
                     if model[path][0] and fpath.startswith(model[path][0]):
-                        self.child.expand_row(path, False)
+                        self.get_child().expand_row(path, False)
             return not bool(paths)
         first = []
-        self.child.get_model().foreach(select, (paths, first))
-        if first: self.child.scroll_to_cell(first[0], None, True, 0.5)
+        self.get_child().get_model().foreach(select, (paths, first))
+        if first:
+            self.get_child().scroll_to_cell(first[0], None, True, 0.5)
 
     def save(self):
-        model, rows = self.child.get_selection().get_selected_rows()
+        model, rows = self.get_child().get_selection().get_selected_rows()
         paths = "\n".join([model[row][0] for row in rows])
         config.set("browsers", "filesystem", paths)
 
     def activate(self):
-        copool.add(self.__songs_selected, self.child)
+        copool.add(self.__songs_selected, self.get_child())
 
     def Menu(self, songs, songlist, library):
         menu = SongsMenu(library, songs, remove=self.__remove_songs,
                          delete=True, accels=songlist.accelerators,
                          parent=self)
-        i = qltk.MenuItem(_("_Add to Library"), gtk.STOCK_ADD)
+        i = qltk.MenuItem(_("_Add to Library"), Gtk.STOCK_ADD)
         i.set_sensitive(False)
         i.connect('activate', self.__add_songs, songs)
         for song in songs:
@@ -181,17 +214,18 @@ class FileSystem(Browser, gtk.ScrolledWindow):
                             self.__library.reload(song)
                         if song in self.__library:
                             songs.append(song)
-            except OSError: pass
+            except OSError:
+                pass
         self.__library.add(to_add)
         yield songs
 
     def __songs_selected(self, view):
-        if self.window:
-            self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+        if self.get_window():
+            self.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
         for songs in self.__find_songs(view.get_selection()):
             yield True
-        if self.window:
-            self.window.set_cursor(None)
+        if self.get_window():
+            self.get_window().set_cursor(None)
         self.emit('songs-selected', songs, None)
 
 browsers = [FileSystem]
