@@ -5,16 +5,14 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from __future__ import with_statement
-
 import os
+import sys
 import bz2
 import urllib2
 import urllib
 import itertools
 
-import gobject
-import gtk
+from gi.repository import Gtk, GLib
 
 from quodlibet import const
 from quodlibet import qltk
@@ -34,8 +32,9 @@ from quodlibet.util.uri import URI
 from quodlibet.qltk.views import AllTreeView
 from quodlibet.qltk.searchbar import SearchBarBox
 from quodlibet.qltk.completion import LibraryTagCompletion
-from quodlibet.qltk.x import MenuItem
-
+from quodlibet.qltk.x import MenuItem, Alignment, ScrolledWindow
+from quodlibet.qltk.x import SymbolicIconImage, SeparatorMenuItem
+from quodlibet.qltk.menubutton import MenuButton
 
 STATION_LIST_URL = "http://quodlibet.googlecode.com/files/radiolist.bz2"
 STATIONS_FAV = os.path.join(const.USERDIR, "stations")
@@ -45,6 +44,10 @@ STATIONS_ALL = os.path.join(const.USERDIR, "stations_all")
 #       - Ranking: reduce duplicate stations (max 3 URLs per station)
 #                  prefer stations that match a genre?
 
+# Migration path for pickle
+sys.modules["browsers.iradio"] = sys.modules[__name__]
+
+
 class IRFile(RemoteFile):
     multisong = True
     can_add = False
@@ -53,8 +56,7 @@ class IRFile(RemoteFile):
 
     __CAN_CHANGE = "title artist grouping".split()
 
-    def __call__(self, key, *args, **kwargs):
-        base_call = super(IRFile, self).__call__
+    def __get(self, base_call, key, *args, **kwargs):
         if key == "title" and "title" not in self and "organization" in self:
             return base_call("organization", *args, **kwargs)
 
@@ -66,41 +68,75 @@ class IRFile(RemoteFile):
             if len(title) > 1:
                 return (key == "title" and title[-1]) or title[0]
 
+        if key in ("artist", TAG_TO_SORT["artist"]) and \
+                not base_call(key, *args) and "website" in self:
+            return base_call("website", *args)
+
         if key == "~format" and "audio-codec" in self:
             return "%s (%s)" % (self.format,
-                base_call("audio-codec", *args, **kwargs))
+                                base_call("audio-codec", *args, **kwargs))
         return base_call(key, *args, **kwargs)
 
-    def get(self, key, *args):
+    def __call__(self, key, *args, **kwargs):
+        base_call = super(IRFile, self).__call__
+        return self.__get(base_call, key, *args, **kwargs)
+
+    def get(self, key, *args, **kwargs):
         base_call = super(IRFile, self).get
-        if key in ("artist", TAG_TO_SORT["artist"]) and \
-            not base_call(key, *args) and "website" in self:
-            return base_call("website", *args)
-        return base_call(key, *args)
+        return self.__get(base_call, key, *args, **kwargs)
 
     def write(self):
         pass
 
+    def to_dump(self):
+        # dump without title
+        title = None
+        if "title" in self:
+            title = self["title"]
+            del self["title"]
+        dump = super(IRFile, self).to_dump()
+        if title is not None:
+            self["title"] = title
+
+        # add all generated tags
+        lines = dump.splitlines()
+        for tag in ["title", "artist", "~format"]:
+            value = self.get(tag)
+            if value is not None:
+                lines.append("%s=%s" % (tag, util.encode(value)))
+        return "\n".join(lines)
+
     def can_change(self, k=None):
-        if k is None:
-            return self.__CAN_CHANGE
+        if self.streamsong:
+            if k is None:
+                return []
+            else:
+                return False
         else:
-            return k in self.__CAN_CHANGE
+            if k is None:
+                return self.__CAN_CHANGE
+            else:
+                return k in self.__CAN_CHANGE
 
 
 def ParsePLS(file):
     data = {}
 
     lines = file.readlines()
-    if not lines or "[playlist]" not in lines.pop(0): return []
+    if not lines or "[playlist]" not in lines.pop(0):
+        return []
 
     for line in lines:
-        try: head, val = line.strip().split("=", 1)
-        except (TypeError, ValueError): continue
+        try:
+            head, val = line.strip().split("=", 1)
+        except (TypeError, ValueError):
+            continue
         else:
             head = head.lower()
-            if head.startswith("length") and val == "-1": continue
-            else: data[head] = val.decode('utf-8', 'replace')
+            if head.startswith("length") and val == "-1":
+                continue
+            else:
+                data[head] = val.decode('utf-8', 'replace')
 
     count = 1
     files = []
@@ -113,12 +149,17 @@ def ParsePLS(file):
             else:
                 irf = IRFile(filename)
                 for key in ["title", "genre", "artist"]:
-                    try: irf[key] = data["%s%d" % (key, count)]
-                    except KeyError: pass
-                try: irf["~#length"] = int(data["length%d" % count])
-                except (KeyError, TypeError, ValueError): pass
+                    try:
+                        irf[key] = data["%s%d" % (key, count)]
+                    except KeyError:
+                        pass
+                try:
+                    irf["~#length"] = int(data["length%d" % count])
+                except (KeyError, TypeError, ValueError):
+                    pass
                 files.append(irf)
-        else: break
+        else:
+            break
         count += 1
 
     if warnings:
@@ -126,8 +167,9 @@ def ParsePLS(file):
             None, _("Unsupported file type"),
             _("Station lists can only contain locations of stations, "
               "not other station lists or playlists. The following locations "
-              "cannot be loaded:\n%s") % "\n  ".join(map(util.escape, warnings))
-            ).run()
+              "cannot be loaded:\n%s") %
+            "\n  ".join(map(util.escape, warnings))
+        ).run()
 
     return files
 
@@ -138,8 +180,10 @@ def ParseM3U(fileobj):
     for line in fileobj:
         line = line.strip()
         if line.startswith("#EXTINF:"):
-            try: pending_title = line.split(",", 1)[1]
-            except IndexError: pending_title = None
+            try:
+                pending_title = line.split(",", 1)[1]
+            except IndexError:
+                pending_title = None
         elif line.startswith("http"):
             irf = IRFile(line)
             if pending_title:
@@ -157,9 +201,11 @@ def add_station(uri):
         uri = uri.encode('utf-8')
 
     if uri.lower().endswith(".pls") or uri.lower().endswith(".m3u"):
-        try: sock = urllib.urlopen(uri)
+        try:
+            sock = urllib.urlopen(uri)
         except EnvironmentError, e:
-            try: err = e.strerror.decode(const.ENCODING, 'replace')
+            try:
+                err = e.strerror.decode(const.ENCODING, 'replace')
             except (TypeError, AttributeError):
                 err = e.strerror[1].decode(const.ENCODING, 'replace')
             qltk.ErrorMessage(None, _("Unable to add station"), err).run()
@@ -187,12 +233,13 @@ def download_taglist(callback, cofuncid, step=1024 * 10):
     an error."""
 
     with Task(_("Internet Radio"), _("Downloading station list")) as task:
-        if cofuncid: task.copool(cofuncid)
+        if cofuncid:
+            task.copool(cofuncid)
 
         try:
             response = urllib2.urlopen(STATION_LIST_URL)
         except urllib2.URLError:
-            gobject.idle_add(callback, None)
+            GLib.idle_add(callback, None)
             return
 
         try:
@@ -208,8 +255,10 @@ def download_taglist(callback, cofuncid, step=1024 * 10):
         while temp or not data:
             read += len(temp)
 
-            if size: task.update(float(read) / size)
-            else: task.pulse()
+            if size:
+                task.update(float(read) / size)
+            else:
+                task.pulse()
             yield True
 
             try:
@@ -226,7 +275,7 @@ def download_taglist(callback, cofuncid, step=1024 * 10):
         if data:
             stations = parse_taglist(data)
 
-        gobject.idle_add(callback, stations)
+        GLib.idle_add(callback, stations)
 
 
 def parse_taglist(data):
@@ -283,10 +332,10 @@ class AddNewStation(GetStringDialog):
         super(AddNewStation, self).__init__(
             parent, _("New Station"),
             _("Enter the location of an Internet radio station:"),
-            okbutton=gtk.STOCK_ADD)
+            okbutton=Gtk.STOCK_ADD)
 
     def _verify_clipboard(self, text):
-        # try to extract an URI from the clipboard
+        # try to extract a URI from the clipboard
         for line in text.splitlines():
             line = line.strip()
 
@@ -303,7 +352,8 @@ class GenreFilter(object):
 
     # This probably needs improvements
     GENRES = {
-        "electronic": (_("Electronic"),
+        "electronic": (
+            _("Electronic"),
             "|(electr,house,techno,trance,/trip.?hop/,&(drum,n,bass),chill,"
             "dnb,minimal,/down(beat|tempo)/,&(dub,step))"),
         "rap": (_("Hip Hop / Rap"), "|(&(hip,hop),rap)"),
@@ -311,7 +361,8 @@ class GenreFilter(object):
         "r&b": (_("R&B"), "/r(\&|n)b/"),
         "japanese": (_("Japanese"), "|(anime,jpop,japan,jrock)"),
         "indian": (_("Indian"), "|(bollywood,hindi,indian,bhangra)"),
-        "religious": (_("Religious"),
+        "religious": (
+            _("Religious"),
             "|(religious,christian,bible,gospel,spiritual,islam)"),
         "charts": (_("Charts"), "|(charts,hits,top)"),
         "turkish": (_("Turkish"), "|(turkish,turkce)"),
@@ -335,7 +386,8 @@ class GenreFilter(object):
         "lounge": (_("Lounge"), None),
         "punk": (_("Punk"), None),
         "reggaeton": (_("Reggaeton"), None),
-        "slavic": (_("Slavic"),
+        "slavic": (
+            _("Slavic"),
             "|(narodna,albanian,manele,shqip,kosova)"),
         "greek": (_("Greek"), None),
         "gothic": (_("Gothic"), None),
@@ -360,7 +412,7 @@ class GenreFilter(object):
         return self.GENRES[key][0]
 
 
-class InternetRadio(gtk.VBox, Browser, util.InstanceTracker):
+class InternetRadio(Gtk.VBox, Browser, util.InstanceTracker):
     __gsignals__ = Browser.__gsignals__
 
     __stations = None
@@ -384,10 +436,10 @@ class InternetRadio(gtk.VBox, Browser, util.InstanceTracker):
         klass.__librarian = library.librarian
 
         klass.__stations = SongLibrary("iradio-remote")
-        klass.__stations.load(STATIONS_ALL, skip=lambda x: True)
+        klass.__stations.load(STATIONS_ALL)
 
         klass.__fav_stations = SongLibrary("iradio")
-        klass.__fav_stations.load(STATIONS_FAV, skip=lambda x: True)
+        klass.__fav_stations.load(STATIONS_FAV)
 
         klass.filters = GenreFilter()
 
@@ -417,7 +469,7 @@ class InternetRadio(gtk.VBox, Browser, util.InstanceTracker):
         if not self.instances():
             self._destroy()
 
-    def __init__(self, library, player):
+    def __init__(self, library, main):
         super(InternetRadio, self).__init__(spacing=12)
 
         if not self.instances():
@@ -427,108 +479,110 @@ class InternetRadio(gtk.VBox, Browser, util.InstanceTracker):
         self.connect('destroy', self.__destroy)
 
         completion = LibraryTagCompletion(self.__stations)
-        self.accelerators = gtk.AccelGroup()
+        self.accelerators = Gtk.AccelGroup()
         self.__searchbar = search = SearchBarBox(completion=completion,
                                                  accel_group=self.accelerators)
         gobject_weak(search.connect, 'query-changed', self.__filter_changed)
 
+        menu = Gtk.Menu()
+        new_item = MenuItem(_("_New Station..."), Gtk.STOCK_ADD)
+        gobject_weak(new_item.connect, 'activate', self.__add)
+        menu.append(new_item)
+        update_item = MenuItem(_("_Update Stations"), Gtk.STOCK_REFRESH)
+        gobject_weak(update_item.connect, 'activate', self.__update)
+        menu.append(update_item)
+        menu.show_all()
+
+        button = MenuButton(
+            SymbolicIconImage("emblem-system", Gtk.IconSize.MENU),
+            arrow=True)
+        button.set_menu(menu)
+
         def focus(widget, *args):
             qltk.get_top_parent(widget).songlist.grab_focus()
         gobject_weak(search.connect, 'focus-out', focus, parent=self)
 
-        # alignment for search bar
-        search_align = gtk.Alignment(1, 1, 1, 1)
-        search_align.set_property('left-padding', 6)
-        search_align.add(search)
-
-        # left filter pane
-        self.__filter_pane = pane = gtk.VBox(spacing=6)
-
-        # new station button
-        add_button = qltk.Button(_("_New Station"),
-                                 gtk.STOCK_ADD, gtk.ICON_SIZE_MENU)
-        gobject_weak(add_button.connect, 'clicked', self.__add)
-
         # treeview
-        scrolled_window = gtk.ScrolledWindow()
-        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
+        scrolled_window = ScrolledWindow()
+        scrolled_window.set_shadow_type(Gtk.ShadowType.IN)
         self.view = view = AllTreeView()
         view.set_headers_visible(False)
-        scrolled_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        scrolled_window.set_policy(
+            Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         scrolled_window.add(view)
-        model = gtk.ListStore(int, str, str, str)
+        model = Gtk.ListStore(int, str, str, str)
 
-        model.append(row=[self.TYPE_ALL, gtk.STOCK_DIRECTORY, "__all",
+        model.append(row=[self.TYPE_ALL, Gtk.STOCK_DIRECTORY, "__all",
                           _("All Stations")])
-        model.append(row=[self.TYPE_SEP, gtk.STOCK_DIRECTORY, "", ""])
+        model.append(row=[self.TYPE_SEP, Gtk.STOCK_DIRECTORY, "", ""])
         #Translators: Favorite radio stations
-        model.append(row=[self.TYPE_FAV, gtk.STOCK_DIRECTORY, "__fav",
+        model.append(row=[self.TYPE_FAV, Gtk.STOCK_DIRECTORY, "__fav",
                           _("Favorites")])
-        model.append(row=[self.TYPE_SEP, gtk.STOCK_DIRECTORY, "", ""])
+        model.append(row=[self.TYPE_SEP, Gtk.STOCK_DIRECTORY, "", ""])
 
         filters = self.filters
         for text, k in sorted([(filters.text(k), k) for k in filters.keys()]):
-            model.append(row=[self.TYPE_FILTER, gtk.STOCK_FIND, k, text])
+            model.append(row=[self.TYPE_FILTER, Gtk.STOCK_FIND, k, text])
 
-        model.append(row=[self.TYPE_NOCAT, gtk.STOCK_DIRECTORY,
+        model.append(row=[self.TYPE_NOCAT, Gtk.STOCK_DIRECTORY,
                           "nocat", _("No Category")])
 
-        def separator(model, iter):
+        def separator(model, iter, data):
             return model[iter][self.TYPE] == self.TYPE_SEP
-        view.set_row_separator_func(separator)
+        view.set_row_separator_func(separator, None)
 
-        def search_func(model, column, key, iter):
+        def search_func(model, column, key, iter, data):
             return key.lower() not in model[iter][column].lower()
         view.set_search_column(self.NAME)
-        view.set_search_equal_func(search_func)
+        view.set_search_equal_func(search_func, None)
 
-        column = gtk.TreeViewColumn("genres")
-        column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+        column = Gtk.TreeViewColumn("genres")
+        column.set_sizing(Gtk.TreeViewColumnSizing.FIXED)
 
-        renderpb = gtk.CellRendererPixbuf()
+        renderpb = Gtk.CellRendererPixbuf()
+        renderpb.props.xpad = 3
         column.pack_start(renderpb, False)
-        column.set_attributes(renderpb, stock_id=self.STOCK)
+        column.add_attribute(renderpb, "stock_id", self.STOCK)
 
-        render = gtk.CellRendererText()
+        render = Gtk.CellRendererText()
         view.append_column(column)
-        column.pack_start(render)
-        column.set_attributes(render, text=self.NAME)
+        column.pack_start(render, True)
+        column.add_attribute(render, "text", self.NAME)
 
         view.set_model(model)
 
         # selection
         selection = view.get_selection()
-        selection.set_mode(gtk.SELECTION_MULTIPLE)
+        selection.set_mode(Gtk.SelectionMode.MULTIPLE)
         self.__changed_sig = gobject_weak(selection.connect, 'changed',
             util.DeferredSignal(lambda x: self.activate()), parent=view)
 
-        # update button
-        update_button = qltk.Button(_("_Update Stations"), gtk.STOCK_REFRESH,
-                             gtk.ICON_SIZE_MENU)
-        gobject_weak(update_button.connect, 'clicked', self.__update)
-
-
-        self.pack_start(search_align)
-        pane.pack_start(add_button, expand=False)
-        pane.pack_start(scrolled_window)
-        pane.pack_start(update_button, expand=False)
+        box = Gtk.HBox(spacing=6)
+        box.pack_start(search, True, True, 0)
+        box.pack_start(button, False, True, 0)
+        if main:
+            self.pack_start(Alignment(box, left=3, right=3, top=3),
+                            True, True, 0)
+        else:
+            self.pack_start(box, True, True, 0)
+        self.__filter_list = scrolled_window
 
         self.show_all()
 
     def pack(self, songpane):
-        container = qltk.RHPaned()
-        right = gtk.VBox(spacing=6)
-        container.pack1(self.__filter_pane, resize=False, shrink=False)
-        container.pack2(right, resize=True, shrink=False)
-        container.show_all()
-        right.pack_start(self, expand=False)
-        right.pack_start(songpane)
+        container = Gtk.VBox(spacing=6)
+        pane = qltk.RHPaned()
+        pane.pack1(self.__filter_list, resize=False, shrink=False)
+        pane.show_all()
+        pane.pack2(songpane, resize=True, shrink=False)
+        container.pack_start(self, False, True, 0)
+        container.pack_start(pane, True, True, 0)
         return container
 
     def unpack(self, container, songpane):
-        right = container.get_child2()
-        right.remove(songpane)
-        right.remove(self)
+        container.remove(self)
+        pane = container.get_children()[0]
+        pane.remove(songpane)
 
     def __update(self, *args):
         copool.add(download_taglist, self.__update_done,
@@ -669,7 +723,7 @@ class InternetRadio(gtk.VBox, Browser, util.InstanceTracker):
                          queue=False, accels=songlist.accelerators,
                          devices=False, parent=self)
 
-        menu.prepend(gtk.SeparatorMenuItem())
+        menu.prepend(SeparatorMenuItem())
 
         in_fav = False
         in_all = False
@@ -681,13 +735,13 @@ class InternetRadio(gtk.VBox, Browser, util.InstanceTracker):
             if in_fav and in_all:
                 break
 
-        button = MenuItem(_("Remove from Favorites"), gtk.STOCK_REMOVE)
+        button = MenuItem(_("Remove from Favorites"), Gtk.STOCK_REMOVE)
         button.set_sensitive(in_fav)
         gobject_weak(button.connect_object, 'activate',
                      self.__remove_fav, songs)
         menu.prepend(button)
 
-        button = MenuItem(_("Add to Favorites"), gtk.STOCK_ADD)
+        button = MenuItem(_("Add to Favorites"), Gtk.STOCK_ADD)
         button.set_sensitive(in_all)
         gobject_weak(button.connect_object, 'activate',
                      self.__add_fav, songs)
@@ -702,34 +756,17 @@ class InternetRadio(gtk.VBox, Browser, util.InstanceTracker):
             self.__filter_changed(self.__searchbar, text, restore=True)
 
         keys = config.get("browsers", "radio").splitlines()
-        selection = self.view.get_selection()
-        model = self.view.get_model()
-        view = self.view
 
-        # get all paths that have a matching key, and the favorites
-        fav = None
-        found = []
-        for row in model:
-            if row[self.TYPE] != self.TYPE_SEP and row[self.KEY] in keys:
-                found.append(row.path)
-            if row[self.TYPE] == self.TYPE_FAV:
-                fav = row.path
+        def select_func(row):
+            return row[self.TYPE] != self.TYPE_SEP and row[self.KEY] in keys
 
-        # if nothing was found, default to favorites
-        if not found:
-            found.append(fav)
-
-        # select accordingly
         self.__inhibit()
-        selection.unselect_all()
-        first = True
-        for path in found:
-            if first:
-                view.scroll_to_cell(path, use_align=True, row_align=0.5)
-                view.set_cursor(path)
-                first = False
-            else:
-                selection.select_path(path)
+        view = self.view
+        if not view.select_by_func(select_func):
+            for row in view.get_model():
+                if row[self.TYPE] == self.TYPE_FAV:
+                    view.set_cursor(row.path)
+                    break
         self.__uninhibit()
 
     def __get_filter(self):
@@ -799,7 +836,8 @@ class InternetRadio(gtk.VBox, Browser, util.InstanceTracker):
         return ngettext("%(count)d station", "%(count)d stations", i)
 
 
-from quodlibet import player
-if player.can_play_uri("http://"):
+from quodlibet import app
+if not app.player or app.player.can_play_uri("http://"):
     browsers = [InternetRadio]
-else: browsers = []
+else:
+    browsers = []
