@@ -7,13 +7,11 @@
 
 import os
 
-import gtk
-import pango
+from gi.repository import Gtk, Pango
 
 from quodlibet import config
 from quodlibet import devices
 from quodlibet import qltk
-from quodlibet import stock
 from quodlibet import util
 
 from quodlibet.browsers._base import Browser
@@ -23,20 +21,22 @@ from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.wlw import WaitLoadBar
 from quodlibet.qltk.browser import LibraryBrowser
 from quodlibet.qltk.delete import DeleteDialog
+from quodlibet.qltk.x import Alignment, ScrolledWindow
 
-class DeviceProperties(gtk.Dialog):
+
+class DeviceProperties(Gtk.Dialog):
     def __init__(self, parent, device):
         super(DeviceProperties, self).__init__(
             _("Device Properties"), qltk.get_top_parent(parent),
-            buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
+            buttons=(Gtk.STOCK_CLOSE, Gtk.ResponseType.CLOSE))
         self.set_default_size(400, -1)
         self.connect('response', self.__close)
 
-        table = gtk.Table()
+        table = Gtk.Table()
         table.set_border_width(8)
         table.set_row_spacings(8)
         table.set_col_spacings(8)
-        self.vbox.pack_start(table, expand=False)
+        self.vbox.pack_start(table, False, True, 0)
 
         props = []
 
@@ -47,32 +47,34 @@ class DeviceProperties(gtk.Dialog):
 
         props.append((None, None, None))
 
-        entry = gtk.Entry()
+        entry = Gtk.Entry()
         entry.set_text(device['name'])
         props.append((_("_Name:"), entry, 'name'))
 
         y = 0
         for title, value, key in props + device.Properties():
-            if title == None:
-                table.attach(gtk.HSeparator(), 0, 2, y, y + 1)
+            if title is None:
+                table.attach(Gtk.HSeparator(), 0, 2, y, y + 1)
             else:
-                if key and isinstance(value, gtk.CheckButton):
+                if key and isinstance(value, Gtk.CheckButton):
                     value.set_label(title)
                     value.set_use_underline(True)
                     value.connect('toggled', self.__changed, key, device)
-                    table.attach(value, 0, 2, y, y + 1, xoptions=gtk.FILL)
+                    table.attach(value, 0, 2, y, y + 1,
+                                 xoptions=Gtk.AttachOptions.FILL)
                 else:
-                    label = gtk.Label()
+                    label = Gtk.Label()
                     label.set_markup("<b>%s</b>" % util.escape(title))
                     label.set_alignment(0.0, 0.5)
-                    table.attach(label, 0, 1, y, y + 1, xoptions=gtk.FILL)
-                    if key and isinstance(value, gtk.Widget):
+                    table.attach(label, 0, 1, y, y + 1,
+                                 xoptions=Gtk.AttachOptions.FILL)
+                    if key and isinstance(value, Gtk.Widget):
                         widget = value
                         label.set_mnemonic_widget(widget)
                         label.set_use_underline(True)
                         widget.connect('changed', self.__changed, key, device)
                     else:
-                        widget = gtk.Label(value)
+                        widget = Gtk.Label(label=value)
                         widget.set_use_markup(True)
                         widget.set_selectable(True)
                         widget.set_alignment(0.0, 0.5)
@@ -81,11 +83,11 @@ class DeviceProperties(gtk.Dialog):
         self.show_all()
 
     def __changed(self, widget, key, device):
-        if isinstance(widget, gtk.Entry):
+        if isinstance(widget, Gtk.Entry):
             value = widget.get_text()
-        elif isinstance(widget, gtk.SpinButton):
+        elif isinstance(widget, Gtk.SpinButton):
             value = widget.get_value()
-        elif isinstance(widget, gtk.CheckButton):
+        elif isinstance(widget, Gtk.CheckButton):
             value = widget.get_active()
         else:
             raise NotImplementedError
@@ -95,14 +97,15 @@ class DeviceProperties(gtk.Dialog):
         dialog.destroy()
         devices.write()
 
+
 # This will be included in SongsMenu
-class Menu(gtk.Menu):
+class Menu(Gtk.Menu):
     def __init__(self, songs, library):
         super(Menu, self).__init__()
         for device in MediaDevices.devices():
-            i = gtk.ImageMenuItem(device['name'])
+            i = Gtk.ImageMenuItem(device['name'])
             i.set_image(
-                gtk.image_new_from_icon_name(device.icon, gtk.ICON_SIZE_MENU))
+                Gtk.Image.new_from_icon_name(device.icon, Gtk.IconSize.MENU))
             i.set_sensitive(device.is_connected())
             i.connect_object(
                 'activate', self.__copy_to_device, device, songs, library)
@@ -118,7 +121,8 @@ class Menu(gtk.Menu):
         browser.select(device)
         browser.dropped(browser.get_toplevel().songlist, songs)
 
-class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
+
+class MediaDevices(Gtk.VBox, Browser, util.InstanceTracker):
     __gsignals__ = Browser.__gsignals__
 
     name = _("Media Devices")
@@ -126,12 +130,12 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
     priority = 25
     replaygain_profiles = ['track']
 
-    __devices = gtk.ListStore(object, str)
+    __devices = Gtk.ListStore(object, str)
     __busy = False
     __last = None
 
     @staticmethod
-    def cell_data(col, render, model, iter):
+    def cell_data(col, render, model, iter, data):
         device = model[iter][0]
         if device.is_connected():
             render.markup = "<b>%s</b>" % util.escape(device['name'])
@@ -160,90 +164,97 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
                 klass.__devices.remove(row.iter)
                 break
 
-    def __init__(self, library, player):
+    def __init__(self, library, main):
         super(MediaDevices, self).__init__(spacing=6)
         self._register_instance()
 
         self.__cache = {}
 
         # Device list on the left pane
-        swin = gtk.ScrolledWindow()
-        swin.set_shadow_type(gtk.SHADOW_IN)
-        swin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
-        self.pack_start(swin)
+        swin = ScrolledWindow()
+        swin.set_shadow_type(Gtk.ShadowType.IN)
+        swin.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
+        self.pack_start(swin, True, True, 0)
 
         self.__view = view = AllTreeView()
         view.set_model(self.__devices)
         view.set_rules_hint(True)
         view.set_headers_visible(False)
-        view.get_selection().set_mode(gtk.SELECTION_BROWSE)
+        view.get_selection().set_mode(Gtk.SelectionMode.BROWSE)
         view.get_selection().connect_object('changed', self.__refresh, False)
         view.connect('popup-menu', self.__popup_menu, library)
-        if player: view.connect('row-activated', lambda *a: player.reset())
+        if main:
+            view.connect('row-activated', lambda *a: self.emit("activated"))
         swin.add(view)
 
-        col = gtk.TreeViewColumn("Devices")
+        col = Gtk.TreeViewColumn("Devices")
         view.append_column(col)
 
-        render = gtk.CellRendererPixbuf()
-        col.pack_start(render, expand=False)
+        render = Gtk.CellRendererPixbuf()
+        col.pack_start(render, False)
         col.add_attribute(render, 'icon-name', 1)
 
-        self.__render = render = gtk.CellRendererText()
-        render.set_property('ellipsize', pango.ELLIPSIZE_END)
+        self.__render = render = Gtk.CellRendererText()
+        render.set_property('ellipsize', Pango.EllipsizeMode.END)
         render.connect('edited', self.__edited)
-        col.pack_start(render)
+        col.pack_start(render, True)
         col.set_cell_data_func(render, MediaDevices.cell_data)
 
-        hbox = gtk.HBox(spacing=6)
+        hbox = Gtk.HBox(spacing=6)
         hbox.set_homogeneous(True)
-        self.pack_start(hbox, expand=False)
+        if main:
+            self.pack_start(Alignment(hbox, left=3, bottom=3), False, True, 0)
+        else:
+            self.pack_start(hbox, False, True, 0)
 
-        self.__refresh_button = refresh = gtk.Button(stock=gtk.STOCK_REFRESH)
+        self.__refresh_button = refresh = Gtk.Button(stock=Gtk.STOCK_REFRESH)
         refresh.connect_object('clicked', self.__refresh, True)
         refresh.set_sensitive(False)
-        hbox.pack_start(refresh)
+        hbox.pack_start(refresh, True, True, 0)
 
-        self.__eject_button = eject = gtk.Button(_("_Eject"))
+        self.__eject_button = eject = Gtk.Button(_("_Eject"),
+                                                 use_underline=True)
         eject.set_image(
-            gtk.image_new_from_icon_name("media-eject", gtk.ICON_SIZE_BUTTON))
+            Gtk.Image.new_from_icon_name("media-eject", Gtk.IconSize.BUTTON))
         eject.connect('clicked', self.__eject)
         eject.set_sensitive(False)
-        hbox.pack_start(eject)
+        hbox.pack_start(eject, True, True, 0)
 
         # Device info on the right pane
-        self.__header = table = gtk.Table()
+        self.__header = table = Gtk.Table()
         table.set_col_spacings(8)
 
-        self.__device_icon = icon = gtk.Image()
+        self.__device_icon = icon = Gtk.Image()
         icon.set_size_request(48, 48)
         table.attach(icon, 0, 1, 0, 2, 0)
 
-        self.__device_name = label = gtk.Label()
+        self.__device_name = label = Gtk.Label()
+        label.set_ellipsize(Pango.EllipsizeMode.END)
         label.set_alignment(0, 0)
         table.attach(label, 1, 3, 0, 1)
 
-        self.__device_space = label = gtk.Label()
+        self.__device_space = label = Gtk.Label()
+        label.set_ellipsize(Pango.EllipsizeMode.END)
         label.set_alignment(0, 0.5)
         table.attach(label, 1, 2, 1, 2)
 
-        self.__progress = progress = gtk.ProgressBar()
+        self.__progress = progress = Gtk.ProgressBar()
         progress.set_size_request(150, -1)
         table.attach(progress, 2, 3, 1, 2, xoptions=0, yoptions=0)
 
-        self.accelerators = gtk.AccelGroup()
-        key, mod = gtk.accelerator_parse('F2')
-        self.accelerators.connect_group(key, mod, 0, self.__rename)
+        self.accelerators = Gtk.AccelGroup()
+        key, mod = Gtk.accelerator_parse('F2')
+        self.accelerators.connect(key, mod, 0, self.__rename)
 
         self.__statusbar = WaitLoadBar()
 
         self.show_all()
 
     def pack(self, songpane):
-        self.__vbox = vbox = gtk.VBox(spacing=6)
-        vbox.pack_start(self.__header, expand=False)
-        vbox.pack_start(songpane)
-        vbox.pack_start(self.__statusbar, expand=False)
+        self.__vbox = vbox = Gtk.VBox(spacing=6)
+        vbox.pack_start(self.__header, False, True, 0)
+        vbox.pack_start(songpane, True, True, 0)
+        vbox.pack_start(self.__statusbar, False, True, 0)
 
         vbox.show()
         self.__header.show_all()
@@ -252,8 +263,8 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
         self.__statusbar.hide()
 
         self.__paned = paned = qltk.RHPaned()
-        paned.pack1(self)
-        paned.pack2(vbox)
+        paned.pack1(self, True, False)
+        paned.pack2(vbox, True, False)
         return paned
 
     def unpack(self, container, songpane):
@@ -269,7 +280,7 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
             delete = False
 
         menu = SongsMenu(library, songs, delete=delete, remove=False,
-            accels=songlist.accelerators, parent=self)
+                         accels=songlist.accelerators, parent=self)
         return menu
 
     def activate(self):
@@ -282,24 +293,32 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
             config.set('browsers', 'media', model[iter][0]['name'])
 
     def restore(self):
-        try: name = config.get('browsers', 'media')
-        except config.error: pass
+        try:
+            name = config.get('browsers', 'media')
+        except config.Error:
+            pass
         else:
             for row in self.__devices:
-                if row[0]['name'] == name: break
-            else: return
+                if row[0]['name'] == name:
+                    break
+            else:
+                return
             selection = self.__view.get_selection()
             selection.unselect_all()
             selection.select_iter(row.iter)
 
     def select(self, device):
         for row in self.__devices:
-            if row[0] == device: break
-        else: return
+            if row[0] == device:
+                break
+        else:
+            return
 
         # Force a full refresh
-        try: del self.__cache[device.bid]
-        except KeyError: pass
+        try:
+            del self.__cache[device.bid]
+        except KeyError:
+            pass
 
         selection = self.__view.get_selection()
         selection.unselect_all()
@@ -321,15 +340,16 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
 
         menu.preseparate()
 
-        props = gtk.ImageMenuItem(gtk.STOCK_PROPERTIES)
+        props = Gtk.ImageMenuItem(Gtk.STOCK_PROPERTIES, use_stock=True)
         props.connect_object('activate', self.__properties, model[iter][0])
         props.set_sensitive(not self.__busy)
         menu.prepend(props)
 
-        ren = gtk.ImageMenuItem(stock.RENAME)
-        keyval, mod = gtk.accelerator_parse("F2")
+        ren = qltk.MenuItem(_("_Rename"), Gtk.STOCK_EDIT)
+        keyval, mod = Gtk.accelerator_parse("F2")
         ren.add_accelerator(
-            'activate', self.accelerators, keyval, mod, gtk.ACCEL_VISIBLE)
+            'activate', self.accelerators, keyval, mod, Gtk.AccelFlags.VISIBLE)
+
         def rename(path):
             self.__render.set_property('editable', True)
             view.set_cursor(path, view.get_columns()[0], start_editing=True)
@@ -338,22 +358,21 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
 
         menu.preseparate()
 
-        eject = gtk.ImageMenuItem(_("_Eject"))
+        eject = Gtk.ImageMenuItem(_("_Eject"), use_underline=True)
         eject.set_image(
-            gtk.image_new_from_icon_name("media-eject", gtk.ICON_SIZE_MENU))
+            Gtk.Image.new_from_icon_name("media-eject", Gtk.IconSize.MENU))
         eject.set_sensitive(
             not self.__busy and device.eject and device.is_connected())
         eject.connect_object('activate', self.__eject, None)
         menu.prepend(eject)
 
-        refresh = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
+        refresh = Gtk.ImageMenuItem(Gtk.STOCK_REFRESH, use_stock=True)
         refresh.set_sensitive(device.is_connected())
         refresh.connect_object('activate', self.__refresh, True)
         menu.prepend(refresh)
 
         menu.show_all()
-        menu.popup(None, None, None, 0, gtk.get_current_event_time())
-        return True
+        return view.popup_menu(menu, 0, Gtk.get_current_event_time())
 
     def __properties(self, device):
         DeviceProperties(self, device).run()
@@ -376,17 +395,19 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
     def __set_name(self, device):
         self.__device_name.set_markup(
             '<span size="x-large"><b>%s</b></span>' %
-                util.escape(device['name']))
+            util.escape(device['name']))
 
     def __refresh(self, rescan=False):
         model, iter = self.__view.get_selection().get_selected()
         if iter:
             path = model[iter].path
-            if not rescan and self.__last == path: return
+            if not rescan and self.__last == path:
+                return
             self.__last = path
 
             device = model[iter][0]
-            self.__device_icon.set_from_icon_name(device.icon, gtk.ICON_SIZE_DIALOG)
+            self.__device_icon.set_from_icon_name(
+                device.icon, Gtk.IconSize.DIALOG)
             self.__set_name(device)
 
             songs = []
@@ -396,8 +417,10 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
                 self.__refresh_button.set_sensitive(True)
                 self.__refresh_space(device)
 
-                try: songs = self.__list_songs(device, rescan)
-                except NotImplementedError: pass
+                try:
+                    songs = self.__list_songs(device, rescan)
+                except NotImplementedError:
+                    pass
             else:
                 self.__eject_button.set_sensitive(False)
                 self.__refresh_button.set_sensitive(False)
@@ -408,7 +431,8 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
             self.emit('songs-selected', [], False)
 
     def __refresh_space(self, device):
-        try: space, free = device.get_space()
+        try:
+            space, free = device.get_space()
         except NotImplementedError:
             self.__device_space.set_text("")
             self.__progress.hide()
@@ -418,7 +442,7 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
 
             self.__device_space.set_markup(
                 _("<b>%s</b> used, <b>%s</b> available") %
-                    (util.format_size(used), util.format_size(free)))
+                (util.format_size(used), util.format_size(free)))
             self.__progress.set_fraction(fraction)
             self.__progress.set_text("%.f%%" % round(fraction * 100))
             self.__progress.show()
@@ -441,7 +465,8 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
 
     def __copy_songs(self, songlist, songs):
         model, iter = self.__view.get_selection().get_selected()
-        if not iter: return False
+        if not iter:
+            return False
 
         device = model[iter][0]
         if not self.__check_device(device, _("Unable to copy songs")):
@@ -450,7 +475,7 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
         self.__busy = True
 
         wlb = self.__statusbar
-        wlb.setup(len(songs), _("Copying <b>%(song)s</b>"), { 'song': '' })
+        wlb.setup(len(songs), _("Copying <b>%(song)s</b>"), {'song': ''})
         wlb.show()
 
         model = songlist.get_model()
@@ -462,7 +487,8 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
 
             if len(model) > 0:
                 songlist.scroll_to_cell(model[-1].path)
-            while gtk.events_pending(): gtk.main_iteration()
+            while Gtk.events_pending():
+                Gtk.main_iteration()
 
             space, free = device.get_space()
             if free < os.path.getsize(song['~filename']):
@@ -476,8 +502,10 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
             status = device.copy(songlist, song)
             if isinstance(status, AudioFile):
                 model.append([status])
-                try: self.__cache[device.bid].append(song)
-                except KeyError: pass
+                try:
+                    self.__cache[device.bid].append(song)
+                except KeyError:
+                    pass
                 self.__refresh_space(device)
             else:
                 msg = _("<b>%s</b> could not be copied.") % label
@@ -510,7 +538,7 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
         self.__busy = True
 
         wlb = self.__statusbar
-        wlb.setup(len(songs), _("Deleting <b>%(song)s</b>"), { 'song': '' })
+        wlb.setup(len(songs), _("Deleting <b>%(song)s</b>"), {'song': ''})
         wlb.show()
 
         model = songlist.get_model()
@@ -521,10 +549,12 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
                 break
 
             status = device.delete(songlist, song)
-            if status == True:
+            if status is True:
                 model.remove(model.find(song))
-                try: self.__cache[device.bid].remove(song)
-                except (KeyError, ValueError): pass
+                try:
+                    self.__cache[device.bid].remove(song)
+                except (KeyError, ValueError):
+                    pass
                 self.__refresh_space(device)
             else:
                 msg = _("<b>%s</b> could not be deleted.") % label
@@ -545,9 +575,7 @@ class MediaDevices(gtk.VBox, Browser, util.InstanceTracker):
         if iter:
             device = model[iter][0]
             status = device.eject()
-            if status == True:
-                self.__refresh(True)
-            else:
+            if status is not True:
                 msg = _("Ejecting <b>%s</b> failed.") % device['name']
                 if status:
                     msg += "\n\n%s" % status
