@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2005 Joe Wreschnig
+#           2012 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,29 +9,32 @@
 import os
 import urllib
 
-import gobject
-import gtk
-import pango
+from gi.repository import Gtk, GLib, Pango, Gdk
 
 from quodlibet import config
 from quodlibet import const
 from quodlibet import formats
 from quodlibet import qltk
-from quodlibet import stock
 from quodlibet import util
 
 from tempfile import NamedTemporaryFile
 
 from quodlibet.browsers._base import Browser
 from quodlibet.formats._audio import AudioFile
+from quodlibet.util.collection import Playlist
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.views import RCMHintedTreeView
 from quodlibet.qltk.wlw import WaitLoadWindow
 from quodlibet.qltk.getstring import GetStringDialog
+from quodlibet.qltk.x import ScrolledWindow, Alignment, SeparatorMenuItem
 from quodlibet.util.uri import URI
+from quodlibet.util.dprint import print_d
+
 
 PLAYLISTS = os.path.join(const.USERDIR, "playlists")
-if not os.path.isdir(PLAYLISTS): util.mkdir(PLAYLISTS)
+if not os.path.isdir(PLAYLISTS):
+    util.mkdir(PLAYLISTS)
+
 
 def ParseM3U(filename, library=None):
     plname = util.fsdecode(os.path.basename(
@@ -38,31 +42,40 @@ def ParseM3U(filename, library=None):
     filenames = []
     for line in file(filename):
         line = line.strip()
-        if line.startswith("#"): continue
-        else: filenames.append(line)
+        if line.startswith("#"):
+            continue
+        else:
+            filenames.append(line)
     return __ParsePlaylist(plname, filename, filenames, library)
 
+
 def ParsePLS(filename, name="", library=None):
     plname = util.fsdecode(os.path.basename(
         os.path.splitext(filename)[0])).encode('utf-8')
     filenames = []
     for line in file(filename):
         line = line.strip()
-        if not line.lower().startswith("file"): continue
+        if not line.lower().startswith("file"):
+            continue
         else:
-            try: line = line[line.index("=")+1:].strip()
-            except ValueError: pass
-            else: filenames.append(line)
+            try:
+                line = line[line.index("=") + 1:].strip()
+            except ValueError:
+                pass
+            else:
+                filenames.append(line)
     return __ParsePlaylist(plname, filename, filenames, library)
 
+
 def __ParsePlaylist(name, plfilename, files, library):
-    playlist = Playlist.new(name, library=library)
+    playlist = Playlist.new(PLAYLISTS, name, library=library)
     songs = []
     win = WaitLoadWindow(
         None, len(files),
         _("Importing playlist.\n\n%(current)d/%(total)d songs added."))
     for i, filename in enumerate(files):
-        try: uri = URI(filename)
+        try:
+            uri = URI(filename)
         except ValueError:
             # Plain filename.
             filename = os.path.realpath(os.path.join(
@@ -83,173 +96,69 @@ def __ParsePlaylist(name, plfilename, files, library):
             else:
                 # Who knows! Hand it off to GStreamer.
                 songs.append(formats.remote.RemoteFile(uri))
-        if win.step(): break
+        if win.step():
+            break
     win.destroy()
     playlist.extend(filter(None, songs))
     return playlist
 
-class Playlist(list):
-    quote = staticmethod(util.escape_filename)
-    unquote = staticmethod(util.unescape_filename)
 
-    def new(klass, base=_("New Playlist"), library={}):
-        p = Playlist("", library=library)
-        i = 0
-        try: p.rename(base)
-        except ValueError:
-            while not p.name:
-                i += 1
-                try: p.rename("%s %d" % (base, i))
-                except ValueError: pass
-        return p
-    new = classmethod(new)
+class ConfirmRemovePlaylistDialog(qltk.Message):
+    def __init__(self, parent, playlist):
+        title = (_("Are you sure you want to delete the playlist '%s'?")
+                 % playlist.name)
+        description = (_("All information about the selected playlist "
+                         "will be deleted and can not be restored."))
 
-    def fromsongs(klass, songs, library={}):
-        if len(songs) == 1:
-            title = songs[0].comma("title")
-        else:
-            title = ngettext(
-                "%(title)s and %(count)d more",
-                "%(title)s and %(count)d more",
-                len(songs) - 1) % (
-                {'title': songs[0].comma("title"), 'count': len(songs) - 1})
-        playlist = klass.new(title, library=library)
-        playlist.extend(songs)
-        return playlist
-    fromsongs = classmethod(fromsongs)
+        super(ConfirmRemovePlaylistDialog, self).__init__(
+            Gtk.MessageType.WARNING, parent, title, description,
+            Gtk.ButtonsType.NONE)
 
-    def __init__(self, name, library=None):
-        super(Playlist, self).__init__()
-        if isinstance(name, unicode) and os.name != "nt":
-            name = name.encode('utf-8')
-        self.name = name
-        basename = self.quote(name)
-        try:
-            for line in file(os.path.join(PLAYLISTS, basename), "r"):
-                line = util.fsnative(line.rstrip())
-                if line in library:
-                    self.append(library[line])
-                elif library and library.masked(line):
-                    self.append(line)
-        except IOError:
-            if self.name: self.write()
+        self.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                         Gtk.STOCK_DELETE, Gtk.ResponseType.YES)
 
-    def rename(self, newname):
-        if isinstance(newname, unicode): newname = newname.encode('utf-8')
-        if newname == self.name: return
-        elif os.path.exists(os.path.join(PLAYLISTS, self.quote(newname))):
-            raise ValueError(
-                _("A playlist named %s already exists.") % newname)
-        else:
-            try: os.unlink(os.path.join(PLAYLISTS, self.quote(self.name)))
-            except EnvironmentError: pass
-            self.name = newname
-            self.write()
 
-    def add_songs(self, filenames, library):
-        changed = False
-        for i in range(len(self)):
-            if isinstance(self[i], basestring) and self[i] in filenames:
-                self[i] = library[self[i]]
-                changed = True
-        return changed
+class ConfirmRemoveDuplicatesDialog(qltk.Message):
+    def __init__(self, parent, playlist, count):
+        title = ngettext("Are you sure you want to remove %d duplicate song?",
+                         "Are you sure you want to remove %d duplicate songs?",
+                         count) % count
+        description = (_("The duplicate songs will be removed "
+                         "from the playlist '%s'.") % playlist.name)
 
-    def remove_songs(self, songs, library):
-        changed = False
-        # TODO: document the "library.masked" business
-        for song in songs:
-            if library.masked(song("~filename")):
-                while True:
-                    try: self[self.index(song)] = song("~filename")
-                    except ValueError: break
-                    else: changed = True
-            else:
-                while song in self: self.remove(song)
-                else: changed = True
-        return changed
+        super(ConfirmRemoveDuplicatesDialog, self).__init__(
+            Gtk.MessageType.WARNING, parent, title, description,
+            Gtk.ButtonsType.NONE)
 
-    def has_songs(self, songs):
-        # TODO(rm): consider the "library.masked" business
-        some, all = False, True
-        for song in songs:
-            found = song in self
-            some = some or found
-            all = all and found
-            if some and not all:
-                break
-        return some, all
+        self.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                         Gtk.STOCK_DELETE, Gtk.ResponseType.YES)
 
-    def delete(self):
-        del(self[:])
-        try: os.unlink(os.path.join(PLAYLISTS, self.quote(self.name)))
-        except EnvironmentError: pass
-
-    def write(self):
-        basename = self.quote(self.name)
-        f = file(os.path.join(PLAYLISTS, basename), "w")
-        for song in self:
-            try: f.write(util.fsencode(song("~filename")) + "\n")
-            except TypeError: f.write(song + "\n")
-        f.close()
-
-    def file_size(self):
-        """Returns the total file size of tracks in this playlist"""
-        try:
-            # TODO: better way of dealing with missing filesize entries
-            return sum([song.get("~#filesize") or 0
-                        for song in self if isinstance(song, AudioFile)])
-        except EnvironmentError:
-            return _("Total size unknown")
-
-    def length(self):
-        """Returns the total length of tracks in this playlist"""
-        return sum([t.get("~#length") or 0
-                    for t in self if isinstance(t, AudioFile)])
-
-    def format(self):
-        total_size = self.file_size()
-        songs_text = ngettext("%d song", "%d songs", len(self)) % len(self)
-        if (total_size > 0):
-            # see Issue 504
-            return "<b>%s</b>\n<small>%s (%s / %s)</small>" % (
-                util.escape(self.name),
-                songs_text,
-                util.format_time(self.length()),
-                util.format_size(total_size))
-        else:
-            return "<b>%s</b>\n<small>%s (%s)</small>" % (
-                util.escape(self.name),
-                songs_text,
-                util.format_time(self.length()))
-
-    def __cmp__(self, other):
-        try: return cmp(self.name, other.name)
-        except AttributeError: return -1
 
 class GetPlaylistName(GetStringDialog):
     def __init__(self, parent):
         super(GetPlaylistName, self).__init__(
             parent, _("New Playlist"),
             _("Enter a name for the new playlist:"),
-            okbutton=gtk.STOCK_ADD)
+            okbutton=Gtk.STOCK_ADD)
 
-class Menu(gtk.Menu):
+
+class Menu(Gtk.Menu):
     def __init__(self, songs, parent=None):
         super(Menu, self).__init__()
-        i = gtk.MenuItem(_("_New Playlist"))
-        i.connect_object('activate',
-            self.__add_to_playlist, None, songs, parent)
+        i = Gtk.MenuItem(_("_New Playlist"), use_underline=True)
+        i.connect_object(
+            'activate', self.__add_to_playlist, None, songs, parent)
         self.append(i)
-        self.append(gtk.SeparatorMenuItem())
-        self.set_size_request(int(i.size_request()[0] * 2), -1)
+        self.append(SeparatorMenuItem())
+        self.set_size_request(int(i.size_request().width * 2), -1)
 
         for playlist in Playlists.playlists():
             name = playlist.name
-            i = gtk.CheckMenuItem(name)
+            i = Gtk.CheckMenuItem(name)
             some, all = playlist.has_songs(songs)
             i.set_active(some)
             i.set_inconsistent(some and not all)
-            i.child.set_ellipsize(pango.ELLIPSIZE_END)
+            i.get_child().set_ellipsize(Pango.EllipsizeMode.END)
             i.connect_object(
                 'activate', self.__add_to_playlist, playlist, songs, parent)
             self.append(i)
@@ -262,40 +171,59 @@ class Menu(gtk.Menu):
                 title = ngettext(
                     "%(title)s and %(count)d more",
                     "%(title)s and %(count)d more",
-                    len(songs) - 1) % (
-                    {'title': songs[0].comma("title"), 'count': len(songs) - 1})
+                    len(songs) - 1) % {
+                        'title': songs[0].comma("title"),
+                        'count': len(songs) - 1
+                    }
             title = GetPlaylistName(qltk.get_top_parent(parent)).run(title)
-            if title is None: return
-            playlist = Playlist.new(title)
+            if title is None:
+                return
+            playlist = Playlist.new(PLAYLISTS, title)
         playlist.extend(songs)
         Playlists.changed(playlist)
     __add_to_playlist = staticmethod(__add_to_playlist)
 
-class Playlists(gtk.VBox, Browser):
+
+DND_QL, DND_URI_LIST, DND_MOZ_URL = range(3)
+
+
+class Playlists(Gtk.VBox, Browser):
     __gsignals__ = Browser.__gsignals__
-    expand = qltk.RHPaned
 
     name = _("Playlists")
     accelerated_name = _("_Playlists")
     priority = 2
     replaygain_profiles = ["track"]
 
+    def pack(self, songpane):
+        container = qltk.RHPaned()
+        container.pack1(self, True, False)
+        container.pack2(songpane, True, False)
+        return container
+
+    def unpack(self, container, songpane):
+        container.remove(songpane)
+        container.remove(self)
+
+    @classmethod
     def init(klass, library):
         model = klass.__lists.get_model()
         for playlist in os.listdir(util.fsnative(PLAYLISTS)):
             try:
-                playlist = Playlist(Playlist.unquote(playlist), library=library)
+                playlist = Playlist(PLAYLISTS, Playlist.unquote(playlist),
+                                    library=library)
                 model.append(row=[playlist])
             except EnvironmentError:
                 pass
         library.connect('removed', klass.__removed)
         library.connect('added', klass.__added)
         library.connect('changed', klass.__changed)
-    init = classmethod(init)
 
-    def playlists(klass): return [row[0] for row in klass.__lists]
-    playlists = classmethod(playlists)
+    @classmethod
+    def playlists(klass):
+        return [row[0] for row in klass.__lists]
 
+    @classmethod
     def changed(klass, playlist, refresh=True):
         model = klass.__lists
         for row in model:
@@ -307,109 +235,131 @@ class Playlists(gtk.VBox, Browser):
         else:
             model.get_model().append(row=[playlist])
             playlist.write()
-    changed = classmethod(changed)
 
+    @classmethod
     def __removed(klass, library, songs):
         for playlist in klass.playlists():
             if playlist.remove_songs(songs, library):
                 Playlists.changed(playlist)
-    __removed = classmethod(__removed)
 
+    @classmethod
     def __added(klass, library, songs):
         filenames = set([song("~filename") for song in songs])
         for playlist in klass.playlists():
             if playlist.add_songs(filenames, library):
                 Playlists.changed(playlist)
-    __added = classmethod(__added)
 
+    @classmethod
     def __changed(klass, library, songs):
         for playlist in klass.playlists():
             for song in songs:
-                if song in playlist:
+                if song in playlist.songs:
                     Playlists.changed(playlist, refresh=False)
                     break
-    __changed = classmethod(__changed)
 
-    def cell_data(col, render, model, iter):
+    @staticmethod
+    def cell_data(col, render, model, iter, data):
         render.markup = model[iter][0].format()
         render.set_property('markup', render.markup)
-    cell_data = staticmethod(cell_data)
 
     def Menu(self, songs, songlist, library):
         menu = super(Playlists, self).Menu(songs, songlist, library)
         model, rows = songlist.get_selection().get_selected_rows()
         iters = map(model.get_iter, rows)
-        i = qltk.MenuItem(_("_Remove from Playlist"), gtk.STOCK_REMOVE)
+        i = qltk.MenuItem(_("_Remove from Playlist"), Gtk.STOCK_REMOVE)
         i.connect_object('activate', self.__remove, iters, model)
         i.set_sensitive(bool(self.__view.get_selection().get_selected()[1]))
         menu.preseparate()
         menu.prepend(i)
         return menu
 
-    __lists = gtk.TreeModelSort(gtk.ListStore(object))
-    __lists.set_default_sort_func(lambda m, a, b: cmp(m[a][0], m[b][0]))
+    __lists = Gtk.TreeModelSort(model=Gtk.ListStore(object))
+    __lists.set_default_sort_func(lambda m, a, b, data: cmp(m[a][0], m[b][0]))
 
-    def __init__(self, library, player):
+    def __init__(self, library, main):
         super(Playlists, self).__init__(spacing=6)
-        self.__main = bool(player)
+        self.__main = main
         self.__view = view = RCMHintedTreeView()
         self.__view.set_enable_search(True)
         self.__view.set_search_column(0)
         self.__view.set_search_equal_func(
-            lambda model, col, key, iter:
-            not model[iter][col].name.lower().startswith(key.lower()))
-        self.__render = render = gtk.CellRendererText()
-        render.set_property('ellipsize', pango.ELLIPSIZE_END)
+            lambda model, col, key, iter, data:
+            not model[iter][col].name.lower().startswith(key.lower()), None)
+        self.__render = render = Gtk.CellRendererText()
+        render.set_property('ellipsize', Pango.EllipsizeMode.END)
         render.connect('editing-started', self.__start_editing)
         render.connect('edited', self.__edited)
-        col = gtk.TreeViewColumn("Playlists", render)
+        col = Gtk.TreeViewColumn("Playlists", render)
         col.set_cell_data_func(render, Playlists.cell_data)
         view.append_column(col)
         view.set_model(self.__lists)
         view.set_rules_hint(True)
         view.set_headers_visible(False)
-        swin = gtk.ScrolledWindow()
-        swin.set_shadow_type(gtk.SHADOW_IN)
-        swin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        swin = ScrolledWindow()
+        swin.set_shadow_type(Gtk.ShadowType.IN)
+        swin.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         swin.add(view)
-        self.pack_start(swin)
+        self.pack_start(swin, True, True, 0)
 
-        newpl = gtk.Button(stock=gtk.STOCK_NEW)
+        newpl = Gtk.Button(stock=Gtk.STOCK_NEW)
         newpl.connect('clicked', self.__new_playlist)
-        importpl = qltk.Button(_("_Import"), gtk.STOCK_ADD)
+        importpl = qltk.Button(_("_Import"), Gtk.STOCK_ADD)
         importpl.connect('clicked', self.__import, library)
-        hb = gtk.HBox(spacing=6)
+        hb = Gtk.HBox(spacing=6)
         hb.set_homogeneous(True)
-        hb.pack_start(newpl)
-        hb.pack_start(importpl)
-        self.pack_start(hb, expand=False)
+        hb.pack_start(newpl, True, True, 0)
+        hb.pack_start(importpl, True, True, 0)
+        self.pack_start(Alignment(hb, left=3, bottom=3), False, True, 0)
 
         view.connect('popup-menu', self.__popup_menu, library)
 
-        targets = [("text/x-quodlibet-songs", gtk.TARGET_SAME_APP, 0),
-                   ("text/uri-list", 0, 1),
-                   ("text/x-moz-url", 0, 2)]
-        view.drag_dest_set(gtk.DEST_DEFAULT_ALL, targets,
-                           gtk.gdk.ACTION_COPY|gtk.gdk.ACTION_DEFAULT)
-        view.drag_source_set(gtk.gdk.BUTTON1_MASK, targets[:2],
-                             gtk.gdk.ACTION_COPY)
+        targets = [
+            ("text/x-quodlibet-songs", Gtk.TargetFlags.SAME_APP, DND_QL),
+            ("text/uri-list", 0, DND_URI_LIST),
+            ("text/x-moz-url", 0, DND_MOZ_URL)
+        ]
+        targets = [Gtk.TargetEntry.new(*t) for t in targets]
+
+        view.drag_dest_set(Gtk.DestDefaults.ALL, targets,
+                           Gdk.DragAction.COPY)
+        view.drag_source_set(Gdk.ModifierType.BUTTON1_MASK, targets[:2],
+                             Gdk.DragAction.COPY)
         view.connect('drag-data-received', self.__drag_data_received, library)
         view.connect('drag-data-get', self.__drag_data_get)
         view.connect('drag-motion', self.__drag_motion)
         view.connect('drag-leave', self.__drag_leave)
-        if player: view.connect('row-activated', self.__play, player)
-        else: render.set_property('editable', True)
+        if main:
+            view.connect('row-activated', lambda *x: self.emit("activated"))
+        else:
+            render.set_property('editable', True)
         view.get_selection().connect('changed', self.activate)
 
         s = view.get_model().connect('row-changed', self.__check_current)
         self.connect_object('destroy', view.get_model().disconnect, s)
 
-        self.accelerators = gtk.AccelGroup()
-        keyval, mod = gtk.accelerator_parse("F2")
-        self.accelerators.connect_group(keyval, mod, 0, self.__rename)
+        self.accelerators = Gtk.AccelGroup()
+        keyval, mod = Gtk.accelerator_parse("F2")
+        self.accelerators.connect(keyval, mod, 0, self.__rename)
+
+        self.connect('key-press-event', self.__key_pressed)
 
         self.show_all()
 
+    def __key_pressed(self, widget, event):
+        if qltk.is_accel(event, "Delete"):
+            model, iter = self.__view.get_selection().get_selected()
+            if not iter:
+                return False
+
+            playlist = model[iter][0]
+            dialog = ConfirmRemovePlaylistDialog(self, playlist)
+            if dialog.run() == Gtk.ResponseType.YES:
+                playlist.delete()
+                model.get_model().remove(
+                    model.convert_iter_to_child_iter(iter))
+
+        return False
+
     def __rename(self, group, acceleratable, keyval, modifier):
         model, iter = self.__view.get_selection().get_selected()
         if iter:
@@ -418,9 +368,6 @@ class Playlists(gtk.VBox, Browser):
                                    self.__view.get_columns()[0],
                                    start_editing=True)
 
-    def __play(self, view, path, column, player):
-        player.reset()
-
     def __check_current(self, model, path, iter):
         model, citer = self.__view.get_selection().get_selected()
         if citer and model.get_path(citer) == path:
@@ -428,29 +375,25 @@ class Playlists(gtk.VBox, Browser):
             self.activate(resort=not songlist.is_sorted())
 
     def __drag_motion(self, view, ctx, x, y, time):
-        if "text/x-quodlibet-songs" in ctx.targets:
-            try: path = view.get_dest_row_at_pos(x, y)[0]
-            except TypeError:
-                path = (len(view.get_model()) - 1,)
-                pos = gtk.TREE_VIEW_DROP_AFTER
-            else: pos = gtk.TREE_VIEW_DROP_INTO_OR_AFTER
-            if path > (-1,): view.set_drag_dest_row(path, pos)
+        targets = [t.name() for t in ctx.list_targets()]
+        if "text/x-quodlibet-songs" in targets:
+            view.set_drag_dest(x, y, into_only=True)
             return True
         else:
             # Highlighting the view itself doesn't work.
-            view.parent.drag_highlight()
+            view.get_parent().drag_highlight()
             return True
 
     def __drag_leave(self, view, ctx, time):
-        view.parent.drag_unhighlight()
+        view.get_parent().drag_unhighlight()
 
     def __remove(self, iters, smodel):
         model, iter = self.__view.get_selection().get_selected()
         if iter:
             map(smodel.remove, iters)
             playlist = model[iter][0]
-            del(playlist[:])
-            for row in smodel: playlist.append(row[0])
+            playlist.clear()
+            playlist.extend([row[0] for row in smodel])
             Playlists.changed(playlist)
             self.activate()
 
@@ -458,34 +401,39 @@ class Playlists(gtk.VBox, Browser):
         # TreeModelSort doesn't support GtkTreeDragDestDrop.
         view.emit_stop_by_name('drag-data-received')
         model = view.get_model()
-        if tid == 0:
-            filenames = sel.data.split("\x00")
+        if tid == DND_QL:
+            filenames = sel.get_data().split("\x00")
             songs = filter(None, map(library.get, filenames))
-            if not songs: return True
-            try: path, pos = view.get_dest_row_at_pos(x, y)
+            if not songs:
+                Gtk.drag_finish(ctx, False, False, etime)
+                return
+            try:
+                path, pos = view.get_dest_row_at_pos(x, y)
             except TypeError:
-                playlist = Playlist.fromsongs(songs, library)
-                gobject.idle_add(self.__select_playlist, playlist)
+                playlist = Playlist.fromsongs(PLAYLISTS, songs, library)
+                GLib.idle_add(self.__select_playlist, playlist)
             else:
                 playlist = model[path][0]
                 playlist.extend(songs)
             Playlists.changed(playlist)
-            ctx.finish(True, False, etime)
+            Gtk.drag_finish(ctx, True, False, etime)
         else:
-            if tid == 1:
+            if tid == DND_URI_LIST:
                 uri = sel.get_uris()[0]
                 name = os.path.basename(uri)
-            elif tid == 2:
-                uri, name = sel.data.decode('utf16', 'replace').split('\n')
+            elif tid == DND_MOZ_URL:
+                data = sel.get_data()
+                uri, name = data.decode('utf16', 'replace').split('\n')
             else:
-                ctx.finish(False, False, etime)
+                Gtk.drag_finish(ctx, False, False, etime)
                 return
             name = name or os.path.basename(uri) or _("New Playlist")
             uri = uri.encode('utf-8')
             try:
                 sock = urllib.urlopen(uri)
                 f = NamedTemporaryFile()
-                f.write(sock.read()); f.flush()
+                f.write(sock.read())
+                f.flush()
                 if uri.lower().endswith('.pls'):
                     playlist = ParsePLS(f.name, library=library)
                 elif uri.lower().endswith('.m3u'):
@@ -493,11 +441,12 @@ class Playlists(gtk.VBox, Browser):
                 else:
                     raise IOError
                 library.add_filename(playlist)
-                if name: playlist.rename(name)
+                if name:
+                    playlist.rename(name)
                 Playlists.changed(playlist)
-                ctx.finish(True, False, etime)
+                Gtk.drag_finish(ctx, True, False, etime)
             except IOError:
-                ctx.finish(False, False, etime)
+                Gtk.drag_finish(ctx, False, False, etime)
                 qltk.ErrorMessage(
                     qltk.get_top_parent(self),
                     _("Unable to import playlist"),
@@ -511,7 +460,8 @@ class Playlists(gtk.VBox, Browser):
             songs += list(model[iter][0])
         if tid == 0:
             filenames = [song("~filename") for song in songs]
-            sel.set("text/x-quodlibet-songs", 8, "\x00".join(filenames))
+            type_ = Gdk.atom_intern("text/x-quodlibet-songs", True)
+            sel.set(type_, 8, "\x00".join(filenames))
         else:
             sel.set_uris([song("~uri") for song in songs])
 
@@ -523,35 +473,75 @@ class Playlists(gtk.VBox, Browser):
                 view.get_selection().select_iter(row.iter)
 
     def __popup_menu(self, view, library):
-        model, iter = view.get_selection().get_selected()
-        if iter is None:
+        # TODO: Consider allowing plugins to expose themselves in playlist
+        model, itr = view.get_selection().get_selected()
+        if itr is None:
             return
-        songs = list(model[iter][0])
+        songs = list(model[itr][0])
         songs = filter(lambda s: isinstance(s, AudioFile), songs)
         menu = SongsMenu(
             library, songs, playlists=False, remove=False, parent=self)
         menu.preseparate()
 
-        rem = gtk.ImageMenuItem(gtk.STOCK_DELETE)
-        def remove(model, iter):
-            model[iter][0].delete()
-            model.get_model().remove(
-                model.convert_iter_to_child_iter(None, iter))
-        rem.connect_object('activate', remove, model, iter)
+        def _de_duplicate(model, itr):
+            playlist = model[itr][0]
+            unique = set()
+            dupes = list()
+            for s in songs:
+                if s in unique:
+                    dupes.append(s)
+                else:
+                    unique.add(s)
+            if len(dupes) < 1:
+                print_d("No duplicates in this playlist")
+                return
+            dialog = ConfirmRemoveDuplicatesDialog(self, playlist, len(dupes))
+            if dialog.run() == Gtk.ResponseType.YES:
+                playlist.remove_songs(dupes, library, True)
+                Playlists.changed(playlist)
+                self.activate()
+
+        de_dupe = Gtk.MenuItem(_("Remove Duplicates"))
+        de_dupe.connect_object('activate', _de_duplicate, model, itr)
+        de_dupe.set_sensitive(not model[itr][0].has_duplicates())
+        menu.prepend(de_dupe)
+
+        def _shuffle(model, itr):
+            playlist = model[itr][0]
+            playlist.shuffle()
+            self.activate()
+
+        shuffle = Gtk.MenuItem(_("_Shuffle"), use_underline=True)
+        shuffle .connect_object('activate', _shuffle, model, itr)
+        shuffle.set_sensitive(bool(len(model[itr][0])))
+        menu.prepend(shuffle)
+        menu.prepend(SeparatorMenuItem())
+
+        def _remove(model, itr):
+            playlist = model[itr][0]
+            dialog = ConfirmRemovePlaylistDialog(self, playlist)
+            if dialog.run() == Gtk.ResponseType.YES:
+                playlist.delete()
+                model.get_model().remove(
+                    model.convert_iter_to_child_iter(itr))
+
+        rem = Gtk.ImageMenuItem(Gtk.STOCK_DELETE, use_stock=True)
+        rem.connect_object('activate', _remove, model, itr)
         menu.prepend(rem)
 
-        ren = gtk.ImageMenuItem(stock.RENAME)
-        keyval, mod = gtk.accelerator_parse("F2")
-        ren.add_accelerator(
-            'activate', self.accelerators, keyval, mod, gtk.ACCEL_VISIBLE)
-        def rename(path):
+        def _rename(path):
             self.__render.set_property('editable', True)
             view.set_cursor(path, view.get_columns()[0], start_editing=True)
-        ren.connect_object('activate', rename, model.get_path(iter))
+
+        ren = qltk.MenuItem(_("_Rename"), Gtk.STOCK_EDIT)
+        keyval, mod = Gtk.accelerator_parse("F2")
+        ren.add_accelerator(
+            'activate', self.accelerators, keyval, mod, Gtk.AccelFlags.VISIBLE)
+        ren.connect_object('activate', _rename, model.get_path(itr))
         menu.prepend(ren)
 
         menu.show_all()
-        return view.popup_menu(menu, 0, gtk.get_current_event_time())
+        return view.popup_menu(menu, 0, Gtk.get_current_event_time())
 
     def activate(self, widget=None, resort=True):
         model, iter = self.__view.get_selection().get_selected()
@@ -565,7 +555,7 @@ class Playlists(gtk.VBox, Browser):
         config.set("browsers", "playlist", name)
 
     def __new_playlist(self, activator):
-        playlist = Playlist.new()
+        playlist = Playlist.new(PLAYLISTS)
         self.__lists.get_model().append(row=[playlist])
         self.__select_playlist(playlist)
 
@@ -573,11 +563,14 @@ class Playlists(gtk.VBox, Browser):
         editable.set_text(self.__lists[path][0].name)
 
     def __edited(self, render, path, newname):
-        try: self.__lists[path][0].rename(newname)
+        try:
+            self.__lists[path][0].rename(newname)
         except ValueError, s:
             qltk.ErrorMessage(
                 None, _("Unable to rename playlist"), s).run()
-        else: self.__lists[path] = self.__lists[path]
+        else:
+            row = self.__lists[path]
+            self.__lists.row_changed(row.path, row.iter)
         render.set_property('editable', not self.__main)
 
     def __import(self, activator, library):
@@ -602,13 +595,11 @@ class Playlists(gtk.VBox, Browser):
             library.add(playlist)
 
     def restore(self):
-        try: name = config.get("browsers", "playlist")
-        except: pass
-        else:
-            for i, row in enumerate(self.__lists):
-                if row[0].name == name:
-                    self.__view.get_selection().select_path((i,))
-                    break
+        try:
+            name = config.get("browsers", "playlist")
+        except Exception:
+            return
+        self.__view.select_by_func(lambda r: r[0].name == name, one=True)
 
     def reordered(self, songlist):
         songs = songlist.get_songs()
@@ -618,9 +609,9 @@ class Playlists(gtk.VBox, Browser):
             playlist = model[iter][0]
             playlist[:] = songs
         elif songs:
-            playlist = Playlist.fromsongs(songs)
-            gobject.idle_add(self.__select_playlist, playlist)
+            playlist = Playlist.fromsongs(PLAYLISTS, songs)
+            GLib.idle_add(self.__select_playlist, playlist)
         if playlist:
-            Playlists.changed(playlist)
+            Playlists.changed(playlist, refresh=False)
 
 browsers = [Playlists]
