@@ -1,4 +1,5 @@
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman
+#           2012 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -7,45 +8,58 @@
 import os
 import sys
 
-from glob import glob
-from os.path import dirname, basename, join, splitext
-from quodlibet import util, const
-
-base = dirname(__file__)
-self = basename(base)
-parent = basename(dirname(base))
-if os.name == 'nt':
-    # Windows needs to load .pyc files
-    glob_pattern = "[!_]*.py*"
-else: glob_pattern = "[!_]*.py"
-modules = [splitext(f)[0] for f in glob(join(base, glob_pattern))]
-modules = ["%s.%s.%s" % (parent, self, basename(m)) for m in set(modules)]
+from quodlibet.util.modulescanner import load_dir_modules
+from quodlibet import util
+from quodlibet import const
+from quodlibet.util.dprint import print_w
+from quodlibet.const import MinVersions
 
+mimes = set()
 _infos = {}
-for i, name in enumerate(modules):
-    try: format = __import__(name, {}, {}, self)
-    except Exception, err:
-        util.print_exc()
-        continue
-    format = __import__(name, {}, {}, self)
-    for ext in format.extensions:
-        _infos[ext] = format.info
-    # Migrate pre-0.16 library, which was using an undocumented "feature".
-    sys.modules[name.replace(".", "/")] = format
-    if name and name.startswith("quodlibet."):
-        sys.modules[name.split(".", 1)[1]] = sys.modules[name]
-    modules[i] = (format.extensions and name.split(".")[-1])
+modules = []
+names = []
 
-if not _infos:
-    raise SystemExit("No formats found!")
 
-try: sys.modules["formats.flac"] = sys.modules["formats.xiph"]
-except KeyError: pass
-try: sys.modules["formats.oggvorbis"] = sys.modules["formats.xiph"]
-except KeyError: pass
+def init():
+    global mimes, _infos, modules, names
+
+    import mutagen
+    if mutagen.version < MinVersions.MUTAGEN:
+        print_w("Mutagen %s required. %s found."
+                % (MinVersions.MUTAGEN, mutagen.version_string))
+
+    base = os.path.dirname(__file__)
+    load_pyc = os.name == 'nt'
+    formats = load_dir_modules(base,
+                               package=__package__,
+                               load_compiled=load_pyc)
+
+    for format in formats:
+        name = format.__name__
+
+        for ext in format.extensions:
+            _infos[ext] = format.info
+
+        if format.extensions:
+            for type_ in format.types:
+                mimes.update(type_.mimes)
+                names.append(type_.format)
+            modules.append(name.split(".")[-1])
+
+        # Migrate pre-0.16 library, which was using an undocumented "feature".
+        sys.modules[name.replace(".", "/")] = format
+        # Migrate old layout
+        if name.startswith("quodlibet."):
+            sys.modules[name.split(".", 1)[1]] = format
+
+    modules.sort()
+    names.sort()
+
+    if not _infos:
+        raise SystemExit("No formats found!")
+
+init()
 
-modules = filter(None, modules)
-modules.sort()
 
 def MusicFile(filename):
     for ext in _infos.keys():
@@ -61,11 +75,13 @@ def MusicFile(filename):
                 return _infos[ext](filename)
             except:
                 print_w(_("Error loading %r") % filename)
-                util.print_exc()
+                if const.DEBUG:
+                    util.print_exc()
                 lt, lv, tb = sys.exc_info()
                 sys.last_type, sys.last_value, sys.last_traceback = lt, lv, tb
                 return None
-    else: return None
+    else:
+        return None
 
 if sys.version_info < (2, 5):
     def supported(song):
@@ -76,13 +92,16 @@ if sys.version_info < (2, 5):
         return False
 else:
     extensions = tuple(_infos.keys())
+
     def supported(song):
         return song.key.lower().endswith(extensions)
 
+
 def filter(filename):
     lower = filename.lower()
     for ext in _infos.keys():
-        if lower.endswith(ext): return True
+        if lower.endswith(ext):
+            return True
     return False
 
 from quodlibet.formats._audio import USEFUL_TAGS, MACHINE_TAGS, PEOPLE
