@@ -5,6 +5,7 @@
 # published by the Free Software Foundation
 
 import os
+import ctypes
 
 from quodlibet.formats._audio import AudioFile
 
@@ -12,33 +13,44 @@ extensions = [
     '.669', '.amf', '.ams', '.dsm', '.far', '.it', '.med', '.mod', '.mt2',
     '.mtm', '.okt', '.s3m', '.stm', '.ult', '.gdm', '.xm']
 
-try:
-    import ctypes
-except ImportError:
-    extensions = []
-else:
-    for so_version in ("libmodplug.so.1", "libmodplug.so.0"):
-        try:
-            _modplug = ctypes.cdll.LoadLibrary(so_version)
-            _modplug.ModPlug_GetName.restype = ctypes.c_char_p
-        except OSError: continue
-        else: break
+
+for so_version in ("libmodplug.so.1", "libmodplug.so.0"):
+    try:
+        _modplug = ctypes.cdll.LoadLibrary(so_version)
+    except OSError:
+        continue
     else:
-        extensions = []
+        _modplug.ModPlug_GetName.argtypes = [ctypes.c_void_p]
+        _modplug.ModPlug_GetName.restype = ctypes.c_char_p
+
+        _modplug.ModPlug_Load.argtypes = [ctypes.c_void_p, ctypes.c_int]
+        _modplug.ModPlug_Load.restype = ctypes.c_void_p
+
+        _modplug.ModPlug_GetLength.argtypes = [ctypes.c_void_p]
+        _modplug.ModPlug_GetLength.restype = ctypes.c_int
+
+        _modplug.ModPlug_Unload.argtypes = [ctypes.c_void_p]
+        _modplug.ModPlug_Unload.restype = None
+        break
+else:
+    extensions = []
+
 
 class ModFile(AudioFile):
 
     format = "MOD/XM/IT"
 
     def __init__(self, filename):
-        size = os.path.getsize(filename)
         data = file(filename).read()
         f = _modplug.ModPlug_Load(data, len(data))
-        if not f: raise IOError("%r not a valid MOD file" % filename)
+        if not f:
+            raise IOError("%r not a valid MOD file" % filename)
         self["~#length"] = _modplug.ModPlug_GetLength(f) // 1000
         title = _modplug.ModPlug_GetName(f) or os.path.basename(filename)
-        try: self["title"] = title.decode('utf-8')
-        except UnicodeError: self["title"] = title.decode("iso-8859-1")
+        try:
+            self["title"] = title.decode('utf-8')
+        except UnicodeError:
+            self["title"] = title.decode("iso-8859-1")
         _modplug.ModPlug_Unload(f)
         self.sanitize(filename)
 
@@ -48,11 +60,14 @@ class ModFile(AudioFile):
     def reload(self, *args):
         artist = self.get("artist")
         super(ModFile, self).reload(*args)
-        if artist is not None: self.setdefault("artist", artist)
+        if artist is not None:
+            self.setdefault("artist", artist)
 
     def can_change(self, k=None):
-        if k is None: return ["artist"]
-        else: return k == "artist"
+        if k is None:
+            return ["artist"]
+        else:
+            return k == "artist"
 
 info = ModFile
-
+types = [ModFile]
