@@ -5,21 +5,21 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import tempfile
-
 from quodlibet import util
 from quodlibet.formats._audio import AudioFile
 
-extensions = ['.mp4', '.m4a']
+extensions = ['.mp4', '.m4a', '.m4v']
 
 try:
     from mutagen.mp4 import MP4
 except ImportError:
     extensions = []
 
+
 class MP4File(AudioFile):
     multiple_values = False
     format = "MPEG-4 AAC"
+    mimes = ["audio/mp4", "audio/x-m4a", "audio/mpeg4", "audio/aac"]
 
     __translate = {
         "\xa9nam": "title",
@@ -89,12 +89,16 @@ class MP4File(AudioFile):
     def write(self):
         audio = MP4(self["~filename"])
         for key in self.__translate.keys() + self.__tupletranslate.keys():
-            try: del(audio[key])
-            except KeyError: pass
+            try:
+                del(audio[key])
+            except KeyError:
+                pass
 
         for key in self.realkeys():
-            try: name = self.__rtranslate[key]
-            except KeyError: continue
+            try:
+                name = self.__rtranslate[key]
+            except KeyError:
+                continue
             values = self.list(key)
             if name == "tmpo":
                 values = map(int, values)
@@ -112,22 +116,21 @@ class MP4File(AudioFile):
 
     def can_change(self, key=None):
         OK = self.__rtranslate.keys() + self.__rtupletranslate.keys()
-        if key is None: return OK
-        else: return super(MP4File, self).can_change(key) and (key in OK)
+        if key is None:
+            return OK
+        else:
+            return super(MP4File, self).can_change(key) and (key in OK)
 
     def get_format_cover(self):
         try:
             tag = MP4(self["~filename"])
-        except (OSError, IOError):
-            return None
+        except Exception:
+            return
         else:
             for cover in tag.get("covr", []):
-                fn = tempfile.NamedTemporaryFile()
-                fn.write(cover)
-                fn.flush()
-                fn.seek(0, 0)
-                return fn
-            else:
-                return None
+                fn = util.get_temp_cover_file(cover)
+                if fn:
+                    return fn
 
 info = MP4File
+types = [MP4File]
