@@ -12,8 +12,10 @@ try:
 except (ImportError, OSError):
     extensions = []
 
+
 class MPCFile(APEv2File):
     format = "Musepack"
+    mimes = ["audio/x-musepack", "audio/x-mpc"]
 
     def __init__(self, filename):
         audio = Musepack(filename)
@@ -40,3 +42,4 @@ class MPCFile(APEv2File):
         self.sanitize(filename)
 
 info = MPCFile
+types = [MPCFile]
