@@ -12,9 +12,11 @@ try:
 except ImportError:
     extensions = []
 
+
 class WavpackFile(APEv2File):
     format = "WavPack"
-    
+    mimes = ["audio/x-wavpack"]
+
     def __init__(self, filename):
         audio = WavPack(filename)
         super(WavpackFile, self).__init__(filename, audio)
@@ -22,3 +24,4 @@ class WavpackFile(APEv2File):
         self.sanitize(filename)
 
 info = WavpackFile
+types = [WavpackFile]
