@@ -1,4 +1,5 @@
-# Copyright 2004-2010 Joe Wreschnig, Michael Urman, Christoph Reiter
+# Copyright 2004-2010 Joe Wreschnig, Michael Urman
+# Copyright 2010,2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -17,13 +18,22 @@ from quodlibet.parse._scanner import Scanner
 # Token types.
 (OPEN, CLOSE, TEXT, COND, EOF) = range(5)
 
-class error(ValueError): pass
-class ParseError(error): pass
-class LexerError(error): pass
+
+class error(ValueError):
+    pass
+
+
+class ParseError(error):
+    pass
+
+
+class LexerError(error):
+    pass
+
 
 class PatternLexeme(object):
-    _reverse = { OPEN: "OPEN", CLOSE: "CLOSE", TEXT: "TEXT", COND: "COND",
-                 EOF: "EOF" }
+    _reverse = {OPEN: "OPEN", CLOSE: "CLOSE", TEXT: "TEXT", COND: "COND",
+                EOF: "EOF"}
 
     def __init__(self, typ, lexeme):
         self.type = typ
@@ -35,6 +45,7 @@ class PatternLexeme(object):
                 str(self._reverse[self.type]) +
                 "), lexeme=" + repr(self.lexeme) + ">")
 
+
 class PatternLexer(Scanner):
     def __init__(self, s):
         self.string = s.strip()
@@ -45,14 +56,18 @@ class PatternLexer(Scanner):
 
     def text(self, scanner, string):
         return PatternLexeme(TEXT, re.sub(r"\\(.)", r"\1", string))
+
     def table(self, scanner, string):
         return PatternLexeme(
             {"|": COND, "<": OPEN, ">": CLOSE}[string], string)
 
     def __iter__(self):
         s = self.scan(self.string)
-        if s[1] != "": raise LexerError("characters left over in string")
-        else: return iter(s[0] + [PatternLexeme(EOF, "")])
+        if s[1] != "":
+            raise LexerError("characters left over in string")
+        else:
+            return iter(s[0] + [PatternLexeme(EOF, "")])
+
 
 class PatternNode(object):
     def __init__(self):
@@ -61,6 +76,7 @@ class PatternNode(object):
     def __repr__(self):
         return "Pattern(%s)" % (", ".join(map(repr, self.children)))
 
+
 class TextNode(object):
     def __init__(self, text):
         self.text = text
@@ -68,6 +84,7 @@ class TextNode(object):
     def __repr__(self):
         return "Text(\"%s\")" % self.text
 
+
 class ConditionNode(object):
     def __init__(self, tag, ifcase, elsecase):
         self.tag = tag
@@ -78,6 +95,7 @@ class ConditionNode(object):
         t, i, e = self.tag, repr(self.ifcase), repr(self.elsecase)
         return "Condition(tag: \"%s\", if: %s, else: %s)" % (t, i, e)
 
+
 class TagNode(object):
     def __init__(self, tag):
         self.tag = tag
@@ -85,6 +103,7 @@ class TagNode(object):
     def __repr__(self):
         return "Tag(\"%s\")" % self.tag
 
+
 class PatternParser(object):
     def __init__(self, tokens):
         self.tokens = iter(tokens)
@@ -108,7 +127,8 @@ class PatternParser(object):
         # fix bad tied tags
         if tag[:1] != "~" and "~" in tag:
                 tag = "~" + tag
-        try: self.match(TEXT)
+        try:
+            self.match(TEXT)
         except ParseError:
             while self.lookahead.type not in [CLOSE, EOF]:
                 self.match(self.lookahead.type)
@@ -119,17 +139,20 @@ class PatternParser(object):
             if self.lookahead.type == COND:
                 self.match(COND)
                 elsecase = self.Pattern()
-            else: elsecase = None
+            else:
+                elsecase = None
             nodes.append(ConditionNode(tag, ifcase, elsecase))
 
-            try: self.match(CLOSE)
+            try:
+                self.match(CLOSE)
             except ParseError:
                 nodes.pop(-1)
                 while self.lookahead.type not in [EOF, OPEN]:
                     self.match(self.lookahead.type)
         else:
             nodes.append(TagNode(tag))
-            try: self.match(CLOSE)
+            try:
+                self.match(CLOSE)
             except ParseError:
                 nodes.pop(-1)
                 while self.lookahead.type not in [EOF, OPEN]:
@@ -144,29 +167,33 @@ class PatternParser(object):
             if self.lookahead.type in tokens:
                 self.lookahead = self.tokens.next()
             else:
-                raise ParseError("The token '%s' is not the type exected." %(
+                raise ParseError("The token '%s' is not the type exected." % (
                     self.lookahead.lexeme))
         except StopIteration:
             self.lookahead = PatternLexeme(EOF, "")
 
+
 class PatternFormatter(object):
-    _formatters = []
+    _format = None
     _post = None
 
     def __init__(self, func, list_func, tags):
         self.__func = func
         self.__list_func = list_func
         self.tags = set(tags)
-        self.format(self.Dummy()) # Validate string
+        self.format(self.Dummy())  # Validate string
 
     class Dummy(dict):
-        def comma(self, *args): return u"_"
-        def list_separate(self, *args): return [u""]
+        def comma(self, *args):
+            return u"_"
+
+        def list_separate(self, *args):
+            return [u""]
 
     class SongProxy(object):
-        def __init__(self, realsong, formatters):
+        def __init__(self, realsong, formatter):
             self.__song = realsong
-            self.__formatters = formatters
+            self.__formatter = formatter
 
         def comma(self, key):
             value = self.__song.comma(key)
@@ -176,8 +203,8 @@ class PatternFormatter(object):
                 if isinstance(value, float):
                     value = "%.2f" % value
                 value = unicode(value)
-            for f in self.__formatters:
-                value = f(key, value)
+            if self.__formatter:
+                return self.__formatter(key, value)
             return value
 
         def list_separate(self, key):
@@ -186,13 +213,15 @@ class PatternFormatter(object):
                 if isinstance(value, float):
                     value = "%.2f" % value
                 values = [unicode(value)]
-            else: values = self.__song.list_separate(key)
-            for f in self.__formatters:
-                values = [f(key, v) for v in values]
+            else:
+                values = self.__song.list_separate(key)
+
+            if self.__formatter:
+                return [self.__formatter(key, v) for v in values]
             return values
 
     def format(self, song):
-        value = "".join(self.__func(self.SongProxy(song, self._formatters)))
+        value = "".join(self.__func(self.SongProxy(song, self._format)))
         if self._post:
             return self._post(value, song)
         return value
@@ -201,7 +230,7 @@ class PatternFormatter(object):
         """Returns a set of formatted patterns with all tag combinations:
         <performer>-bla returns [performer1-bla, performer2-bla]"""
         vals = [""]
-        for val in self.__list_func(self.SongProxy(song, self._formatters)):
+        for val in self.__list_func(self.SongProxy(song, self._format)):
             if type(val) == list:
                 vals = [r + part for part in val for r in vals]
             else:
@@ -212,6 +241,7 @@ class PatternFormatter(object):
 
     __mod__ = format
 
+
 class PatternCompiler(object):
     def __init__(self, root):
         self.__root = root.node
@@ -226,8 +256,10 @@ class PatternCompiler(object):
         content.extend(self.__pattern(self.__root, {}, tags))
         content.append("  return r")
         code = "\n".join(content)
-        exec compile(code, "<string>", "exec")
-        return f, tags
+
+        scope = {}
+        exec compile(code, "<string>", "exec") in scope
+        return scope["f"], tags
 
     def __escape(self, text):
         text = text.replace("\\", r"\\")
@@ -273,6 +305,7 @@ class PatternCompiler(object):
                 text.extend(self.__tag(child, scope, tags))
         return map("  ".__add__, text)
 
+
 def Pattern(string, Kind=PatternFormatter, MAX_CACHE_SIZE=100, cache={}):
     if (Kind, string) not in cache:
         if len(cache) > MAX_CACHE_SIZE:
@@ -283,42 +316,67 @@ def Pattern(string, Kind=PatternFormatter, MAX_CACHE_SIZE=100, cache={}):
         cache[(Kind, string)] = Kind(func, list_func, tags)
     return cache[(Kind, string)]
 
+
 def _number(key, value):
     if key == "tracknumber":
         parts = value.split("/")
-        try: decimals = len(str(int(parts[1])))
-        except (IndexError, ValueError): decimals = 2
-        try: return "%0*d" % (max(2, decimals), int(parts[0]))
-        except (TypeError, ValueError): return value
+        try:
+            decimals = len(str(int(parts[1])))
+        except (IndexError, ValueError):
+            decimals = 2
+        try:
+            return "%0*d" % (max(2, decimals), int(parts[0]))
+        except (TypeError, ValueError):
+            return value
     elif key == "discnumber":
         parts = value.split("/")
-        try: return "%02d" % int(parts[0])
-        except (TypeError, ValueError): return value
-    else: return value
+        try:
+            return "%02d" % int(parts[0])
+        except (TypeError, ValueError):
+            return value
+    else:
+        return value
+
 
 class _FileFromPattern(PatternFormatter):
-    _formatters = [_number,
-                   (lambda k, s: s.replace(os.sep, "_")),
-                   (lambda k, s: s.replace(u"\uff0f", "_")),
-                   (lambda k, s: s.strip()),
-                   (lambda k, s: (len(s) > 100 and s[:100] + "...") or s),
-                   ]
+
+    def _format(self, key, value):
+        value = _number(key, value)
+        value = value.replace(os.sep, "_")
+        value = value.replace(u"\uff0f", "_")
+        value = value.strip()
+        return value
 
     def _post(self, value, song):
         if value:
             fn = song.get("~filename", ".")
             ext = fn[fn.rfind("."):].lower()
             val_ext = value[-len(ext):].lower()
-            if not ext == val_ext: value += ext.lower()
+            if not ext == val_ext:
+                value += ext.lower()
             if os.name == "nt":
                 value = util.strip_win32_incompat_from_path(value)
+
             value = util.expanduser(value)
+
+            # Limit each path section to 255 (bytes on linux, chars on win).
+            # http://en.wikipedia.org/wiki/Comparison_of_file_systems#Limits
+            path, ext = os.path.splitext(value)
+            path = map(util.fsnative, path.split(os.sep))
+            limit = [255] * len(path)
+            limit[-1] -= len(util.fsnative(ext))
+            elip = lambda (p, l): (len(p) > l and p[:l - 2] + "..") or p
+            path = os.sep.join(map(elip, zip(path, limit)))
+            value = util.fsdecode(path) + ext
+
             if os.sep in value and not os.path.isabs(value):
                 raise ValueError("Pattern is not rooted")
         return value
 
+
 class _XMLFromPattern(PatternFormatter):
-    _formatters = [lambda k, s: util.escape(s)]
+    def _format(self, key, value):
+        return util.escape(value)
 
 
 def FileFromPattern(string):
@@ -329,5 +387,6 @@ def FileFromPattern(string):
         string = string.replace("\\", r"\\")
     return Pattern(string, _FileFromPattern)
 
+
 def XMLFromPattern(string):
     return Pattern(string, _XMLFromPattern)
