@@ -1,29 +1,27 @@
 # Copyright 2004-2011 Joe Wreschnig, Michael Urman, Steven Robertson,
-#                     Christoph Reiter
+#           2011-2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import gtk
-import gobject
+import gi
+try:
+    gi.require_version("Gst", "1.0")
+    gi.require_version("GstPbutils", "1.0")
+except ValueError, e:
+    raise ImportError(e)
+
+from gi.repository import Gtk, Gst, GLib, GstPbutils
+
 import os
 import threading
 
-import pygst
-pygst.require("0.10")
-
-import gst
-try:
-    from gst import pbutils
-except ImportError:
-    pbutils = None
-
 from quodlibet import config
 from quodlibet import const
 
 from quodlibet.util import fver, sanitize_tags
-from quodlibet.player import error as PlayerError
+from quodlibet.player import PlayerError
 from quodlibet.player._base import BasePlayer
 from quodlibet.player._gstutils import *
 from quodlibet.qltk.msg import ErrorMessage
@@ -31,19 +29,11 @@ from quodlibet.qltk.notif import Task
 from quodlibet.qltk.entry import UndoEntry
 from quodlibet.qltk.x import Button
 
-USE_PLAYBIN2 = gst.version() >= (0, 10, 24)
 
-if 'QUODLIBET_PLAYBIN1' in os.environ:
-    print_d("QUODLIBET_PLAYBIN1")
-    USE_PLAYBIN2 = False
+STATE_CHANGE_TIMEOUT = Gst.SECOND * 4
 
-USE_QUEUE = 'QUODLIBET_GSTBE_QUEUE' in os.environ
-if USE_QUEUE:
-    print_d("QUODLIBET_GSTBE_QUEUE")
 
-USE_TRACK_CHANGE = gst.version() >= (0, 10, 28)
-
-class GStreamerPlayer(BasePlayer):
+class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
     __gproperties__ = BasePlayer._gproperties_
     __gsignals__ = BasePlayer._gsignals_
 
@@ -70,51 +60,63 @@ class GStreamerPlayer(BasePlayer):
                 "playback, such as 'alsasink device=default'. "
                 "Leave blank for default pipeline."))
         e.set_text(config.get('player', 'gst_pipeline'))
+
         def changed(entry):
             config.set('player', 'gst_pipeline', entry.get_text())
         e.connect('changed', changed)
 
-        l = gtk.Label(_('_Output pipeline:'))
-        l.set_use_underline(True)
-        l.set_mnemonic_widget(e)
+        pipe_label = Gtk.Label(label=_('_Output pipeline:'))
+        pipe_label.set_use_underline(True)
+        pipe_label.set_mnemonic_widget(e)
 
-        b = gtk.Button(stock=gtk.STOCK_APPLY)
-        b.connect_object('clicked', lambda x: x.__rebuild_pipeline(), self)
-
-        hb = gtk.HBox(spacing=6)
-        hb.pack_start(l, expand=False)
-        hb.pack_start(e)
-        hb.pack_start(b, expand=False)
+        apply_button = Gtk.Button(stock=Gtk.STOCK_APPLY)
 
         def format_buffer(scale, value):
-            # seconds
-            return _("%.1f s") % value
+            return _("%.1f seconds") % value
 
         def scale_changed(scale):
             config.set("player", "gst_buffer", scale.get_value())
             if self.bin:
-                duration = int(scale.get_value() * 1000) * gst.MSECOND
+                duration = int(scale.get_value() * 1000) * Gst.MSECOND
                 self.bin.set_property('buffer-duration', duration)
 
         duration = config.getfloat("player", "gst_buffer")
-        scale = gtk.HScale(gtk.Adjustment(duration, 0.2, 10))
-        scale.set_value_pos(gtk.POS_RIGHT)
+        scale = Gtk.HScale.new(Gtk.Adjustment(duration, 0.2, 10))
+        scale.set_value_pos(Gtk.PositionType.RIGHT)
         scale.set_show_fill_level(True)
         scale.connect('format-value', format_buffer)
         scale.connect('value-changed', scale_changed)
 
-        l = gtk.Label(_('_Buffer duration:'))
-        l.set_use_underline(True)
-        l.set_mnemonic_widget(scale)
+        buffer_label = Gtk.Label(label=_('_Buffer duration:'))
+        buffer_label.set_use_underline(True)
+        buffer_label.set_mnemonic_widget(scale)
 
-        hb2 = gtk.HBox(spacing=6)
-        hb2.pack_start(l, expand=False)
-        hb2.pack_start(scale)
+        def rebuild_pipeline(*args):
+            self._rebuild_pipeline()
+        apply_button.connect('clicked', rebuild_pipeline)
 
-        vbox = gtk.VBox(spacing=6)
-        vbox.pack_start(hb)
-        if USE_PLAYBIN2:
-            vbox.pack_start(hb2)
+        widgets = [(pipe_label, e, apply_button),
+                   (buffer_label, scale, None),
+        ]
+
+        table = Gtk.Table(len(widgets), 3)
+        table.set_col_spacings(6)
+        table.set_row_spacings(6)
+        for i, (left, middle, right) in enumerate(widgets):
+            left.set_alignment(0.0, 0.5)
+            table.attach(left, 0, 1, i, i + 1,
+                         xoptions=Gtk.AttachOptions.FILL |
+                         Gtk.AttachOptions.SHRINK)
+            if right:
+                table.attach(middle, 1, 2, i, i + 1)
+                table.attach(right, 2, 3, i, i + 1,
+                             xoptions=Gtk.AttachOptions.FILL |
+                             Gtk.AttachOptions.SHRINK)
+            else:
+                table.attach(middle, 1, 3, i, i + 1)
+
+        vbox = Gtk.VBox(spacing=6)
+        vbox.pack_start(table, True, True, 0)
 
         if const.DEBUG:
             def print_bin(player):
@@ -123,18 +125,28 @@ class GStreamerPlayer(BasePlayer):
                 else:
                     print_e("No active pipeline.")
 
-            b = Button("Print Pipeline", gtk.STOCK_DIALOG_INFO)
+            b = Button("Print Pipeline", Gtk.STOCK_DIALOG_INFO)
             b.connect_object('clicked', print_bin, self)
-            vbox.pack_start(b)
+            vbox.pack_start(b, True, True, 0)
 
         return vbox
 
     def __init__(self, librarian=None):
+        GStreamerPluginHandler.__init__(self)
         super(GStreamerPlayer, self).__init__()
-        self.version_info = "GStreamer: %s / PyGSt: %s" % (
-            fver(gst.version()), fver(gst.pygst_version))
+        self.version_info = "GStreamer: %s" % fver(Gst.version())
         self._librarian = librarian
-        self.connect('destroy', lambda s: self.__destroy_pipeline())
+        self._pipeline_desc = None
+
+    def destroy(self):
+        self.__destroy_pipeline()
+
+    @property
+    def name(self):
+        name = "GStreamer"
+        if self._pipeline_desc:
+            name += " (%s)" % self._pipeline_desc
+        return name
 
     def __init_pipeline(self):
         """Creates a gstreamer pipeline with the following elements
@@ -149,14 +161,15 @@ class GStreamerPlayer(BasePlayer):
                 -> audioconvert -> user defined elements
                 -> gconf/autoaudiosink ]
         """
-        if self.bin: return True
+        if self.bin:
+            return True
 
         pipeline = config.get("player", "gst_pipeline")
-        pipeline, self.name = GStreamerSink(pipeline)
+        pipeline, self._pipeline_desc = GStreamerSink(pipeline)
         if not pipeline:
             return False
 
-        if self._use_eq and gst.element_factory_find('equalizer-10bands'):
+        if self._use_eq and Gst.ElementFactory.find('equalizer-10bands'):
             # The equalizer only operates on 16-bit ints or floats, and
             # will only pass these types through even when inactive.
             # We push floats through to this point, then let the second
@@ -164,79 +177,72 @@ class GStreamerPlayer(BasePlayer):
             # pipeline supports. As a bonus, this seems to automatically
             # select the highest-precision format supported by the
             # rest of the chain.
-            filt = gst.element_factory_make('capsfilter')
+            filt = Gst.ElementFactory.make('capsfilter', None)
             filt.set_property('caps',
-                              gst.caps_from_string('audio/x-raw-float'))
-            eq = gst.element_factory_make('equalizer-10bands')
+                              Gst.Caps.from_string('audio/x-raw,format=F32LE'))
+            eq = Gst.ElementFactory.make('equalizer-10bands', None)
             self._eq_element = eq
             self.update_eq_values()
-            conv = gst.element_factory_make('audioconvert')
-            pipeline = [filt, eq, conv] + pipeline
+            conv = Gst.ElementFactory.make('audioconvert', None)
+            resample = Gst.ElementFactory.make('audioresample', None)
+            pipeline = [filt, eq, conv, resample] + pipeline
 
-        if USE_PLAYBIN2:
-            prefix = []
+        # playbin2 has started to control the volume through pulseaudio,
+        # which means the volume property can change without us noticing.
+        # Use our own volume element for now until this works with PA.
+        self._vol_element = Gst.ElementFactory.make('volume', None)
+        pipeline.insert(0, self._vol_element)
 
-            if USE_QUEUE:
-                queue = gst.element_factory_make('queue')
-                queue.set_property('max-size-time', 500 * gst.MSECOND)
-                prefix.append(queue)
+        # Get all plugin elements and append audio converters.
+        # playbin already includes one at the end
+        plugin_pipeline = []
+        for plugin in self._get_plugin_elements():
+            plugin_pipeline.append(plugin)
+            plugin_pipeline.append(
+                Gst.ElementFactory.make('audioconvert', None))
+            plugin_pipeline.append(
+                Gst.ElementFactory.make('audioresample', None))
+        pipeline = plugin_pipeline + pipeline
 
-            # playbin2 has started to control the volume through pulseaudio,
-            # which means the volume property can change without us noticing.
-            # Use our own volume element for now until this works with PA.
-            # Also, when using the queue, this removes the delay..
-            self._vol_element = gst.element_factory_make('volume')
-            prefix.append(self._vol_element)
-
-            pipeline = prefix + pipeline
-
-        bufbin = gst.Bin()
+        bufbin = Gst.Bin()
         map(bufbin.add, pipeline)
         if len(pipeline) > 1:
-            try:
-                gst.element_link_many(*pipeline)
-            except gst.LinkError, e:
-                print_w(
-                    _("Could not link GStreamer pipeline: '%s'") % e)
+            if not link_many(pipeline):
+                print_w("Could not link GStreamer pipeline")
                 self.__destroy_pipeline()
                 return False
 
         # Test to ensure output pipeline can preroll
-        bufbin.set_state(gst.STATE_READY)
-        result, state, pending = bufbin.get_state(timeout=gst.SECOND/2)
-        if result == gst.STATE_CHANGE_FAILURE:
-            bufbin.set_state(gst.STATE_NULL)
+        bufbin.set_state(Gst.State.READY)
+        result, state, pending = bufbin.get_state(timeout=STATE_CHANGE_TIMEOUT)
+        if result == Gst.StateChangeReturn.FAILURE:
+            bufbin.set_state(Gst.State.NULL)
             self.__destroy_pipeline()
             return False
 
         # Make the sink of the first element the sink of the bin
-        gpad = gst.GhostPad('sink', pipeline[0].get_pad('sink'))
+        gpad = Gst.GhostPad.new('sink', pipeline[0].get_static_pad('sink'))
         bufbin.add_pad(gpad)
 
-        if USE_PLAYBIN2:
-            self.bin = gst.element_factory_make('playbin2')
-            self.__atf_id = self.bin.connect('about-to-finish',
-                self.__about_to_finish)
-            duration = config.getfloat("player", "gst_buffer")
-            duration = int(duration * 1000) * gst.MSECOND
-            self.bin.set_property('buffer-duration', duration)
-        else:
-            self.bin = gst.element_factory_make('playbin')
-            self._vol_element = self.bin
+        self.bin = Gst.ElementFactory.make('playbin', None)
+        self.__atf_id = self.bin.connect('about-to-finish',
+            self.__about_to_finish)
+        duration = config.getfloat("player", "gst_buffer")
+        duration = int(duration * 1000) * Gst.MSECOND
+        self.bin.set_property('buffer-duration', duration)
 
         self.bin.set_property('audio-sink', bufbin)
 
         # by default playbin will render video -> suppress using fakesink
-        fakesink = gst.element_factory_make('fakesink')
+        fakesink = Gst.ElementFactory.make('fakesink', None)
         self.bin.set_property('video-sink', fakesink)
 
-        # disable all video/text decoding in playbin2
-        if USE_PLAYBIN2:
-            GST_PLAY_FLAG_VIDEO = 1 << 0
-            GST_PLAY_FLAG_TEXT = 1 << 2
-            flags = self.bin.get_property("flags")
-            flags &= ~(GST_PLAY_FLAG_VIDEO | GST_PLAY_FLAG_TEXT)
-            self.bin.set_property("flags", flags)
+        # disable all video/text decoding in playbin
+        GST_PLAY_FLAG_VIDEO = 1 << 0
+        GST_PLAY_FLAG_TEXT = 1 << 2
+        flags = self.bin.get_property("flags")
+        flags &= ~(GST_PLAY_FLAG_VIDEO | GST_PLAY_FLAG_TEXT)
+        self.bin.set_property("flags", flags)
 
         # ReplayGain information gets lost when destroying
         self.volume = self.volume
@@ -245,9 +251,14 @@ class GStreamerPlayer(BasePlayer):
         bus.add_signal_watch()
         self.__bus_id = bus.connect('message', self.__message, self._librarian)
 
+        if self.song:
+            self.bin.set_property('uri', self.song("~uri"))
+
         return True
 
     def __destroy_pipeline(self):
+        self._remove_plugin_elements()
+
         if self.__bus_id:
             bus = self.bin.get_bus()
             bus.disconnect(self.__bus_id)
@@ -259,8 +270,8 @@ class GStreamerPlayer(BasePlayer):
             self.__atf_id = False
 
         if self.bin:
-            self.bin.set_state(gst.STATE_NULL)
-            self.bin.get_state(timeout=gst.SECOND/2)
+            self.bin.set_state(Gst.State.NULL)
+            self.bin.get_state(timeout=STATE_CHANGE_TIMEOUT)
             self.bin = None
 
         if self._task:
@@ -274,7 +285,7 @@ class GStreamerPlayer(BasePlayer):
         self._vol_element = None
         self._eq_element = None
 
-    def __rebuild_pipeline(self):
+    def _rebuild_pipeline(self):
         """If a pipeline is active, rebuild it and restore vol, position etc"""
 
         if not self.bin:
@@ -285,46 +296,55 @@ class GStreamerPlayer(BasePlayer):
 
         self.__destroy_pipeline()
         self.paused = True
-        if self.__init_pipeline():
-            self.bin.set_property('uri', self.song("~uri"))
+        self.__init_pipeline()
         self.paused = paused
         self.seek(pos)
 
     def __message(self, bus, message, librarian):
-        if message.type == gst.MESSAGE_EOS:
+        if message.type == Gst.MessageType.EOS:
             print_d("Stream EOS")
             if not self._in_gapless_transition:
                 self._source.next_ended()
             self._end(False)
-        elif message.type == gst.MESSAGE_TAG:
+        elif message.type == Gst.MessageType.TAG:
             self.__tag(message.parse_tag(), librarian)
-        elif message.type == gst.MESSAGE_ERROR:
+        elif message.type == Gst.MessageType.ERROR:
             err, debug = message.parse_error()
             err = str(err).decode(const.ENCODING, 'replace')
-            self.error(err, True)
-        elif message.type == gst.MESSAGE_BUFFERING:
+            self._error(err)
+        elif message.type == Gst.MessageType.BUFFERING:
             percent = message.parse_buffering()
             self.__buffering(percent)
-        elif message.type == gst.MESSAGE_ELEMENT:
-            name = ""
-            if hasattr(message.structure, "get_name"):
-                name = message.structure.get_name()
+        elif message.type == Gst.MessageType.STREAM_START:
+            if self._in_gapless_transition:
+                print_d("Stream changed")
+                self._end(False)
+        elif message.type == Gst.MessageType.ELEMENT:
+            message_name = message.get_structure().get_name()
 
-            # This gets sent on song change. Because it is not in the docs
-            # we can not rely on it. Additionally we check in get_position
-            # which should trigger shortly after this.
-            if USE_TRACK_CHANGE and self._in_gapless_transition and \
-                name == "playbin2-stream-changed":
-                    print_d("Stream changed")
-                    self._end(False)
-
-            if pbutils and name.startswith('missing-'):
+            if message_name == "missing-plugin":
                 self.stop()
-                detail = pbutils.missing_plugin_message_get_installer_detail(
-                    message)
-                context = pbutils.InstallPluginsContext()
-                pbutils.install_plugins_async([detail], context,
-                    lambda *args: gst.update_registry())
+                details = \
+                    GstPbutils.missing_plugin_message_get_installer_detail(
+                        message)
+                if details is not None:
+                    message = (_(
+                        "No GStreamer element found to handle the following "
+                        "media format: %(format_details)r") %
+                        {"format_details": details})
+                    print_w(message)
+
+                    context = GstPbutils.InstallPluginsContext.new()
+
+                    # TODO: track success
+                    def install_done_cb(*args):
+                        Gst.update_registry()
+                    res = GstPbutils.install_plugins_async(
+                        [details], context, install_done_cb, None)
+                    print_d("Gstreamer plugin install result: %r" % res)
+                    if res in (GstPbutils.InstallPluginsReturn.HELPER_MISSING,
+                            GstPbutils.InstallPluginsReturn.INTERNAL_FAILURE):
+                        self._error(message)
 
         return True
 
@@ -341,8 +361,8 @@ class GStreamerPlayer(BasePlayer):
 
         # push in the main loop and wait for it to finish
         event = threading.Event()
-        gobject.idle_add(change_in_main_loop, event, self._source,
-                         priority=gobject.PRIORITY_HIGH)
+        GLib.idle_add(change_in_main_loop, event, self._source,
+                         priority=GLib.PRIORITY_HIGH)
         event.wait()
 
         song = self._source.current
@@ -351,11 +371,9 @@ class GStreamerPlayer(BasePlayer):
         if song and bin:
             bin.set_property('uri', song("~uri"))
 
-        if not USE_TRACK_CHANGE:
-            gobject.idle_add(self._end, False)
-
     def stop(self):
-        self._end(True, stop=True)
+        super(GStreamerPlayer, self).stop()
+        self.__destroy_pipeline()
 
     def do_set_property(self, property, v):
         if property.name == 'volume':
@@ -376,21 +394,21 @@ class GStreamerPlayer(BasePlayer):
         or 0 if no song is playing."""
         p = self._last_position
         if self.song and self.bin:
-            try: p = self.bin.query_position(gst.FORMAT_TIME)[0]
-            except gst.QueryError: pass
-            else:
-                p //= gst.MSECOND
+            ok, p = self.bin.query_position(Gst.Format.TIME)
+            if ok:
+                p //= Gst.MSECOND
                 # During stream seeking querying the position fails.
                 # Better return the last valid one instead of 0.
                 self._last_position = p
         return p
 
     def __buffering(self, percent):
-        def stop_buf(*args): self.paused = True
+        def stop_buf(*args):
+            self.paused = True
 
         if percent < 100:
             if self._task:
-                self._task.update(percent/100.0)
+                self._task.update(percent / 100.0)
             else:
                 self._task = Task(_("Stream"), _("Buffering"), stop=stop_buf)
         elif self._task:
@@ -409,9 +427,9 @@ class GStreamerPlayer(BasePlayer):
 
         self._inhibit_play = inhibit
         if inhibit:
-            self.bin.set_state(gst.STATE_PAUSED)
+            self.bin.set_state(Gst.State.PAUSED)
         elif not self.paused:
-            self.bin.set_state(gst.STATE_PLAYING)
+            self.bin.set_state(Gst.State.PLAYING)
 
     def _set_paused(self, paused):
         if paused == self._paused:
@@ -430,14 +448,15 @@ class GStreamerPlayer(BasePlayer):
             if self.bin:
                 if self.song.is_file:
                     # fast path
-                    self.bin.set_state(gst.STATE_PAUSED)
+                    self.bin.set_state(Gst.State.PAUSED)
                 else:
                     # seekable streams (seem to) have a duration >= 0
-                    try: d = self.bin.query_duration(gst.FORMAT_TIME)[0]
-                    except gst.QueryError: d = -1
+                    ok, d = self.bin.query_duration(Gst.Format.TIME)
+                    if not ok:
+                        d = -1
 
                     if d >= 0:
-                        self.bin.set_state(gst.STATE_PAUSED)
+                        self.bin.set_state(Gst.State.PAUSED)
                     else:
                         # destroy so that we rebuffer on resume
                         self.__destroy_pipeline()
@@ -445,11 +464,10 @@ class GStreamerPlayer(BasePlayer):
             if self.bin:
                 # don't start while we are buffering
                 if not self._inhibit_play:
-                    self.bin.set_state(gst.STATE_PLAYING)
+                    self.bin.set_state(Gst.State.PLAYING)
             else:
                 if self.__init_pipeline():
-                    self.bin.set_property('uri', self.song("~uri"))
-                    self.bin.set_state(gst.STATE_PLAYING)
+                    self.bin.set_state(Gst.State.PLAYING)
                 else:
                     # Backend error; show message and halt playback
                     ErrorMessage(None, _("Output Error"),
@@ -462,31 +480,40 @@ class GStreamerPlayer(BasePlayer):
 
         self.emit((paused and 'paused') or 'unpaused')
 
-    def _get_paused(self): return self._paused
+    def _get_paused(self):
+        return self._paused
     paused = property(_get_paused, _set_paused)
 
-    def error(self, message, lock):
+    def _error(self, message):
         print_w(message)
-        self.emit('error', self.song, message, lock)
+        self.emit('error', self.song, message)
         if not self.paused:
             self.next()
 
     def seek(self, pos):
         """Seek to a position in the song, in milliseconds."""
         # Don't allow seeking during gapless. We can't go back to the old song.
-        if self.bin and self.song and not self._in_gapless_transition:
-            # ensure any pending state changes have completed
-            self.bin.get_state(timeout=gst.SECOND/2)
+        if not self.song or self._in_gapless_transition:
+            return
+
+        if self.__init_pipeline():
+            # ensure any pending state changes have completed and we have
+            # at least paused state, so we can seek
+            state = self.bin.get_state(timeout=STATE_CHANGE_TIMEOUT)[1]
+            if state < Gst.State.PAUSED:
+                self.bin.set_state(Gst.State.PAUSED)
+                self.bin.get_state(timeout=STATE_CHANGE_TIMEOUT)
+
             pos = max(0, int(pos))
-            gst_time = pos * gst.MSECOND
-            event = gst.event_new_seek(
-                1.0, gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH,
-                gst.SEEK_TYPE_SET, gst_time, gst.SEEK_TYPE_NONE, 0)
+            gst_time = pos * Gst.MSECOND
+            event = Gst.Event.new_seek(
+                1.0, Gst.Format.TIME, Gst.SeekFlags.FLUSH,
+                Gst.SeekType.SET, gst_time, Gst.SeekType.NONE, 0)
             if self.bin.send_event(event):
                 self._last_position = pos
                 self.emit('seek', self.song, pos)
 
-    def _end(self, stopped, stop=False):
+    def _end(self, stopped):
         print_d("End song")
         song, info = self.song, self.info
 
@@ -504,8 +531,7 @@ class GStreamerPlayer(BasePlayer):
         self.emit('song-ended', song, stopped)
 
         # Then, set up the next song.
-        if not stop:
-            self.song = self.info = self._source.current
+        self.song = self.info = self._source.current
         self.emit('song-started', self.song)
 
         print_d("Next song")
@@ -516,14 +542,13 @@ class GStreamerPlayer(BasePlayer):
                 # entire pipeline and recreate it each time we're not in
                 # a gapless transition.
                 self.__destroy_pipeline()
-                if self.__init_pipeline():
-                    self.bin.set_property('uri', self.song("~uri"))
-                else: self.paused = True
+                if not self.__init_pipeline():
+                    self.paused = True
             if self.bin:
                 if self.paused:
-                    self.bin.set_state(gst.STATE_PAUSED)
+                    self.bin.set_state(Gst.State.PAUSED)
                 else:
-                    self.bin.set_state(gst.STATE_PLAYING)
+                    self.bin.set_state(Gst.State.PLAYING)
         else:
             self.__destroy_pipeline()
             self.paused = True
@@ -542,6 +567,7 @@ class GStreamerPlayer(BasePlayer):
         if not new_info:
             new_info = type(self.song)(self.song["~filename"])
             new_info.multisong = False
+            new_info.streamsong = True
 
             # copy from the old songs
             # we should probably listen to the library for self.song changes
@@ -551,6 +577,7 @@ class GStreamerPlayer(BasePlayer):
         changed = False
         info_changed = False
 
+        tags = TagListWrapper(tags, merge=True)
         tags = parse_gstreamer_taglist(tags)
 
         for key, value in sanitize_tags(tags, stream=False).iteritems():
@@ -588,7 +615,7 @@ class GStreamerPlayer(BasePlayer):
 
     @property
     def eq_bands(self):
-        if gst.element_factory_find('equalizer-10bands'):
+        if Gst.ElementFactory.find('equalizer-10bands'):
             return [29, 59, 119, 237, 474, 947, 1889, 3770, 7523, 15011]
         else:
             return []
@@ -597,27 +624,36 @@ class GStreamerPlayer(BasePlayer):
         need_eq = any(self._eq_values)
         if need_eq != self._use_eq:
             self._use_eq = need_eq
-            self.__rebuild_pipeline()
+            self._rebuild_pipeline()
 
         if self._eq_element:
             for band, val in enumerate(self._eq_values):
                 self._eq_element.set_property('band%d' % band, val)
 
-def can_play_uri(uri):
-    return gst.element_make_from_uri(gst.URI_SRC, uri, '') is not None
+    def can_play_uri(self, uri):
+        if not Gst.uri_is_valid(uri):
+            return False
+        try:
+            Gst.Element.make_from_uri(Gst.URIType.SRC, uri, '')
+        except GLib.GError:
+            return False
+        return True
+
 
 def init(librarian):
-    gst.debug_set_default_threshold(gst.LEVEL_ERROR)
+    # Enable error messages by default
+    if Gst.debug_get_default_threshold() == Gst.DebugLevel.NONE:
+        Gst.debug_set_default_threshold(Gst.DebugLevel.ERROR)
 
     # the fluendo decoder is twice as slow as mad, but wins
     # at autoplug because it has the same rank and f < m
     # -> put it slightly behind mad or leave it if it already is
-    flu, mad = map(gst.element_factory_find, ["flump3dec", "mad"])
+    flu, mad = map(Gst.ElementFactory.find, ["flump3dec", "mad"])
     if flu and mad:
         flu.set_rank(min(flu.get_rank(), max(mad.get_rank() - 1, 0)))
 
-    if gst.element_make_from_uri(
-        gst.URI_SRC,
+    if Gst.Element.make_from_uri(
+        Gst.URIType.SRC,
         "file:///fake/path/for/gst", ""):
         return GStreamerPlayer(librarian)
     else:
