@@ -4,10 +4,10 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from quodlibet.player import error as PlayerError
 from quodlibet.player._base import BasePlayer
 from quodlibet.qltk.songlist import PlaylistModel
 
+
 class NullPlayer(BasePlayer):
     __gproperties__ = BasePlayer._gproperties_
     __gsignals__ = BasePlayer._gsignals_
@@ -18,11 +18,12 @@ class NullPlayer(BasePlayer):
         super(NullPlayer, self).__init__()
         self._set_paused(True)
         self._source = PlaylistModel()
+        self._position = 0
 
     def get_position(self):
         """Return the current playback position in milliseconds,
         or 0 if no song is playing."""
-        return 0
+        return self._position
 
     def _set_paused(self, paused):
         if paused != self._paused:
@@ -33,7 +34,9 @@ class NullPlayer(BasePlayer):
                 # Something wants us to pause between songs, or when
                 # we've got no song playing (probably StopAfterMenu).
                 self.emit('paused')
-    def _get_paused(self): return self._paused
+
+    def _get_paused(self):
+        return self._paused
     paused = property(_get_paused, _set_paused)
 
     def do_set_property(self, property, v):
@@ -42,8 +45,8 @@ class NullPlayer(BasePlayer):
         else:
             raise AttributeError
 
-    def error(self, message, lock):
-        self.emit('error', self.song, message, lock)
+    def error(self, message):
+        self.emit('error', self.song, message)
         if not self.paused:
             self.next()
 
@@ -51,6 +54,7 @@ class NullPlayer(BasePlayer):
         """Seek to a position in the song, in milliseconds."""
         if self.song:
             self.emit('seek', self.song, pos)
+        self._position = pos
 
     def _end(self, stopped):
         # We need to set self.song to None before calling our signal
@@ -67,9 +71,11 @@ class NullPlayer(BasePlayer):
         if self.song is None:
             self.paused = True
 
-def can_play_uri(uri):
-    return False
+        self._position = 0
+
+    def can_play_uri(self, uri):
+        return False
+
 
 def init(librarian):
     return NullPlayer(librarian)
-
