@@ -5,9 +5,8 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import gtk
+from gi.repository import Gtk
 
-from quodlibet.plugins import Manager
 
 class RenameFilesPlugin(object):
     """Plugins of this type must subclass a GTK widget. They will be
@@ -33,13 +32,17 @@ class RenameFilesPlugin(object):
     _order = 0.0
     active = False
 
-    def filter(self, original_filename, value): return value
-    def filter_list(self, origs, names): return map(self.filter, origs, names)
+    def filter(self, original_filename, value):
+        return value
+
+    def filter_list(self, origs, names):
+        return map(self.filter, origs, names)
 
     def __cmp__(self, other):
         return (cmp(self._order, other._order) or
                 cmp(type(self).__name__, type(other).__name__))
 
+
 class TagsFromPathPlugin(object):
     """Plugins of this type must subclass a GTK widget. They will be
     packed into the TagsFromPath pane (currently a ScrolledWindow hidden
@@ -68,13 +71,15 @@ class TagsFromPathPlugin(object):
     _order = 0
     active = False
 
-    def filter(self, tag, value): return value
+    def filter(self, tag, value):
+        return value
 
     def __cmp__(self, other):
         return (cmp(self._order, other._order) or
                 cmp(type(self).__name__, type(other).__name__))
 
-class EditTagsPlugin(gtk.ImageMenuItem):
+
+class EditTagsPlugin(Gtk.ImageMenuItem):
     """Plugins of this type are subclasses of gtk.ImageMenuItem.
     They will be added to the context menu of the EditTags tree view.
 
@@ -117,7 +122,8 @@ class EditTagsPlugin(gtk.ImageMenuItem):
     needs = []
     _order = 2.0
 
-    def activated(self, tag, value): return [(tag, value)]
+    def activated(self, tag, value):
+        return [(tag, value)]
 
     def connect(self, signal, callback, *args, **kwargs):
         if self.get_submenu():
@@ -126,15 +132,3 @@ class EditTagsPlugin(gtk.ImageMenuItem):
         else:
             super(EditTagsPlugin, self).connect(
                 signal, callback, *args, **kwargs)
-
-class EditingPlugins(Manager):
-    Kinds = [EditTagsPlugin, RenameFilesPlugin, TagsFromPathPlugin]
-
-    def RenamePlugins(self):
-        return super(EditingPlugins, self).find_subclasses(RenameFilesPlugin)
-
-    def TagsFromPathPlugins(self):
-        return super(EditingPlugins, self).find_subclasses(TagsFromPathPlugin)
-
-    def EditTagsPlugins(self):
-        return super(EditingPlugins, self).find_subclasses(EditTagsPlugin)
