@@ -5,45 +5,54 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import gtk
+from gi.repository import Gtk
 import mutagen
 
+from quodlibet.qltk import gtk_version, pygobject_version
 from quodlibet import const
 from quodlibet import formats
 from quodlibet.util import fver
 
-class AboutDialog(gtk.AboutDialog):
-    def __init__(self, parent, player, name):
+
+class AboutDialog(Gtk.AboutDialog):
+    def __init__(self, parent, player, name, icon):
         super(AboutDialog, self).__init__()
         self.set_transient_for(parent)
         self.set_program_name(name)
         self.set_version(const.VERSION)
         self.set_authors(const.AUTHORS)
         self.set_artists(const.ARTISTS)
-        fmts = ", ".join(formats.modules)
+        self.set_logo_icon_name(icon)
+
+        def chunks(l, n):
+            return [l[i:i + n] for i in range(0, len(l), n)]
+
+        fmts = ",\n".join(", ".join(c) for c in chunks(formats.names, 4))
         text = []
         text.append(_("Supported formats: %s") % fmts)
+        text.append("")
         if player:
             text.append(_("Audio device: %s") % player.name)
         text.append("Mutagen: %s" % fver(mutagen.version))
-        text.append("GTK+: %s / PyGTK: %s" %(
-            fver(gtk.gtk_version), fver(gtk.pygtk_version)))
+        text.append("GTK+: %s" % fver(gtk_version))
+        text.append("PyGObject: %s" % fver(pygobject_version))
         if player:
             text.append(player.version_info)
         self.set_comments("\n".join(text))
-        # Translators: Replace this with your name/email to have it appear
-        # in the "About" dialog.
-        self.set_translator_credits(_('translator-credits'))
-        self.set_website("http://code.google.com/p/quodlibet")
-        self.set_copyright(
-            "Copyright © 2004-2011 Joe Wreschnig, Michael Urman, & others\n"
-            "<quod-libet-development@googlegroups.com>")
-        self.child.show_all()
+        self.set_translator_credits("\n".join(const.TRANSLATORS))
+        self.set_website(const.WEBSITE)
+        self.set_copyright(const.COPYRIGHT + "\n" +
+                           "<%s>" % const.SUPPORT_EMAIL)
+        self.get_child().show_all()
+
 
 class AboutQuodLibet(AboutDialog):
     def __init__(self, parent, player):
-        super(AboutQuodLibet, self).__init__(parent, player, "Quod Libet")
+        super(AboutQuodLibet, self).__init__(
+            parent, player, "Quod Libet", "quodlibet")
+
 
 class AboutExFalso(AboutDialog):
     def __init__(self, parent, player=None):
-        super(AboutExFalso, self).__init__(parent, player, "Ex Falso")
+        super(AboutExFalso, self).__init__(
+            parent, player, "Ex Falso", "exfalso")
