@@ -7,8 +7,7 @@
 
 import sys
 
-import gtk
-import pango
+from gi.repository import Gtk, Pango, Gdk
 
 from quodlibet import qltk
 
@@ -20,13 +19,18 @@ from quodlibet.util import massagers
 
 from quodlibet.qltk.completion import LibraryValueCompletion
 from quodlibet.qltk.tagscombobox import TagsComboBox, TagsComboBoxEntry
-from quodlibet.qltk.views import RCMHintedTreeView
+from quodlibet.qltk.views import RCMHintedTreeView, TreeViewColumn
 from quodlibet.qltk.wlw import WritingWindow
+from quodlibet.qltk.x import SeparatorMenuItem
+from quodlibet.qltk._editpane import EditingPluginHandler
+from quodlibet.plugins import PluginManager
+
 
 class AudioFileGroup(dict):
 
     class Comment(unicode):
         complete = True
+
         def __repr__(self):
             return '%s %s' % (str(self), self.paren())
 
@@ -52,16 +56,25 @@ class AudioFileGroup(dict):
                 return ", ".join([d, m])
 
         def safenicestr(self):
-            if self.shared and self.complete: return str(self)
+            if self.shared and self.complete:
+                return util.escape(self.encode("utf-8"))
             elif self.shared:
                 return "\n".join(['%s<i> (%s)</i>' % (s, self.paren())
                                   for s in str(self).split("\n")])
-            else: return '<i>(%s)</i>' % self.paren()
+            else:
+                return '<i>(%s)</i>' % self.paren()
 
-    class SharedComment(Comment): shared = True
-    class UnsharedComment(Comment): shared = False
-    class PartialSharedComment(SharedComment): complete = False
-    class PartialUnsharedComment(UnsharedComment): complete = False
+    class SharedComment(Comment):
+        shared = True
+
+    class UnsharedComment(Comment):
+        shared = False
+
+    class PartialSharedComment(SharedComment):
+        complete = False
+
+    class PartialUnsharedComment(UnsharedComment):
+        complete = False
 
     def realkeys(self):
         return filter(lambda s: s and "~" not in s and "=" not in s, self)
@@ -93,9 +106,12 @@ class AudioFileGroup(dict):
                     value = self.PartialUnsharedComment(first[comment])
             else:
                 decoded = first[comment]
-                if isinstance(decoded, str): decoded = util.decode(decoded)
-                if all[comment]: value = self.SharedComment(decoded)
-                else: value = self.UnsharedComment(decoded)
+                if isinstance(decoded, str):
+                    decoded = util.decode(decoded)
+                if all[comment]:
+                    value = self.SharedComment(decoded)
+                else:
+                    value = self.UnsharedComment(decoded)
             value.have = count
             value.total = total
             value.missing = total - count
@@ -107,23 +123,29 @@ class AudioFileGroup(dict):
             can = True
             for song in self.songs:
                 cantoo = song.can_change()
-                if can is True: can = cantoo
-                elif cantoo is True: pass
-                else: can = set(can) | set(cantoo)
+                if can is True:
+                    can = cantoo
+                elif cantoo is True:
+                    pass
+                else:
+                    can = set(can) | set(cantoo)
         else:
-            if not self.songs: return False
+            if not self.songs:
+                return False
             can = min([song.can_change(k) for song in self.songs])
         return can
 
-class SplitValues(gtk.ImageMenuItem):
+
+class SplitValues(Gtk.ImageMenuItem):
     tags = False
     needs = []
     _order = 0.0
 
     def __init__(self, tag, value):
-        super(SplitValues, self).__init__(_("Split into _Multiple Values"))
-        self.set_image(gtk.image_new_from_stock(
-            gtk.STOCK_FIND_AND_REPLACE, gtk.ICON_SIZE_MENU))
+        super(SplitValues, self).__init__(
+            _("Split into _Multiple Values"), use_underline=True)
+        self.set_image(Gtk.Image.new_from_stock(
+            Gtk.STOCK_FIND_AND_REPLACE, Gtk.IconSize.MENU))
         spls = config.get("editing", "split_on").decode(
             'utf-8', 'replace').split()
         self.set_sensitive(len(util.split_value(value, spls)) > 1)
@@ -131,32 +153,36 @@ class SplitValues(gtk.ImageMenuItem):
     def activated(self, tag, value):
         spls = config.get("editing", "split_on").decode(
             'utf-8', 'replace').split()
-        return [(tag, value) for value in util.split_value(value, spls)]
+        return [(tag, v) for v in util.split_value(value, spls)]
 
-class SplitDisc(gtk.ImageMenuItem):
+
+class SplitDisc(Gtk.ImageMenuItem):
     tags = ["album"]
     needs = ["discnumber"]
     _order = 0.5
 
     def __init__(self, tag, value):
-        super(SplitDisc, self).__init__(_("Split Disc out of _Album"))
-        self.set_image(gtk.image_new_from_stock(
-            gtk.STOCK_FIND_AND_REPLACE, gtk.ICON_SIZE_MENU))
+        super(SplitDisc, self).__init__(
+            _("Split Disc out of _Album"), use_underline=True)
+        self.set_image(Gtk.Image.new_from_stock(
+            Gtk.STOCK_FIND_AND_REPLACE, Gtk.IconSize.MENU))
         self.set_sensitive(util.split_album(value)[1] is not None)
 
     def activated(self, tag, value):
         album, disc = util.split_album(value)
         return [(tag, album), ("discnumber", disc)]
 
-class SplitTitle(gtk.ImageMenuItem):
+
+class SplitTitle(Gtk.ImageMenuItem):
     tags = ["title"]
     needs = ["version"]
     _order = 0.5
 
     def __init__(self, tag, value):
-        super(SplitTitle, self).__init__(_("Split _Version out of Title"))
-        self.set_image(gtk.image_new_from_stock(
-            gtk.STOCK_FIND_AND_REPLACE, gtk.ICON_SIZE_MENU))
+        super(SplitTitle, self).__init__(
+            _("Split _Version out of Title"), use_underline=True)
+        self.set_image(Gtk.Image.new_from_stock(
+            Gtk.STOCK_FIND_AND_REPLACE, Gtk.IconSize.MENU))
         spls = config.get("editing", "split_on").decode(
             'utf-8', 'replace').split()
         self.set_sensitive(bool(util.split_title(value, spls)[1]))
@@ -167,14 +193,15 @@ class SplitTitle(gtk.ImageMenuItem):
         title, versions = util.split_title(value, spls)
         return [(tag, title)] + [("version", v) for v in versions]
 
-class SplitPerson(gtk.ImageMenuItem):
+
+class SplitPerson(Gtk.ImageMenuItem):
     tags = ["artist"]
     _order = 0.5
 
     def __init__(self, tag, value):
-        super(SplitPerson, self).__init__(self.title)
-        self.set_image(gtk.image_new_from_stock(
-            gtk.STOCK_FIND_AND_REPLACE, gtk.ICON_SIZE_MENU))
+        super(SplitPerson, self).__init__(self.title, use_underline=True)
+        self.set_image(Gtk.Image.new_from_stock(
+            Gtk.STOCK_FIND_AND_REPLACE, Gtk.IconSize.MENU))
         spls = config.get("editing", "split_on").decode(
             'utf-8', 'replace').split()
         self.set_sensitive(bool(util.split_people(value, spls)[1]))
@@ -185,44 +212,49 @@ class SplitPerson(gtk.ImageMenuItem):
         artist, others = util.split_people(value, spls)
         return [(tag, artist)] + [(self.needs[0], o) for o in others]
 
+
 class SplitArranger(SplitPerson):
     needs = ["arranger"]
     title = _("Split Arranger out of Ar_tist")
 
+
 class SplitPerformer(SplitPerson):
     needs = ["performer"]
     title = _("Split _Performer out of Artist")
 
+
 class SplitPerformerFromTitle(SplitPerson):
     tags = ["title"]
     needs = ["performer"]
     title = _("Split _Performer out of Title")
 
 
-class AddTagDialog(gtk.Dialog):
+class SplitOriginalArtistFromTitle(SplitPerson):
+    tags = ["title"]
+    needs = ["originalartist"]
+    title = _("Split _Originalartist out of Title")
+
+
+class AddTagDialog(Gtk.Dialog):
+
     def __init__(self, parent, can_change, library):
-        if can_change == True:
-            can = sorted(formats.USEFUL_TAGS)
-        else:
-            can = sorted(can_change)
         super(AddTagDialog, self).__init__(
             _("Add a Tag"), qltk.get_top_parent(parent))
         self.set_border_width(6)
-        self.set_has_separator(False)
         self.set_resizable(False)
-        self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
-        add = self.add_button(gtk.STOCK_ADD, gtk.RESPONSE_OK)
+        self.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
+        add = self.add_button(Gtk.STOCK_ADD, Gtk.ResponseType.OK)
         self.vbox.set_spacing(6)
-        self.set_default_response(gtk.RESPONSE_OK)
-        table = gtk.Table(2, 2)
+        self.set_default_response(Gtk.ResponseType.OK)
+        table = Gtk.Table(2, 2)
         table.set_row_spacings(12)
         table.set_col_spacings(6)
         table.set_border_width(6)
 
-        if can_change == True: self.__tag = TagsComboBoxEntry()
-        else: self.__tag = TagsComboBox(can_change)
+        self.__tag = (TagsComboBoxEntry() if can_change is True
+                      else TagsComboBox(can_change))
 
-        label = gtk.Label()
+        label = Gtk.Label()
         label.set_alignment(0.0, 0.5)
         label.set_text(_("_Tag:"))
         label.set_use_underline(True)
@@ -230,26 +262,26 @@ class AddTagDialog(gtk.Dialog):
         table.attach(label, 0, 1, 0, 1)
         table.attach(self.__tag, 1, 2, 0, 1)
 
-        self.__val = gtk.Entry()
+        self.__val = Gtk.Entry()
         self.__val.set_completion(LibraryValueCompletion("", library))
-        label = gtk.Label()
+        label = Gtk.Label()
         label.set_text(_("_Value:"))
         label.set_alignment(0.0, 0.5)
         label.set_use_underline(True)
         label.set_mnemonic_widget(self.__val)
-        valuebox = gtk.EventBox()
+        valuebox = Gtk.EventBox()
         table.attach(label, 0, 1, 1, 2)
         table.attach(valuebox, 1, 2, 1, 2)
-        hbox = gtk.HBox()
+        hbox = Gtk.HBox()
         valuebox.add(hbox)
-        hbox.pack_start(self.__val)
+        hbox.pack_start(self.__val, True, True, 0)
         hbox.set_spacing(6)
-        invalid = gtk.image_new_from_stock(
-            gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_SMALL_TOOLBAR)
-        hbox.pack_start(invalid)
+        invalid = Gtk.Image.new_from_stock(
+            Gtk.STOCK_DIALOG_WARNING, Gtk.IconSize.SMALL_TOOLBAR)
+        hbox.pack_start(invalid, True, True, 0)
 
-        self.vbox.pack_start(table)
-        self.child.show_all()
+        self.vbox.pack_start(table, True, True, 0)
+        self.get_child().show_all()
         invalid.hide()
 
         for entry in [self.__tag, self.__val]:
@@ -258,9 +290,9 @@ class AddTagDialog(gtk.Dialog):
         self.__tag.connect('changed', self.__set_value_completion, library)
         self.__set_value_completion(self.__tag, library)
 
-        if can_change == True:
-            self.__tag.child.connect_object(
-                'activate', gtk.Entry.grab_focus, self.__val)
+        if can_change is True:
+            self.__tag.get_child().connect_object(
+                'activate', Gtk.Entry.grab_focus, self.__val)
 
     def __set_value_completion(self, tag, library):
         completion = self.__val.get_completion()
@@ -268,7 +300,8 @@ class AddTagDialog(gtk.Dialog):
             completion.set_tag(self.__tag.tag, library)
 
     def get_tag(self):
-        try: return self.__tag.tag
+        try:
+            return self.__tag.tag
         except AttributeError:
             return self.__tag.tag
 
@@ -279,12 +312,14 @@ class AddTagDialog(gtk.Dialog):
         tag = self.get_tag()
         value = self.get_value()
         fmt = massagers.tags.get(tag)
-        if fmt: valid = fmt.is_valid(value)
-        else: valid = True
+        if fmt:
+            valid = fmt.is_valid(value)
+        else:
+            valid = True
         add.set_sensitive(valid)
         if valid:
             invalid.hide()
-            box.set_tooltip_text(None)
+            box.set_tooltip_text("")
         else:
             invalid.show()
             box.set_tooltip_text(fmt.error)
@@ -297,103 +332,143 @@ class AddTagDialog(gtk.Dialog):
 
 TAG, VALUE, EDITED, CANEDIT, DELETED, ORIGVALUE, RENAMED, ORIGTAG = range(8)
 
+
 def is_special(string):
     return string.endswith("</i>")
 
+
 def is_different(string):
     return is_special(string) and string.startswith("<i>")
 
+
 def is_missing(string):
     return is_special(string) and not string.startswith("<i>")
 
-def strip_missing(string):
-    try: return string[:string.index(" <i>")]
-    except ValueError: return string
 
-class EditTags(gtk.VBox):
+def strip_missing(string):
+    try:
+        return string[:string.index(" <i>")]
+    except ValueError:
+        return string
+
+
+class EditTagsPluginHandler(EditingPluginHandler):
+    from quodlibet.plugins.editing import EditTagsPlugin
+    Kind = EditTagsPlugin
+
+
+class EditTags(Gtk.VBox):
+    _SAVE_BUTTON_KEY = 'ql-save'
+    _REVERT_BUTTON_KEY = 'ql-revert'
+    # Translators: translate only to override the text
+    # for the tag "save" button
+    _SAVE_BUTTON_TEXT = _('ql-save')
+    # Translators: translate only to override the  for the tag "revert" button
+    _REVERT_BUTTON_TEXT = _('ql-revert')
+    handler = EditTagsPluginHandler()
+
+    @classmethod
+    def init_plugins(cls):
+        PluginManager.instance.register_handler(cls.handler)
+
     def __init__(self, parent, library):
         super(EditTags, self).__init__(spacing=12)
         self.title = _("Edit Tags")
         self.set_border_width(12)
 
-        model = gtk.ListStore(str, str, bool, bool, bool, str, bool, str)
+        model = Gtk.ListStore(str, str, bool, bool, bool, str, bool, str)
         view = RCMHintedTreeView(model)
         selection = view.get_selection()
-        render = gtk.CellRendererPixbuf()
-        column = gtk.TreeViewColumn(_("Write"), render)
+        render = Gtk.CellRendererPixbuf()
+        column = TreeViewColumn(_("Write"), render)
 
         style = view.get_style()
-        pixbufs = [ style.lookup_icon_set(stock)
-                    .render_icon(style, gtk.TEXT_DIR_NONE, state,
-                        gtk.ICON_SIZE_MENU, view, None)
-                    for state in (gtk.STATE_INSENSITIVE, gtk.STATE_NORMAL)
-                        for stock in (gtk.STOCK_EDIT, gtk.STOCK_DELETE) ]
+        pixbufs = [style.lookup_icon_set(stock)
+                   .render_icon(style, Gtk.TextDirection.NONE, state,
+                                Gtk.IconSize.MENU, view, None)
+                   for state in (Gtk.StateType.INSENSITIVE,
+                                 Gtk.StateType.NORMAL)
+                   for stock in (Gtk.STOCK_EDIT, Gtk.STOCK_DELETE)]
+
         def cdf_write(col, rend, model, iter, (write, delete)):
             row = model[iter]
             if row[CANEDIT]:
                 rend.set_property('stock-id', None)
-                rend.set_property('pixbuf', pixbufs[2*row[EDITED]+row[DELETED]])
+                rend.set_property('pixbuf',
+                    pixbufs[2 * row[EDITED] + row[DELETED]])
             else:
-                rend.set_property('stock-id', gtk.STOCK_DIALOG_AUTHENTICATION)
+                rend.set_property('stock-id', Gtk.STOCK_DIALOG_AUTHENTICATION)
         column.set_cell_data_func(render, cdf_write, (2, 4))
         view.append_column(column)
 
-        render = gtk.CellRendererText()
-        column = gtk.TreeViewColumn(
+        render = Gtk.CellRendererText()
+        column = TreeViewColumn(
             _('Tag'), render, text=0, editable=3, strikethrough=4)
-        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
+        column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         render.set_property('editable', True)
         render.connect('edited', self.__edit_tag_name, model)
         render.connect(
             'editing-started', self.__tag_editing_started, model, library)
         view.append_column(column)
 
-        render = gtk.CellRendererText()
-        render.set_property('ellipsize', pango.ELLIPSIZE_END)
+        render = Gtk.CellRendererText()
+        render.set_property('ellipsize', Pango.EllipsizeMode.END)
         render.set_property('editable', True)
         render.connect('edited', self.__edit_tag, model)
         render.connect(
             'editing-started', self.__value_editing_started, model, library)
         render.markup = 1
-        column = gtk.TreeViewColumn(
+        column = TreeViewColumn(
             _('Value'), render, markup=1, editable=3, strikethrough=4)
-        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
+        column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         view.append_column(column)
 
-        sw = gtk.ScrolledWindow()
-        sw.set_shadow_type(gtk.SHADOW_IN)
-        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        sw = Gtk.ScrolledWindow()
+        sw.set_shadow_type(Gtk.ShadowType.IN)
+        sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         sw.add(view)
-        self.pack_start(sw)
+        self.pack_start(sw, True, True, 0)
 
-        buttonbox = gtk.HBox(spacing=18)
-        bbox1 = gtk.HButtonBox()
+        # Add and Remove [tags] buttons
+        buttonbox = Gtk.HBox(spacing=18)
+        bbox1 = Gtk.HButtonBox()
         bbox1.set_spacing(6)
-        bbox1.set_layout(gtk.BUTTONBOX_START)
-        add = gtk.Button(stock=gtk.STOCK_ADD)
+        bbox1.set_layout(Gtk.ButtonBoxStyle.START)
+        add = Gtk.Button(stock=Gtk.STOCK_ADD)
         add.set_focus_on_click(False)
         add.connect('clicked', self.__add_tag, model, library)
-        remove = gtk.Button(stock=gtk.STOCK_REMOVE)
+        bbox1.pack_start(add, True, True, 0)
+        # Remove button
+        remove = Gtk.Button(stock=Gtk.STOCK_REMOVE)
         remove.set_focus_on_click(False)
         remove.connect('clicked', self.__remove_tag, view)
         remove.set_sensitive(False)
-        bbox1.pack_start(add)
-        bbox1.pack_start(remove)
 
-        bbox2 = gtk.HButtonBox()
+        bbox1.pack_start(remove, True, True, 0)
+
+        # Revert and save buttons
+        # Both can have customised translated text (and thus accels)
+        bbox2 = Gtk.HButtonBox()
         bbox2.set_spacing(6)
-        bbox2.set_layout(gtk.BUTTONBOX_END)
-        revert = gtk.Button(stock=gtk.STOCK_REVERT_TO_SAVED)
-        save = gtk.Button(stock=gtk.STOCK_SAVE)
+        bbox2.set_layout(Gtk.ButtonBoxStyle.END)
+        revert = (Gtk.Button(stock=Gtk.STOCK_REVERT_TO_SAVED)
+                  if self._REVERT_BUTTON_KEY == self._REVERT_BUTTON_TEXT
+                  else Gtk.Button(label=self._REVERT_BUTTON_TEXT,
+                                  use_underline=True))
         revert.set_sensitive(False)
+        # Save button.
+        save = (Gtk.Button(stock=Gtk.STOCK_SAVE)
+                if self._SAVE_BUTTON_TEXT == self._SAVE_BUTTON_KEY
+                else Gtk.Button(label=self._SAVE_BUTTON_TEXT,
+                                use_underline=True))
         save.set_sensitive(False)
-        bbox2.pack_start(revert)
-        bbox2.pack_start(save)
+        self.save = save
+        bbox2.pack_start(revert, True, True, 0)
+        bbox2.pack_start(save, True, True, 0)
 
-        buttonbox.pack_start(bbox1)
-        buttonbox.pack_start(bbox2)
-
-        self.pack_start(buttonbox, expand=False)
+        buttonbox.pack_start(bbox1, True, True, 0)
+        buttonbox.pack_start(bbox2, True, True, 0)
+        self.pack_start(buttonbox, False, True, 0)
 
         UPDATE_ARGS = [
             view, buttonbox, model, add, [save, revert, remove]]
@@ -413,21 +488,28 @@ class EditTags(gtk.VBox):
         view.connect('button-press-event', self.__button_press)
         view.connect('key-press-event', self.__view_key_press_event)
         selection.connect('changed', self.__tag_select, remove)
-        selection.set_mode(gtk.SELECTION_MULTIPLE)
+        selection.set_mode(Gtk.SelectionMode.MULTIPLE)
         self.show_all()
 
     def __view_key_press_event(self, view, event):
         # We can't use a real accelerator to this because it would
         # interfere with typeahead and row editing.
-        if event.keyval == gtk.accelerator_parse("Delete")[0]:
+        ctrl = event.get_state() & Gdk.ModifierType.CONTROL_MASK
+        keyval_name = Gdk.keyval_name(event.keyval)
+        if event.keyval == Gtk.accelerator_parse("Delete")[0]:
             self.__remove_tag(view, view)
+        elif ctrl and keyval_name == 's':
+            # Issue 697: allow Ctrl-s to save.
+            self.save.emit('clicked')
 
     def __enable_save(self, *args):
         buttons = args[-1]
-        for b in buttons: b.set_sensitive(True)
+        for b in buttons:
+            b.set_sensitive(True)
 
     def __paste(self, clip, text, (rend, path)):
-        if text: rend.emit('edited', path, text.strip())
+        if text:
+            rend.emit('edited', path, text.strip())
 
     def __menu_activate(self, activator, view):
         model, (iter,) = view.get_selection().get_selected_rows()
@@ -442,22 +524,24 @@ class EditTags(gtk.VBox):
                     replaced = True
                     row[VALUE] = util.escape(aval)
                     row[EDITED] = True
-                else: self.__add_new_tag(model, atag, aval)
-        elif vals: replaced = True
-        if not replaced: row[EDITED] = row[DELETED] = True
+                else:
+                    self.__add_new_tag(model, atag, aval)
+        elif vals:
+            replaced = True
+        if not replaced:
+            row[EDITED] = row[DELETED] = True
 
     def __popup_menu(self, view, parent):
-        menu = gtk.Menu()
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        menu = Gtk.Menu()
 
         view.ensure_popup_selection()
         model, rows = view.get_selection().get_selected_rows()
         can_change = min([model[path][CANEDIT] for path in rows])
 
         items = [SplitDisc, SplitTitle, SplitPerformer, SplitArranger,
-                 SplitValues, SplitPerformerFromTitle]
-        items.extend(parent.plugins.EditTagsPlugins())
+                 SplitValues, SplitPerformerFromTitle,
+                 SplitOriginalArtistFromTitle]
+        items.extend(self.handler.plugins)
         items.sort(key=lambda item: (item._order, item.__name__))
 
         if len(rows) == 1:
@@ -468,29 +552,31 @@ class EditTags(gtk.VBox):
             multi = (value.split("<")[0] != value)
 
             for Item in items:
-                if Item.tags and row[TAG] not in Item.tags: continue
+                if Item.tags and row[TAG] not in Item.tags:
+                    continue
 
-                try: b = Item(row[TAG], text)
+                try:
+                    b = Item(row[TAG], text)
                 except:
                     util.print_exc()
                 else:
                     b.connect('activate', self.__menu_activate, view)
 
-                    if not min(map(self.__songinfo.can_change, b.needs)+[1]) \
-                        or multi:
+                    if (not min(map(self.__songinfo.can_change, b.needs) + [1])
+                            or multi):
                         b.set_sensitive(False)
 
                     menu.append(b)
 
+            if menu.get_children():
+                menu.append(SeparatorMenuItem())
 
-            if menu.get_children(): menu.append(gtk.SeparatorMenuItem())
-
-        b = gtk.ImageMenuItem(gtk.STOCK_REMOVE, gtk.ICON_SIZE_MENU)
+        b = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_REMOVE, None)
         b.connect('activate', self.__remove_tag, view)
-        keyval, mod = gtk.accelerator_parse("Delete")
-        menu.__accels = gtk.AccelGroup()
+        keyval, mod = Gtk.accelerator_parse("Delete")
+        menu.__accels = Gtk.AccelGroup()
         b.add_accelerator(
-            'activate', menu.__accels, keyval, mod, gtk.ACCEL_VISIBLE)
+            'activate', menu.__accels, keyval, mod, Gtk.AccelFlags.VISIBLE)
         menu.append(b)
 
         menu.show_all()
@@ -499,7 +585,10 @@ class EditTags(gtk.VBox):
         for c in menu.get_children():
             c.set_sensitive(can_change and c.get_property('sensitive'))
         menu.connect('selection-done', lambda m: m.destroy())
-        return view.popup_menu(menu, 3, gtk.get_current_event_time())
+
+        # XXX: Keep reference
+        self.__menu = menu
+        return view.popup_menu(menu, 3, Gtk.get_current_event_time())
 
     def __tag_select(self, selection, remove):
         model, rows = selection.get_selected_rows()
@@ -515,21 +604,20 @@ class EditTags(gtk.VBox):
             qltk.ErrorMessage(self, title, msg).run()
             return
 
-        edited = True
-        edit = True
-        orig = None
-        deleted = False
         iters = [row.iter for row in model if row[TAG] == tag]
         row = [tag, util.escape(value), True, True, False, None, False, None]
-        if len(iters): model.insert_after(iters[-1], row=row)
-        else: model.append(row=row)
+        if len(iters):
+            model.insert_after(iters[-1], row=row)
+        else:
+            model.append(row=row)
 
     def __add_tag(self, activator, model, library):
         add = AddTagDialog(self, self.__songinfo.can_change(), library)
 
         while True:
             resp = add.run()
-            if resp != gtk.RESPONSE_OK: break
+            if resp != Gtk.ResponseType.OK:
+                break
             tag = add.get_tag()
             value = add.get_value()
             if tag in massagers.tags:
@@ -554,7 +642,8 @@ class EditTags(gtk.VBox):
         for row in rows:
             if row[ORIGVALUE] is not None:
                 row[EDITED] = row[DELETED] = True
-            else: model.remove(row.iter)
+            else:
+                model.remove(row.iter)
 
     def __save_files(self, save, revert, model, library):
         updated = {}
@@ -600,8 +689,10 @@ class EditTags(gtk.VBox):
                 for (new_value, old_value) in values:
                     new_value = util.unescape(new_value)
                     if song.can_change(key):
-                        if old_value is None: song.add(key, new_value)
-                        else: song.change(key, old_value, new_value)
+                        if old_value is None:
+                            song.add(key, new_value)
+                        else:
+                            song.change(key, old_value, new_value)
                         changed = True
             for key, values in added.iteritems():
                 for value in values:
@@ -638,24 +729,28 @@ class EditTags(gtk.VBox):
                 song.add(tag, value)
 
             if changed:
-                try: song.write()
+                try:
+                    song.write()
                 except:
+                    util.print_exc()
                     qltk.ErrorMessage(
                         self, _("Unable to save song"),
                         _("Saving <b>%s</b> failed. The file "
                           "may be read-only, corrupted, or you "
-                          "do not have permission to edit it.")%(
+                          "do not have permission to edit it.") % (
                         util.escape(util.fsdecode(
                         song('~basename'))))).run()
                     library.reload(song, changed=was_changed)
                     break
                 was_changed.append(song)
 
-            if win.step(): break
+            if win.step():
+                break
 
         win.destroy()
         library.changed(was_changed)
-        for b in [save, revert]: b.set_sensitive(False)
+        for b in [save, revert]:
+            b.set_sensitive(False)
 
     def __edit_tag(self, renderer, path, new_value, model):
         new_value = ', '.join(new_value.splitlines())
@@ -665,10 +760,11 @@ class EditTags(gtk.VBox):
             if not fmt.is_valid(new_value):
                 qltk.WarningMessage(
                     self, _("Invalid value"),
-                    _("Invalid value: <b>%(value)s</b>\n\n%(error)s") %{
+                    _("Invalid value: <b>%(value)s</b>\n\n%(error)s") % {
                     "value": new_value, "error": fmt.error}).run()
                 return
-            else: new_value = fmt.validate(new_value)
+            else:
+                new_value = fmt.validate(new_value)
         tag = self.__songinfo.get(row[TAG], None)
         if row[VALUE].split('<')[0] != new_value or (
                 tag and tag.shared and not tag.complete):
@@ -702,13 +798,12 @@ class EditTags(gtk.VBox):
                 if not fmt.is_valid(v):
                     qltk.WarningMessage(
                         self, _("Invalid value"),
-                        _("Invalid value: <b>%(value)s</b>\n\n%(error)s") %{
+                        _("Invalid value: <b>%(value)s</b>\n\n%(error)s") % {
                         "value": row[VALUE], "error": fmt.error}).run()
                     return
                 value = fmt.validate(v)
             else:
                 value = row[VALUE]
-                idx = value.find('<i>')
                 value = util.unescape(value)
 
             if row[ORIGVALUE] is None:
@@ -725,28 +820,35 @@ class EditTags(gtk.VBox):
                 row[TAG] = new_tag
 
     def __button_press(self, view, event):
-        if event.button not in [1, 2]: return False
+        if event.button not in [1, 2]:
+            return False
         x, y = map(int, [event.x, event.y])
-        try: path, col, cellx, celly = view.get_path_at_pos(x, y)
-        except TypeError: return False
+        try:
+            path, col, cellx, celly = view.get_path_at_pos(x, y)
+        except TypeError:
+            return False
 
         if event.button == 1 and col is view.get_columns()[0]:
             row = view.get_model()[path]
             row[EDITED] = not row[EDITED]
             if row[EDITED]:
                 idx = row[VALUE].find('<i>')
-                if idx >= 0: row[VALUE] = row[VALUE][:idx].strip()
+                if idx >= 0:
+                    row[VALUE] = row[VALUE][:idx].strip()
             return True
         elif event.button == 2 and col == view.get_columns()[2]:
-            display = gtk.gdk.display_manager_get().get_default_display()
+            display = Gdk.display_manager_get().get_default_display()
             clipboardname = ["PRIMARY", "CLIPBOARD"][sys.platform == "win32"]
-            clipboard = gtk.Clipboard(display, clipboardname)
+            clipboard = Gtk.Clipboard(display, clipboardname)
             for rend in col.get_cell_renderers():
                 if rend.get_property('editable'):
-                    clipboard.request_text(self.__paste, (rend, path[0]))
+                    clipboard.request_text(self.__paste,
+                                           (rend, path.get_indices()[0]))
                     return True
-            else: return False
-        else: return False
+            else:
+                return False
+        else:
+            return False
 
     def __update(self, songs, view, buttonbox, model, add, buttons):
         if songs is None:
@@ -764,9 +866,12 @@ class EditTags(gtk.VBox):
 
         # reverse order here so insertion puts them in proper order.
         for tag in ['album', 'artist', 'title']:
-            try: keys.remove(tag)
-            except ValueError: pass
-            else: keys.insert(0, tag)
+            try:
+                keys.remove(tag)
+            except ValueError:
+                pass
+            else:
+                keys.insert(0, tag)
 
         for tag in keys:
             # Handle with care.
@@ -787,7 +892,8 @@ class EditTags(gtk.VBox):
                                       orig_value[i], renamed, newtag])
 
         buttonbox.set_sensitive(bool(songinfo.can_change()))
-        for b in buttons: b.set_sensitive(False)
+        for b in buttons:
+            b.set_sensitive(False)
         add.set_sensitive(bool(songs))
 
     def __value_editing_started(self, render, editable, path, model, library):
@@ -798,14 +904,14 @@ class EditTags(gtk.VBox):
                 editable.set_completion(completion)
         except AttributeError:
             pass
-        if isinstance(editable, gtk.Entry):
+        if isinstance(editable, Gtk.Entry):
             editable.set_text(util.unescape(model[path][VALUE].split('<')[0]))
 
     def __tag_editing_started(self, render, editable, path, model, library):
         try:
             if not editable.get_completion():
                 tags = self.__songinfo.can_change()
-                if tags == True:
+                if tags is True:
                     from quodlibet.formats import USEFUL_TAGS as tags
                 completion = qltk.EntryCompletion(tags)
                 editable.set_completion(completion)
