@@ -1,31 +1,25 @@
-# Copyright 2005-2011 Joe Wreschnig, Michael Urman, Christoph Reiter
+# Copyright 2005 Joe Wreschnig, Michael Urman
+#           2011, 2012 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import gtk
-import gobject
+from gi.repository import Gtk, GObject, Gdk, Gio
 
-# We use sexy.IconEntry with older GTK+, but ComboboxEntry only allows
-# gtk.Entry (it kind of works, but alignment is wrong and there are lots of
-# warnings), so also provide all entries without libsexy.
+from quodlibet.qltk.x import is_accel
+from quodlibet.qltk.x import SeparatorMenuItem
 
-IconEntry = gtk.Entry
-if not getattr(gtk.Entry, "set_icon_from_stock", None):
-    try: from sexy import IconEntry
-    except ImportError: pass
-
-from quodlibet import config
-from quodlibet.qltk.x import ClearButton, is_accel
 
 class EditableUndo(object):
     """A simple undo/redo implementation for gtk widgets that
     support the gtk.Editable interface"""
 
     def set_undo(self, val):
-        if val: self.__enable_undo()
-        else: self.__disable_undo()
+        if val:
+            self.__enable_undo()
+        else:
+            self.__disable_undo()
 
     def reset_undo(self):
         self.__history = []
@@ -76,11 +70,11 @@ class EditableUndo(object):
         del self.__del_pos
 
     def __popup(self, entry, menu):
-        undo =  gtk.ImageMenuItem(gtk.STOCK_UNDO)
-        redo =  gtk.ImageMenuItem(gtk.STOCK_REDO)
-        sep = gtk.SeparatorMenuItem()
+        undo = Gtk.ImageMenuItem(Gtk.STOCK_UNDO, use_stock=True)
+        redo = Gtk.ImageMenuItem(Gtk.STOCK_REDO, use_stock=True)
+        sep = SeparatorMenuItem()
 
-        map(gtk.Widget.show, (sep, redo, undo))
+        map(Gtk.Widget.show, (sep, redo, undo))
 
         undo.connect('activate', lambda *x: self.undo())
         redo.connect('activate', lambda *x: self.redo())
@@ -131,7 +125,8 @@ class EditableUndo(object):
         return bool(source)
 
     def __do(self, source, target):
-        if not self.__can_do(source): return
+        if not self.__can_do(source):
+            return
         self.__del_pos = self.__in_pos = -1
         self.__inhibit()
         now = self.__all()
@@ -144,45 +139,41 @@ class EditableUndo(object):
         self.set_position(pos)
         self.__uninhibit()
 
-class UndoEntry(IconEntry, EditableUndo):
-    def __init__(self, *args, **kwargs):
-        super(UndoEntry, self).__init__(*args, **kwargs)
-        self.set_undo(True)
 
-    def set_text(self, *args, **kwargs):
-        super(UndoEntry, self).set_text(*args, **kwargs)
-        self.reset_undo()
+class Entry(Gtk.Entry):
+    def do_get_preferred_width(self):
+        # 150 min width since GTK+3.2 is way too much
+        minimum, natural = Gtk.Entry.do_get_preferred_width(self)
+        return (30, natural)
 
-class UndoNoSexyEntry(gtk.Entry, EditableUndo):
+
+class UndoEntry(Entry, EditableUndo):
     def __init__(self, *args):
-        super(UndoNoSexyEntry, self).__init__(*args)
+        super(UndoEntry, self).__init__(*args)
         self.set_undo(True)
 
     def set_text(self, *args):
-        super(UndoNoSexyEntry, self).set_text(*args)
+        super(UndoEntry, self).set_text(*args)
         self.reset_undo()
 
+
 class ClearEntryMixin(object):
-    """A clear icon mixin supporting newer gtk.Entry or sexy.IconEntry /
+    """A clear icon mixin supporting newer Gtk.Entry or
     a separate clear button as a fallback.
     """
 
-    __gsignals__ = {'clear': (
-        gobject.SIGNAL_RUN_LAST|gobject.SIGNAL_ACTION, gobject.TYPE_NONE, ())}
+    __gsignals__ = {
+        'clear': (GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION,
+                  None, ())
+    }
 
-    def pack_clear_button(self, container=None):
-        """Enables the clear icon in the entry. For older gtk+
-        versions and without libsexy, a clear button will be packed in
-        container.
-        """
-        if getattr(self, "set_icon_from_stock", None):
-            self.set_icon_from_stock(
-                gtk.ENTRY_ICON_SECONDARY, gtk.STOCK_CLEAR)
-            self.connect("icon-release", self.__clear)
-        elif container:
-            button = ClearButton(self)
-            container.pack_start(button, False)
-            button.connect('clicked', self.__clear)
+    def enable_clear_button(self):
+        """Enables the clear icon in the entry"""
+
+        gicon = Gio.ThemedIcon.new_from_names(
+            ["edit-clear-symbolic", "edit-clear"])
+        self.set_icon_from_gicon(Gtk.EntryIconPosition.SECONDARY, gicon)
+        self.connect("icon-release", self.__clear)
 
     def __clear(self, button, *args):
         # TODO: don't change the order.. we connect to clear and remove all
@@ -190,43 +181,47 @@ class ClearEntryMixin(object):
         self.delete_text(0, -1)
         self.emit('clear')
 
+
 class ClearEntry(UndoEntry, ClearEntryMixin):
     __gsignals__ = ClearEntryMixin.__gsignals__
-class ClearNoSexyEntry(UndoNoSexyEntry, ClearEntryMixin):
-    __gsignals__ = ClearEntryMixin.__gsignals__
+
 
 class ValidatingEntryMixin(object):
     """An entry with visual feedback as to whether it is valid or not.
     The given validator function gets a string and returns True (green),
-    False (red), or a color string, or None (black).
+    False (red), or None (black).
 
     parse.Query.is_valid_color mimicks the behavior of the search bar.
 
     If the "Color search terms" option is off, the entry will not
     change color."""
 
+    INVALID = Gdk.RGBA(0.8, 0, 0)
+    VALID = Gdk.RGBA(0.3, 0.6, 0.023)
+
     def set_validate(self, validator=None):
-        if validator: self.connect_object('changed', self.__color, validator)
+        if validator:
+            self.connect_object('changed', self.__color, validator)
 
     def __color(self, validator):
-        if config.getboolean('browsers', 'color'):
-            value = validator(self.get_text())
-            if value is True: color = "dark green"
-            elif value is False: color = "red"
-            elif isinstance(value, str): color = value
-            else: color = None
-
-            if color and self.get_property('sensitive'):
-                self.modify_text(gtk.STATE_NORMAL, gtk.gdk.color_parse(color))
+        value = validator(self.get_text())
+        if value is True:
+            color = self.VALID
+        elif value is False:
+            color = self.INVALID
+        elif value and isinstance(value, str):
+            color = Gdk.RGBA()
+            color.parse(value)
         else:
-            self.modify_text(gtk.STATE_NORMAL, None)
+            color = None
+
+        if color and self.get_property('sensitive'):
+            self.override_color(Gtk.StateType.NORMAL, color)
+        else:
+            self.override_color(Gtk.StateType.NORMAL, None)
+
 
 class ValidatingEntry(ClearEntry, ValidatingEntryMixin):
     def __init__(self, validator=None, *args):
         super(ValidatingEntry, self).__init__(*args)
         self.set_validate(validator)
-
-class ValidatingNoSexyEntry(ClearNoSexyEntry, ValidatingEntryMixin):
-    def __init__(self, validator=None, *args):
-        super(ValidatingNoSexyEntry, self).__init__(*args)
-        self.set_validate(validator)
