@@ -9,8 +9,7 @@ import os
 import urlparse
 import errno
 
-import gobject
-import gtk
+from gi.repository import Gtk, GObject, Gdk
 
 from quodlibet import const
 from quodlibet import formats
@@ -19,6 +18,9 @@ from quodlibet import util
 
 from quodlibet.qltk.getstring import GetStringDialog
 from quodlibet.qltk.views import AllTreeView, RCMTreeView, MultiDragTreeView
+from quodlibet.qltk.views import TreeViewColumn
+from quodlibet.qltk.x import ScrolledWindow
+
 
 def search_func(model, column, key, iter, handledirs):
     check = model.get_value(iter, 0)
@@ -28,6 +30,7 @@ def search_func(model, column, key, iter, handledirs):
         check = os.path.basename(check) or os.sep
     return key not in check.lower() and key not in check
 
+
 def filesel_filter(filename):
     IMAGES = [".jpg", ".png", ".jpeg"]
     if formats.filter(filename):
@@ -38,8 +41,9 @@ def filesel_filter(filename):
                 return True
     return False
 
+
 class DirectoryTree(RCMTreeView, MultiDragTreeView):
-    def cell_data(column, cell, model, iter):
+    def cell_data(column, cell, model, iter, userdata):
         value = model[iter][0]
         if value is not None:
             cell.set_property('text', util.fsdecode(
@@ -47,17 +51,17 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
     cell_data = staticmethod(cell_data)
 
     def __init__(self, initial=None, folders=None):
-        super(DirectoryTree, self).__init__(gtk.TreeStore(str))
-        column = gtk.TreeViewColumn(_("Folders"))
-        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
-        render = gtk.CellRendererPixbuf()
-        render.set_property('stock_id', gtk.STOCK_DIRECTORY)
-        column.pack_start(render, expand=False)
-        render = gtk.CellRendererText()
-        column.pack_start(render)
+        super(DirectoryTree, self).__init__(Gtk.TreeStore(str))
+        column = TreeViewColumn(_("Folders"))
+        column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
+        render = Gtk.CellRendererPixbuf()
+        render.set_property('stock_id', Gtk.STOCK_DIRECTORY)
+        column.pack_start(render, False)
+        render = Gtk.CellRendererText()
+        column.pack_start(render, True)
         column.set_cell_data_func(render, self.cell_data)
 
-        column.set_attributes(render, text=0)
+        column.add_attribute(render, "text", 0)
         self.append_column(column)
         self.set_search_equal_func(search_func, True)
 
@@ -68,9 +72,11 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
             try:
                 from win32com.shell import shell, shellcon as con
                 import pywintypes
-            except ImportError: pass
+            except ImportError:
+                pass
             else:
-                if folders: folders.append(None)
+                if folders:
+                    folders.append(None)
 
                 try:
                     desktop = shell.SHGetFolderPath(0, con.CSIDL_DESKTOP, 0, 0)
@@ -88,16 +94,20 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
                 else:
                     folders.append(music)
 
-            if folders: folders.append(None)
+            if folders:
+                folders.append(None)
             drives = [letter + ":\\" for letter in "CDEFGHIJKLMNOPQRSTUVWXYZ"]
             map(folders.append, filter(os.path.isdir, drives))
         else:
-            if folders: folders.append(None)
+            if folders:
+                folders.append(None)
             folders.extend([const.HOME, "/"])
 
         # Read in the GTK bookmarks list; gjc says this is the right way
-        try: f = file(os.path.join(const.HOME, ".gtk-bookmarks"))
-        except EnvironmentError: pass
+        try:
+            f = file(os.path.join(const.HOME, ".gtk-bookmarks"))
+        except EnvironmentError:
+            pass
         else:
             folders.append(None)
             for line in (l for l in f.readlines() if l.strip()):
@@ -114,24 +124,26 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
             niter = self.get_model().append(None, [path])
             if path is not None:
                 self.get_model().append(niter, ["dummy"])
-        self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+        self.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
         self.connect(
             'test-expand-row', DirectoryTree.__expanded, self.get_model())
-        self.set_row_separator_func(lambda model, iter: model[iter][0] is None)
+        self.set_row_separator_func(
+            lambda model, iter, data: model[iter][0] is None, None)
 
-        if initial: self.go_to(initial)
+        if initial:
+            self.go_to(initial)
 
-        menu = gtk.Menu()
-        m = qltk.MenuItem(_("_New Folder..."), gtk.STOCK_NEW)
+        menu = Gtk.Menu()
+        m = qltk.MenuItem(_("_New Folder..."), Gtk.STOCK_NEW)
         m.connect('activate', self.__mkdir)
         menu.append(m)
-        m = gtk.ImageMenuItem(gtk.STOCK_DELETE)
+        m = Gtk.ImageMenuItem(Gtk.STOCK_DELETE, use_stock=True)
         m.connect('activate', self.__rmdir)
         menu.append(m)
-        m = gtk.ImageMenuItem(gtk.STOCK_REFRESH)
+        m = Gtk.ImageMenuItem(Gtk.STOCK_REFRESH, use_stock=True)
         m.connect('activate', self.__refresh)
         menu.append(m)
-        m = qltk.MenuItem(_("_Select All Subfolders"), gtk.STOCK_DIRECTORY)
+        m = qltk.MenuItem(_("_Select All Subfolders"), Gtk.STOCK_DIRECTORY)
         m.connect('activate', self.__expand)
         menu.append(m)
         menu.show_all()
@@ -147,38 +159,49 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
                     return (not t.startswith(".") and
                             os.access(joined, os.X_OK) and
                             os.path.isdir(joined))
-                try: dirs = filter(isvisibledir,
-                                   sorted(os.listdir(util.fsnative(head))))
-                except OSError: break
-                try: path.insert(0, dirs.index(tail))
-                except ValueError: break
+                try:
+                    dirs = filter(isvisibledir,
+                                  sorted(os.listdir(util.fsnative(head))))
+                except OSError:
+                    break
+                try:
+                    path.insert(0, dirs.index(tail))
+                except ValueError:
+                    break
             head, tail = os.path.split(head)
 
         if initial.startswith(const.HOME):
             path.insert(0, 0)
-        else: path.insert(0, 1)
+        else:
+            path.insert(0, 1)
         for i in range(len(path)):
-            self.expand_row(tuple(path[:i+1]), False)
+            self.expand_row(Gtk.TreePath(tuple(path[:i + 1])), False)
         self.get_selection().select_path(tuple(path))
         self.scroll_to_cell(tuple(path))
 
     def __popup_menu(self, menu):
-        try: model, (path,) = self.get_selection().get_selected_rows()
-        except ValueError: return True
+        try:
+            model, (path,) = self.get_selection().get_selected_rows()
+        except ValueError:
+            return True
         directory = model[path][0]
         delete = menu.get_children()[1]
-        try: delete.set_sensitive(len(os.listdir(util.fsnative(directory))) == 0)
+        try:
+            delete.set_sensitive(
+                len(os.listdir(util.fsnative(directory))) == 0)
         except OSError, err:
-            if err.errno == errno.ENOENT: model.remove(model.get_iter(path))
+            if err.errno == errno.ENOENT:
+                model.remove(model.get_iter(path))
         else:
             selection = self.get_selection()
             selection.unselect_all()
             selection.select_path(path)
-            return self.popup_menu(menu, 0, gtk.get_current_event_time())
+            return self.popup_menu(menu, 0, Gtk.get_current_event_time())
 
     def __mkdir(self, button):
         model, rows = self.get_selection().get_selected_rows()
-        if len(rows) != 1: return
+        if len(rows) != 1:
+            return
 
         row = rows[0]
         directory = model[row][0]
@@ -188,7 +211,8 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
         if dir:
             dir = util.fsnative(dir.decode('utf-8'))
             fullpath = os.path.realpath(os.path.join(directory, dir))
-            try: os.makedirs(fullpath)
+            try:
+                os.makedirs(fullpath)
             except EnvironmentError, err:
                 error = "<b>%s</b>: %s" % (err.filename, err.strerror)
                 qltk.ErrorMessage(
@@ -199,9 +223,11 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
 
     def __rmdir(self, button):
         model, rows = self.get_selection().get_selected_rows()
-        if len(rows) != 1: return
+        if len(rows) != 1:
+            return
         directory = model[rows[0]][0]
-        try: os.rmdir(directory)
+        try:
+            os.rmdir(directory)
         except EnvironmentError, err:
             error = "<b>%s</b>: %s" % (err.filename, err.strerror)
             qltk.ErrorMessage(
@@ -210,7 +236,8 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
             prow = rows[0][:-1]
             expanded = self.row_expanded(prow)
             self.emit('test-expand-row', model.get_iter(prow), prow)
-            if expanded: self.expand_row(prow, False)
+            if expanded:
+                self.expand_row(prow, False)
 
     def __expand(self, button):
         selection = self.get_selection()
@@ -233,7 +260,8 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
     def __refresh(self, button):
         model, rows = self.get_selection().get_selected_rows()
         expanded = set()
-        self.map_expanded_rows(lambda s, iter: expanded.add(model[iter][0]))
+        self.map_expanded_rows(
+            lambda s, iter, data: expanded.add(model[iter][0]), None)
         needs_expanding = []
         for row in rows:
             if self.row_expanded(row):
@@ -251,10 +279,10 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
                 child = model.iter_next(child)
 
     def __expanded(self, iter, path, model):
-        window = self.window
+        window = self.get_window()
         if window:
-            window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
-            gtk.main_iteration(block=False)
+            window.set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
+            Gtk.main_iteration_do(False)
         try:
             try:
                 if model is None:
@@ -264,7 +292,8 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
                 folder = model[iter][0]
                 for path in util.listdir(folder):
                     try:
-                        if not os.path.isdir(path): continue
+                        if not os.path.isdir(path):
+                            continue
                         for filename in util.listdir(path):
                             if os.path.isdir(filename):
                                 niter = model.append(iter, [path])
@@ -282,16 +311,18 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
             if window:
                 window.set_cursor(None)
 
-class FileSelector(gtk.VPaned):
-    def cell_data(column, cell, model, iter):
+
+class FileSelector(Gtk.VPaned):
+    def cell_data(column, cell, model, iter, userdata):
         value = model[iter][0]
         if value is not None:
             cell.set_property('text', util.fsdecode(os.path.basename(value)))
     cell_data = staticmethod(cell_data)
 
-    __gsignals__ = { 'changed': (gobject.SIGNAL_RUN_LAST,
-                                 gobject.TYPE_NONE, (gtk.TreeSelection,))
-                     }
+    __gsignals__ = {
+        'changed': (GObject.SignalFlags.RUN_LAST, None,
+                    (Gtk.TreeSelection,))
+    }
 
     def __init__(self, initial=None, filter=filesel_filter, folders=None):
         super(FileSelector, self).__init__()
@@ -300,19 +331,20 @@ class FileSelector(gtk.VPaned):
         if initial and os.path.isfile(initial):
             initial = os.path.dirname(initial)
         dirlist = DirectoryTree(initial, folders=folders)
-        filelist = AllTreeView(gtk.ListStore(str))
-        column = gtk.TreeViewColumn(_("Songs"))
-        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
-        render = gtk.CellRendererPixbuf()
-        render.set_property('stock_id', gtk.STOCK_FILE)
-        column.pack_start(render, expand=False)
-        render = gtk.CellRendererText()
-        column.pack_start(render)
+        filelist = AllTreeView(Gtk.ListStore(str))
+        column = TreeViewColumn(_("Songs"))
+        column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
+        render = Gtk.CellRendererPixbuf()
+        render.set_property('stock_id', Gtk.STOCK_FILE)
+        render.props.xpad = 3
+        column.pack_start(render, False)
+        render = Gtk.CellRendererText()
+        column.pack_start(render, True)
         column.set_cell_data_func(render, self.cell_data)
-        column.set_attributes(render, text=0)
+        column.add_attribute(render, "text", 0)
         filelist.append_column(column)
         filelist.set_rules_hint(True)
-        filelist.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+        filelist.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
         filelist.set_search_equal_func(search_func, False)
 
         self.__sig = filelist.get_selection().connect(
@@ -321,26 +353,27 @@ class FileSelector(gtk.VPaned):
         dirlist.get_selection().connect(
             'changed', self.__fill, filelist)
         dirlist.get_selection().emit('changed')
+
         def select_all_files(view, path, col, fileselection):
             view.expand_row(path, False)
             fileselection.select_all()
         dirlist.connect('row-activated', select_all_files,
             filelist.get_selection())
 
-        sw = gtk.ScrolledWindow()
+        sw = ScrolledWindow()
         sw.add(dirlist)
-        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        sw.set_shadow_type(gtk.SHADOW_IN)
+        sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+        sw.set_shadow_type(Gtk.ShadowType.IN)
         self.pack1(sw, resize=True)
 
-        sw = gtk.ScrolledWindow()
+        sw = ScrolledWindow()
         sw.add(filelist)
-        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        sw.set_shadow_type(gtk.SHADOW_IN)
+        sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
+        sw.set_shadow_type(Gtk.ShadowType.IN)
         self.pack2(sw, resize=True)
 
     def rescan(self, *args):
-        self.get_child1().child.get_selection().emit('changed')
+        self.get_child1().get_child().get_selection().emit('changed')
 
     def __changed(self, selection):
         self.emit('changed', selection)
