@@ -13,28 +13,28 @@ import os
 import threading
 import urllib
 
-import gobject
-import gtk
+from gi.repository import Gtk, GLib
 
 from quodlibet import const
 from quodlibet import qltk
 from quodlibet import util
 
-class LyricsPane(gtk.VBox):
+
+class LyricsPane(Gtk.VBox):
     def __init__(self, song):
         # Commented code in this method is due to Lyric Wiki's disappearance.
         # See issue 273.
         super(LyricsPane, self).__init__(spacing=12)
         self.set_border_width(12)
-        view = gtk.TextView()
-        sw = gtk.ScrolledWindow()
+        view = Gtk.TextView()
+        sw = Gtk.ScrolledWindow()
         sw.add(view)
-        refresh = qltk.Button(_("_Download"), gtk.STOCK_CONNECT)
-        save = gtk.Button(stock=gtk.STOCK_SAVE)
-        delete = gtk.Button(stock=gtk.STOCK_DELETE)
-        add = gtk.Button(stock=gtk.STOCK_EDIT)
-        view.set_wrap_mode(gtk.WRAP_WORD)
-        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        refresh = qltk.Button(_("_Download"), Gtk.STOCK_CONNECT)
+        save = Gtk.Button(stock=Gtk.STOCK_SAVE)
+        delete = Gtk.Button(stock=Gtk.STOCK_DELETE)
+        add = Gtk.Button(stock=Gtk.STOCK_EDIT)
+        view.set_wrap_mode(Gtk.WrapMode.WORD)
+        sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
 
         lyricname = song.lyric_filename
         buffer = view.get_buffer()
@@ -44,18 +44,15 @@ class LyricsPane(gtk.VBox):
         delete.connect('clicked', self.__delete, lyricname, save)
         add.connect('clicked', self.__add, song)
 
-        sw.set_shadow_type(gtk.SHADOW_IN)
-        self.pack_start(sw, expand=True)
+        sw.set_shadow_type(Gtk.ShadowType.IN)
+        self.pack_start(sw, True, True, 0)
 
-        #self.pack_start(gtk.Label(_("Lyrics provided by %s.") %(
-        #    "http://lyricwiki.org")), expand=False)
-
-        bbox = gtk.HButtonBox()
-        bbox.pack_start(save)
-        bbox.pack_start(delete)
-        #bbox.pack_start(refresh)
-        bbox.pack_start(add)
-        self.pack_start(bbox, expand=False)
+        bbox = Gtk.HButtonBox()
+        bbox.pack_start(save, True, True, 0)
+        bbox.pack_start(delete, True, True, 0)
+        #bbox.pack_start(refresh, True, True, 0)
+        bbox.pack_start(add, True, True, 0)
+        self.pack_start(bbox, False, True, 0)
 
         save.set_sensitive(False)
         add.set_sensitive(True)
@@ -72,8 +69,6 @@ class LyricsPane(gtk.VBox):
 
     def __add(self, add, song):
         artist = song.comma('artist').encode('utf-8')
-        title = song.comma('title').encode('utf-8')
-        album = song.comma('album').encode('utf-8')
 
         util.website("http://lyricwiki.org/%s" % (urllib.quote(artist)))
 
@@ -92,45 +87,54 @@ class LyricsPane(gtk.VBox):
         try:
             sock = urllib.urlopen(
                 "http://lyricwiki.org/api.php?"
-                "client=QuodLibet&func=getSong&artist=%s&song=%s&fmt=text"%(
+                "client=QuodLibet&func=getSong&artist=%s&song=%s&fmt=text" % (
                 urllib.quote(artist.encode('utf-8')),
                 urllib.quote(title.encode('utf-8'))))
             text = sock.read()
         except Exception, err:
-            try: err = err.strerror.decode(const.ENCODING, 'replace')
-            except: err = _("Unable to download lyrics.")
-            gobject.idle_add(buffer.set_text, err)
+            try:
+                err = err.strerror.decode(const.ENCODING, 'replace')
+            except:
+                err = _("Unable to download lyrics.")
+            GLib.idle_add(buffer.set_text, err)
             return
 
         sock.close()
 
         if text == 'Not found':
-            gobject.idle_add(
+            GLib.idle_add(
                 buffer.set_text, _("No lyrics found for this song."))
             return
         else:
-            gobject.idle_add(buffer.set_text, text)
-            gobject.idle_add(refresh.set_sensitive, True)
+            GLib.idle_add(buffer.set_text, text)
+            GLib.idle_add(refresh.set_sensitive, True)
 
     def __save(self, save, lyricname, buffer, delete):
-        try: os.makedirs(os.path.dirname(lyricname))
-        except EnvironmentError, err: pass
+        try:
+            os.makedirs(os.path.dirname(lyricname))
+        except EnvironmentError, err:
+            pass
 
-        try: f = file(lyricname, "w")
+        try:
+            f = file(lyricname, "w")
         except EnvironmentError, err:
             print_w(err.strerror.decode(const.ENCODING, "replace"))
         else:
             start, end = buffer.get_bounds()
-            f.write(buffer.get_text(start, end))
+            f.write(buffer.get_text(start, end, True))
             f.close()
         delete.set_sensitive(True)
         save.set_sensitive(False)
 
     def __delete(self, delete, lyricname, save):
-        try: os.unlink(lyricname)
-        except EnvironmentError: pass
+        try:
+            os.unlink(lyricname)
+        except EnvironmentError:
+            pass
         lyricname = os.path.dirname(lyricname)
-        try: os.rmdir(lyricname)
-        except EnvironmentError: pass
+        try:
+            os.rmdir(lyricname)
+        except EnvironmentError:
+            pass
         delete.set_sensitive(False)
         save.set_sensitive(True)
