@@ -27,14 +27,12 @@ Of course, right now it does none of these things.
 # TODO: Add notification button/callback support (prereq for global undo)
 # TODO: Optimize performance (deferred signals, etc)
 
-import gobject
-import gtk
-import pango
-import traceback
+from gi.repository import Gtk, GLib, Pango
 
 from quodlibet.util import copool
 
-SIZE = gtk.ICON_SIZE_MENU
+SIZE = Gtk.IconSize.MENU
+
 
 class ParentProperty(object):
     """
@@ -44,12 +42,14 @@ class ParentProperty(object):
     """
     def __get__(self, inst, owner):
         return getattr(inst, '_parent', None)
+
     def __set__(self, inst, value):
         if getattr(inst, '_parent', None) is not None and value is not None:
             raise ValueError("Cannot set parent property without first "
                     "setting it to 'None'.")
         inst._parent = value
 
+
 class Task(object):
     def __init__(self, source, desc, known_length=True, controller=None,
                  pause=None, stop=None):
@@ -110,7 +110,7 @@ class Task(object):
         try:
             if hasattr(gen, '__len__'):
                 for i, x in enumerate(gen):
-                    self.update(float(i)/len(gen))
+                    self.update(float(i) / len(gen))
                     yield x
             else:
                 for x in gen:
@@ -151,6 +151,7 @@ class Task(object):
         self.finish()
         return False
 
+
 class TaskController(object):
     """
     Controller logic for displaying and managing a list of Tasks. Also
@@ -192,9 +193,11 @@ class TaskController(object):
         for t in self.active_tasks:
             if t.pausable:
                 t.paused = val
+
     def __get_paused(self):
         pausable = [t for t in self.active_tasks if t.pausable]
-        if not pausable: return False
+        if not pausable:
+            return False
         return not [t for t in pausable if not t.paused]
     paused = property(__get_paused, __set_paused)
 
@@ -221,28 +224,29 @@ class TaskController(object):
 # Oh so deliciously hacky.
 TaskController.default_instance = TaskController()
 
-class TaskWidget(gtk.HBox):
+
+class TaskWidget(Gtk.HBox):
     """
     Displays a task.
     """
     def __init__(self, task):
-        super(TaskWidget, self).__init__()
+        super(TaskWidget, self).__init__(spacing=2)
         self.task = task
-        self.label = gtk.Label()
+        self.label = Gtk.Label()
         self.label.set_alignment(1.0, 0.5)
-        self.label.set_ellipsize(pango.ELLIPSIZE_END)
-        self.pack_start(self.label, padding=12)
-        self.progress = gtk.ProgressBar()
-        self.progress.set_size_request(200, -1)
-        self.pack_start(self.progress, expand=False)
-        self.pause = gtk.ToggleButton()
-        self.pause.add(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PAUSE, SIZE))
+        self.label.set_ellipsize(Pango.EllipsizeMode.END)
+        self.pack_start(self.label, True, True, 12)
+        self.progress = Gtk.ProgressBar()
+        self.progress.set_size_request(100, -1)
+        self.pack_start(self.progress, True, True, 0)
+        self.pause = Gtk.ToggleButton()
+        self.pause.add(Gtk.Image.new_from_stock(Gtk.STOCK_MEDIA_PAUSE, SIZE))
         self.pause.connect('toggled', self.__pause_toggled)
-        self.pack_start(self.pause, expand=False)
-        self.stop = gtk.Button()
-        self.stop.add(gtk.image_new_from_stock(gtk.STOCK_MEDIA_STOP, SIZE))
+        self.pack_start(self.pause, False, True, 0)
+        self.stop = Gtk.Button()
+        self.stop.add(Gtk.Image.new_from_stock(Gtk.STOCK_MEDIA_STOP, SIZE))
         self.stop.connect('clicked', self.__stop_clicked)
-        self.pack_start(self.stop, expand=False)
+        self.pack_start(self.stop, False, True, 0)
 
     def __pause_toggled(self, btn):
         if self.task.pausable:
@@ -268,7 +272,8 @@ class TaskWidget(gtk.HBox):
         if self.stop.props.sensitive != self.task.stoppable:
             self.stop.props.sensitive = self.task.stoppable
 
-class StatusBar(gtk.HBox):
+
+class StatusBar(Gtk.HBox):
     def __init__(self, task_controller):
         super(StatusBar, self).__init__()
         self.__dirty = False
@@ -276,26 +281,31 @@ class StatusBar(gtk.HBox):
         self.task_controller = task_controller
         self.task_controller.parent = self
 
-        self.default_label = gtk.Label()
+        self.default_label = Gtk.Label()
         self.default_label.set_alignment(1.0, 0.5)
         self.default_label.set_text(_("No time information"))
-        self.pack_start(self.default_label)
+        self.default_label.set_ellipsize(Pango.EllipsizeMode.END)
+        self.pack_start(self.default_label, True, True, 0)
         self.task_widget = TaskWidget(task_controller)
-        self.pack_start(self.task_widget)
+        self.pack_start(self.task_widget, True, True, 0)
         # The history button will eventually hold the full list of running
         # tasks, as well as the list of previous notifications.
-        #self.history_btn = gtk.Button(stock=gtk.STOCK_MISSING_IMAGE)
-        #self.pack_start(self.history_btn, expand=False)
+        #self.history_btn = Gtk.Button(stock=Gtk.STOCK_MISSING_IMAGE)
+        #self.pack_start(self.history_btn, False, True, 0)
 
         self.show_all()
         self.set_no_show_all(True)
         self.__set_shown('default')
 
     def __set_shown(self, type):
-        if type == 'default':   self.default_label.show()
-        else:                   self.default_label.hide()
-        if type == 'task':      self.task_widget.show()
-        else:                   self.task_widget.hide()
+        if type == 'default':
+            self.default_label.show()
+        else:
+            self.default_label.hide()
+        if type == 'task':
+            self.task_widget.show()
+        else:
+            self.task_widget.hide()
 
     def set_default_text(self, text):
         self.default_label.set_text(text)
@@ -311,5 +321,4 @@ class StatusBar(gtk.HBox):
     def update(self):
         if not self.__dirty:
             self.__dirty = True
-            gobject.idle_add(self.__update)
-
+            GLib.idle_add(self.__update)
