@@ -6,9 +6,11 @@
 
 import random
 
-import gtk
+from gi.repository import Gtk
 
 from quodlibet import config
+from quodlibet.plugins import PluginManager
+
 
 class Order(object):
     name = "unknown_order"
@@ -62,6 +64,7 @@ class Order(object):
     def reset(self, playlist):
         pass
 
+
 class OrderInOrder(Order):
     name = "inorder"
     display_name = _("In Order")
@@ -84,13 +87,15 @@ class OrderInOrder(Order):
         elif iter is None:
             return playlist[(len(playlist) - 1,)].iter
         else:
-            path = max(1, playlist.get_path(iter)[0])
-            try: return playlist.get_iter((path - 1,))
+            path = max(1, playlist.get_path(iter).get_indices()[0])
+            try:
+                return playlist.get_iter((path - 1,))
             except ValueError:
                 if playlist.repeat:
                     return playlist[(len(playlist) - 1,)].iter
         return None
 
+
 class OrderRemembered(Order):
     # Shared class for all the shuffle modes that keep a memory
     # of their previously played songs.
@@ -101,21 +106,25 @@ class OrderRemembered(Order):
 
     def next(self, playlist, iter):
         if iter is not None:
-            self._played.append(playlist.get_path(iter)[0])
+            self._played.append(playlist.get_path(iter).get_indices()[0])
 
     def previous(self, playlist, iter):
-        try: path = self._played.pop()
-        except IndexError: return None
-        else: return playlist.get_iter(path)
+        try:
+            path = self._played.pop()
+        except IndexError:
+            return None
+        else:
+            return playlist.get_iter(path)
 
     def set(self, playlist, iter):
         if iter is not None:
-            self._played.append(playlist.get_path(iter)[0])
+            self._played.append(playlist.get_path(iter).get_indices()[0])
         return iter
 
     def reset(self, playlist):
         del(self._played[:])
 
+
 class OrderShuffle(OrderRemembered):
     name = "shuffle"
     display_name = _("Shuffle")
@@ -137,6 +146,7 @@ class OrderShuffle(OrderRemembered):
             del(self._played[:])
             return None
 
+
 class OrderWeighted(OrderRemembered):
     name = "Weighted"
     display_name = _("Weighted")
@@ -156,6 +166,7 @@ class OrderWeighted(OrderRemembered):
         else:
             return playlist.get_iter_first()
 
+
 class OrderOneSong(OrderInOrder):
     name = "onesong"
     display_name = _("One Song")
@@ -169,6 +180,8 @@ class OrderOneSong(OrderInOrder):
             return None
 
 ORDERS = []
+
+
 def set_orders(orders):
     ORDERS[:] = [OrderInOrder, OrderShuffle, OrderWeighted, OrderOneSong]
     ORDERS.extend(orders)
@@ -176,22 +189,56 @@ def set_orders(orders):
                 cmp(K1.priority, K2.priority) or cmp(K1.name, K2.name))
 set_orders([])
 
-class PlayOrder(gtk.ComboBox):
+
+class PlayOrder(Gtk.ComboBoxText):
     def __init__(self, model, player):
-        super(PlayOrder, self).__init__(gtk.ListStore(str))
-        cell = gtk.CellRendererText()
+        super(PlayOrder, self).__init__()
+
+        cell = self.get_cells()[0]
         cell.props.xpad = 1
         cell.props.ypad = 0
-        self.pack_start(cell, True)
-        self.add_attribute(cell, 'text', 0)
-        for order in ORDERS:
-            self.append_text(order.display_name)
-        self.connect_object('changed', self.__changed_order, model, player)
+
+        self.__plugins = []
+        if PluginManager.instance:
+            PluginManager.instance.register_handler(self)
+
+        self.refresh()
+        self.__sig = self.connect_object(
+            'changed', self.__changed_order, model, player)
         self.set_active(config.get("memory", "order"))
+        self.emit("changed")
+
+    def plugin_handle(self, plugin):
+        from quodlibet.plugins.playorder import PlayOrderPlugin
+        return issubclass(plugin, PlayOrderPlugin)
+
+    def plugin_enable(self, plugin, obj):
+        if plugin.name is None:
+            plugin.name = plugin.PLUGIN_ID
+        if plugin.display_name is None:
+            plugin.display_name = plugin.PLUGIN_NAME
+        if plugin.accelerated_name is None:
+            plugin.accelerated_name = plugin.display_name
+
+        self.__plugins.append(plugin)
+        self.refresh()
+
+    def plugin_disable(self, plugin):
+        self.__plugins.remove(plugin)
+
+        # Don't safe changes from plugin changes
+        # so that disables on shutdown don't change the config.
+        if self.__sig:
+            self.handler_block(self.__sig)
+            self.refresh()
+            self.handler_unblock(self.__sig)
+        else:
+            self.refresh()
 
     def refresh(self):
         name = self.get_active_name()
         self.get_model().clear()
+        set_orders(self.__plugins)
         for order in ORDERS:
             self.append_text(order.display_name)
         if name:
@@ -200,20 +247,25 @@ class PlayOrder(gtk.ComboBox):
             self.set_active(ORDERS[0]).name
 
     def set_active(self, value):
-        try: value = ORDERS.index(value)
+        try:
+            value = ORDERS.index(value)
         except ValueError:
             if isinstance(value, str):
                 for i, Order in enumerate(ORDERS):
                     if Order.name.lower() == value.lower():
                         value = i
                         break
-        try: value = int(value)
-        except ValueError: value = 0
+        try:
+            value = int(value)
+        except ValueError:
+            value = 0
         super(PlayOrder, self).set_active(value)
 
     def get_active_name(self):
-        try: return ORDERS[self.get_active()].name
-        except IndexError: return ORDERS[0].name
+        try:
+            return ORDERS[self.get_active()].name
+        except IndexError:
+            return ORDERS[0].name
 
     def __changed_order(self, model, player):
         Order = ORDERS[self.get_active()]
