@@ -1,67 +1,79 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2005 Joe Wreschnig, Michael Urman, Iñigo Serna
+#                2012 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-import gobject
-import gtk
-import pango
+from gi.repository import Gtk, GObject, Pango
 
 from quodlibet import qltk
 from quodlibet import util
+from quodlibet import config
 
 from quodlibet.qltk.edittags import EditTags
 from quodlibet.qltk.renamefiles import RenameFiles
 from quodlibet.qltk.tagsfrompath import TagsFromPath
 from quodlibet.qltk.tracknumbers import TrackNumbers
 from quodlibet.qltk.views import HintedTreeView
+from quodlibet.qltk.window import PersistentWindowMixin
 
-class SongProperties(qltk.Window):
-    __gsignals__ = { 'changed': (gobject.SIGNAL_RUN_LAST,
-                                 gobject.TYPE_NONE, (object,))
-                     }
+
+class SongProperties(qltk.Window, PersistentWindowMixin):
+    __gsignals__ = {
+        'changed': (GObject.SignalFlags.RUN_LAST, None, (object,))
+    }
 
     def __init__(self, library, songs, parent=None):
         super(SongProperties, self).__init__(dialog=False)
         self.set_transient_for(qltk.get_top_parent(parent))
-        if len(songs) > 1: self.set_default_size(600, 400)
-        else: self.set_default_size(400, 400)
-        self.plugins.rescan()
 
-        paned = gtk.HPaned()
+        default_width = 600
+        config_suffix = ""
+        if len(songs) <= 1:
+            default_width -= 200
+            config_suffix += "single"
+        self.set_default_size(default_width, 400)
+
+        self.enable_window_tracking("quodlibet_properties",
+                                    size_suffix=config_suffix)
+
+        self.auto_save_on_change = config.getboolean(
+                'editing', 'auto_save_changes', False)
+
+        paned = Gtk.HPaned()
         notebook = qltk.Notebook()
         pages = []
         pages.extend([Ctr(self, library) for Ctr in
                       [EditTags, TagsFromPath, RenameFiles]])
         if len(songs) > 1:
             pages.append(TrackNumbers(self, library))
-        for page in pages: notebook.append_page(page)
+        for page in pages:
+            notebook.append_page(page)
         self.set_border_width(12)
 
-        fbasemodel = gtk.ListStore(object, str)
-        fmodel = gtk.TreeModelSort(fbasemodel)
+        fbasemodel = Gtk.ListStore(object, str)
+        fmodel = Gtk.TreeModelSort(model=fbasemodel)
         fview = HintedTreeView(fmodel)
         fview.connect('button-press-event', self.__pre_selection_changed)
         fview.set_rules_hint(True)
         selection = fview.get_selection()
-        selection.set_mode(gtk.SELECTION_MULTIPLE)
-        csig = selection.connect('changed', self.__selection_changed)
+        selection.set_mode(Gtk.SelectionMode.MULTIPLE)
         self.__save = None
 
         if len(songs) > 1:
-            render = gtk.CellRendererText()
-            c1 = gtk.TreeViewColumn(_('File'), render, text=1)
-            render.set_property('ellipsize', pango.ELLIPSIZE_END)
+            render = Gtk.CellRendererText()
+            c1 = Gtk.TreeViewColumn(_('File'), render, text=1)
+            render.set_property('ellipsize', Pango.EllipsizeMode.END)
             c1.set_sort_column_id(1)
             fview.append_column(c1)
-            sw = gtk.ScrolledWindow()
+            sw = Gtk.ScrolledWindow()
             sw.add(fview)
-            sw.set_shadow_type(gtk.SHADOW_IN)
-            sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+            sw.set_shadow_type(Gtk.ShadowType.IN)
+            sw.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
             sw.show_all()
-            paned.pack1(sw, shrink=True, resize=True)
+            paned.pack1(sw, shrink=False, resize=True)
 
         # Invisible selections behave a little strangely. So, when
         # handling this selection, there's a lot of if len(model) == 1
@@ -75,6 +87,7 @@ class SongProperties(qltk.Window):
         selection.select_all()
         paned.pack2(notebook, shrink=False, resize=True)
 
+        csig = selection.connect('changed', self.__selection_changed)
         s1 = library.connect(
             'changed', self.__refresh, fbasemodel, fview)
         s2 = library.connect(
@@ -98,8 +111,10 @@ class SongProperties(qltk.Window):
         # selection or we risk voiding edits on a selection that
         # doesn't include the removed songs.
         selection.handler_block(sig)
-        if len(model) == 1: rows = [(0,)]
-        else: rows = selection.get_selected_rows()[1]
+        if len(model) == 1:
+            rows = [Gtk.TreePath((0,))]
+        else:
+            rows = selection.get_selected_rows()[1]
         to_remove = []
         changed = False
         for row in model:
@@ -113,20 +128,24 @@ class SongProperties(qltk.Window):
 
     def __set_title(self, songs):
         if songs:
-            if len(songs) == 1: title = songs[0].comma("title")
+            if len(songs) == 1:
+                title = songs[0].comma("title")
             else:
                 title = ngettext(
                     "%(title)s and %(count)d more",
                     "%(title)s and %(count)d more",
-                    len(songs) - 1) % (
-                    {'title': songs[0].comma("title"), 'count': len(songs) - 1})
+                    len(songs) - 1) % {'title': songs[0].comma("title"),
+                                       'count': len(songs) - 1}
             self.set_title("%s - %s" % (title, _("Properties")))
-        else: self.set_title(_("Properties"))
+        else:
+            self.set_title(_("Properties"))
 
     def __refresh(self, library, songs, model, view):
         view.freeze_notify()
-        if len(model) == 1: rows = [(0,)]
-        else: rows = view.get_selection().get_selected_rows()[1]
+        if len(model) == 1:
+            rows = [Gtk.TreePath((0,))]
+        else:
+            rows = view.get_selection().get_selected_rows()[1]
         changed = False
         for row in model:
             song = row[0]
@@ -142,10 +161,16 @@ class SongProperties(qltk.Window):
 
     def __pre_selection_changed(self, view, event):
         if self.__save:
+            if self.auto_save_on_change:
+                self.__save.clicked()
+                return
             resp = qltk.CancelRevertSave(self).run()
-            if resp == gtk.RESPONSE_YES: self.__save.clicked()
-            elif resp == gtk.RESPONSE_NO: return False
-            else: return True # cancel or closed
+            if resp == Gtk.ResponseType.YES:
+                self.__save.clicked()
+            elif resp == Gtk.ResponseType.NO:
+                return False
+            else:
+                return True # cancel or closed
 
     def __selection_changed(self, selection):
         model = selection.get_tree_view().get_model()
