@@ -6,13 +6,12 @@
 # published by the Free Software Foundation
 
 import os
-import random
 import re
 import sys
 
-import gobject
-import gtk
+from gi.repository import GObject, Gdk, GLib
 
+import quodlibet
 from quodlibet import browsers
 from quodlibet import config
 from quodlibet import const
@@ -22,40 +21,51 @@ from quodlibet.util.uri import URI
 from quodlibet.qltk.browser import LibraryBrowser
 from quodlibet.qltk.properties import SongProperties
 from quodlibet.util import copool
+from quodlibet.util.library import scan_libary
+
 
 class FSInterface(object):
     """Provides a file in ~/.quodlibet to indicate what song is playing."""
     def __init__(self, player):
         player.connect('song-started', self.__started)
         player.connect('song-ended', self.__ended)
-        gtk.quit_add(1, self.__cleanup)
+        quodlibet.quit_add(1, self.destroy)
 
-    def __cleanup(self):
-        try: os.unlink(const.CURRENT)
-        except EnvironmentError: pass
+    def destroy(self):
+        try:
+            os.unlink(const.CURRENT)
+        except EnvironmentError:
+            pass
 
     def __started(self, player, song):
         if song:
-            try: f = file(const.CURRENT, "w")
-            except EnvironmentError: pass
+            try:
+                f = file(const.CURRENT, "w")
+            except EnvironmentError:
+                pass
             else:
                 f.write(song.to_dump())
                 f.close()
 
     def __ended(self, player, song, stopped):
-        try: os.unlink(const.CURRENT)
-        except EnvironmentError: pass
+        try:
+            os.unlink(const.CURRENT)
+        except EnvironmentError:
+            pass
+
 
 class FIFOControl(object):
     """A FIFO to control the player/library from."""
 
     def __init__(self, library, window, player):
         self.__open(library, window, player)
-        gtk.quit_add(1, self.__cleanup)
+        quodlibet.quit_add(1, self.destroy)
 
-    def __cleanup(self):
-        try: os.unlink(const.CONTROL)
-        except EnvironmentError: pass
+    def destroy(self):
+        try:
+            os.unlink(const.CONTROL)
+        except EnvironmentError:
+            pass
 
     def __open(self, *args):
         try:
@@ -64,71 +74,100 @@ class FIFOControl(object):
                 os.mkfifo(const.CONTROL, 0600)
             fifo = os.open(const.CONTROL, os.O_NONBLOCK)
             f = os.fdopen(fifo, "r", 4096)
-            gobject.io_add_watch(
-                f, gtk.gdk.INPUT_READ, self.__process, *args)
-        except (EnvironmentError, AttributeError): pass
+            GObject.io_add_watch(
+                f, GLib.IO_IN | GLib.IO_ERR | GLib.IO_HUP,
+                self.__process, *args)
+        except (EnvironmentError, AttributeError):
+            pass
 
     def __getitem__(self, key):
         key = key.replace("-", "_")
-        if key.startswith("_"): raise ValueError
+        if key.startswith("_"):
+            raise ValueError
         else:
-            try: return getattr(self, "_"+key)
-            except AttributeError: raise KeyError, key
+            try:
+                return getattr(self, "_" + key)
+            except AttributeError:
+                raise KeyError(key)
 
     def __process(self, source, condition, *args):
-        commands = source.read().rstrip("\n").splitlines()
-        if commands == []:
+        if condition in (GLib.IO_ERR, GLib.IO_HUP):
             self.__open(*args)
             return False
-        else:
-            for command in commands:
-                try:
-                    try: cmd, arg = command.split(' ', 1)
-                    except ValueError: self[command](*args)
-                    else:
-                        print_d("Running %r with params %r " % (cmd, arg))
-                        self[cmd](arg, *args)
-                except KeyError:
-                    commands = args[1].browser.commands
-                    try:
-                        try: cmd, arg = command.split(' ', 1)
-                        except ValueError: commands[command](*args)
-                        else: commands[cmd](arg, *args)
-                    except:
-                        print_w(_("Invalid command %r received.") % command)
-                except:
-                    e = sys.exc_info()[1]
-                    print_e(_("Error running command %r, caused by: %r.") %
-                        (command, e))
-            return True
 
-    def _previous(self, library, window, player): player.previous()
-    def _force_previous(self, library, window, player): player.previous(True)
-    def _next(self, library, window, player): player.next()
-    def _pause(self, library, window, player): player.paused = True
+        commands = source.read().rstrip("\n").splitlines()
+        for command in commands:
+            try:
+                try:
+                    cmd, arg = command.split(' ', 1)
+                except ValueError:
+                    self[command](*args)
+                else:
+                    print_d("Running %r with params %r " % (cmd, arg))
+                    self[cmd](arg, *args)
+            except KeyError:
+                commands = args[1].browser.commands
+                try:
+                    try:
+                        cmd, arg = command.split(' ', 1)
+                    except ValueError:
+                        commands[command](*args)
+                    else:
+                        commands[cmd](arg, *args)
+                except:
+                    print_w(_("Invalid command %r received.") % command)
+            except:
+                e = sys.exc_info()[1]
+                print_e(_("Error running command %r, caused by: %r.") %
+                    (command, e))
+        return True
+
+    def _previous(self, library, window, player):
+        player.previous()
+
+    def _force_previous(self, library, window, player):
+        player.previous(True)
+
+    def _next(self, library, window, player):
+        player.next()
+
+    def _pause(self, library, window, player):
+        player.paused = True
+
     def _play(self, library, window, player):
-        if player.song: player.paused = False
+        if player.song:
+            player.paused = False
+
     def _play_pause(self, library, window, player):
         if player.song is None:
             player.reset()
-        else: player.paused ^= True
+        else:
+            player.paused ^= True
+
+    def _stop(self, library, window, player):
+        player.stop()
 
     def _focus(self, library, window, player):
-        window.show()
-        window.present()
+        from quodlibet import app
+        app.present()
 
     def _volume(self, value, library, window, player):
         if value[0] in ('+', '-'):
             if len(value) > 1:
-                try: change = (int(value[1:]) / 100.0)
-                except ValueError: return
+                try:
+                    change = (int(value[1:]) / 100.0)
+                except ValueError:
+                    return
             else:
                 change = 0.05
-            if value[0] == '-': change = -change
+            if value[0] == '-':
+                change = -change
             volume = player.volume + change
         else:
-            try: volume = (int(value) / 100.0)
-            except ValueError: return
+            try:
+                volume = (int(value) / 100.0)
+            except ValueError:
+                return
         player.volume = min(1.0, max(0.0, volume))
 
     def _order(self, value, library, window, player):
@@ -137,24 +176,30 @@ class FIFOControl(object):
             order.set_active(
                 ["inorder", "shuffle", "weighted", "onesong"].index(value))
         except ValueError:
-            try: order.set_active(int(value))
+            try:
+                order.set_active(int(value))
             except (ValueError, TypeError):
                 if value in ["t", "toggle"]:
                     order.set_active(not order.get_active())
 
     def _repeat(self, value, library, window, player):
         repeat = window.repeat
-        if value in ["0", "off"]: repeat.set_active(False)
-        elif value in ["1", "on"]: repeat.set_active(True)
+        if value in ["0", "off"]:
+            repeat.set_active(False)
+        elif value in ["1", "on"]:
+            repeat.set_active(True)
         elif value in ["t", "toggle"]:
             repeat.set_active(not repeat.get_active())
 
     def _seek(self, time, library, window, player):
         seek_to = player.get_position()
-        if time[0] == "+": seek_to += util.parse_time(time[1:]) * 1000
-        elif time[0] == "-": seek_to -= util.parse_time(time[1:]) * 1000
-        else: seek_to = util.parse_time(time) * 1000
-        seek_to = min(player.song.get("~#length", 0) * 1000 -1,
+        if time[0] == "+":
+            seek_to += util.parse_time(time[1:]) * 1000
+        elif time[0] == "-":
+            seek_to -= util.parse_time(time[1:]) * 1000
+        else:
+            seek_to = util.parse_time(time) * 1000
+        seek_to = min(player.song.get("~#length", 0) * 1000 - 1,
                       max(0, seek_to))
         player.seek(seek_to)
 
@@ -172,15 +217,15 @@ class FIFOControl(object):
 
     def _add_directory(self, value, library, window, player):
         filename = os.path.normpath(os.path.realpath(value))
-        for added in library.scan([filename]): pass
-        if window.browser.can_filter(None):
-            window.browser.set_text(
+        for added in library.scan([filename]):
+            pass
+        if window.browser.can_filter_text():
+            window.browser.filter_text(
                 "filename = /^%s/c" % re.escape(filename))
-            window.browser.activate()
         else:
             basepath = filename + "/"
-            songs = [song for (filename, song) in library.iteritems()
-                     if filename.startswith(basepath)]
+            songs = [song for (fn, song) in library.iteritems()
+                     if fn.startswith(basepath)]
             songs.sort(reverse=True)
             queue = window.playlist.q
             for song in songs:
@@ -188,39 +233,47 @@ class FIFOControl(object):
         player.next()
 
     def _toggle_window(self, library, window, player):
-        if window.get_property('visible'): window.hide()
-        else: window.present()
+        from quodlibet import app
+        if window.get_property('visible'):
+            app.hide()
+        else:
+            app.show()
 
     def _hide_window(self, library, window, player):
-        window.hide()
+        from quodlibet import app
+        app.hide()
 
-    _show_window = _focus
+    def _show_window(self, library, window, player):
+        from quodlibet import app
+        app.show()
 
     def _set_rating(self, value, library, window, player):
         song = player.song
         if song:
-            try: song["~#rating"] = max(0.0, min(1.0, float(value)))
-            except (ValueError, TypeError): pass
-            else: library.changed([song])
+            try:
+                song["~#rating"] = max(0.0, min(1.0, float(value)))
+            except (ValueError, TypeError):
+                pass
+            else:
+                library.changed([song])
 
     def _set_browser(self, value, library, window, player):
         Kind = browsers.get(value)
         if Kind is not browsers.search.EmptyBar:
             window.select_browser(None, value, library, player)
-        else: print_w(_("Unknown browser %r.") % value)
+        else:
+            print_w(_("Unknown browser %r.") % value)
 
     def _open_browser(self, value, library, window, player):
         Kind = browsers.get(value)
         if Kind is not browsers.search.EmptyBar:
             LibraryBrowser(Kind, library)
-        else: print_w(_("Unknown browser %r.") % value)
+        else:
+            print_w(_("Unknown browser %r.") % value)
 
     def _random(self, tag, library, window, player):
         if window.browser.can_filter(tag):
-            values = window.browser.list(tag)
-            if values:
-                value = random.choice(values)
-                window.browser.filter(tag, [value])
+            window.browser.filter_random(tag)
 
     def _filter(self, value, library, window, player):
         tag, values = value.split('=', 1)
@@ -236,18 +289,27 @@ class FIFOControl(object):
             # no value given, use the current song; slide arguments
             # to the right.
             value, library, window, player = None, value, library, window
+
         if value:
-            if value in library: songs = [library[value]]
-            else: songs = library.query(value)
-            SongProperties(songs, library, parent=self)
-        else: SongProperties([player.song], library, parent=self)
+            if value in library:
+                songs = [library[value]]
+            else:
+                songs = library.query(value)
+        else:
+            songs = [player.song]
+        songs = filter(None, songs)
+
+        if songs:
+            SongProperties(library, songs, parent=window)
 
     def _enqueue(self, value, library, window, player):
         playlist = window.playlist
-        if value in library: songs = [library[value]]
+        if value in library:
+            songs = [library[value]]
         elif os.path.isfile(value):
             songs = [library.add_filename(os.path.realpath(value))]
-        else: songs = library.query(value)
+        else:
+            songs = library.query(value)
         songs.sort()
         playlist.enqueue(songs)
 
@@ -262,71 +324,85 @@ class FIFOControl(object):
                 song_path = URI(param).filename
             except ValueError:
                 song_path = param
-            if song_path in library: songs.append(library[song_path])
+            if song_path in library:
+                songs.append(library[song_path])
             elif os.path.isfile(song_path):
                 songs.append(library.add_filename(os.path.realpath(value)))
-        if songs: window.playlist.enqueue(songs)
+        if songs:
+            window.playlist.enqueue(songs)
 
     def _unqueue(self, value, library, window, player):
         playlist = window.playlist
-        if value in library: songs = [library[value]]
-        else: songs = library.query(value)
+        if value in library:
+            songs = [library[value]]
+        else:
+            songs = library.query(value)
         playlist.unqueue(songs)
 
     def _quit(self, library, window, player):
-        gtk.gdk.threads_enter()
-        window.destroy()
-        gtk.gdk.threads_leave()
+        from quodlibet import app
+        app.quit()
 
     def _status(self, value, library, window, player):
-        try: f = file(value, "w")
-        except EnvironmentError: pass
+        try:
+            f = file(value, "w")
+        except EnvironmentError:
+            pass
         else:
-            if player.paused: strings = ["paused"]
-            else: strings = ["playing"]
+            if player.paused:
+                strings = ["paused"]
+            else:
+                strings = ["playing"]
             strings.append(type(window.browser).__name__)
-            strings.append("%0.3f" % window.volume.get_value())
+            strings.append("%0.3f" % player.volume)
             strings.append(window.order.get_active_name())
             strings.append((window.repeat.get_active() and "on") or "off")
             progress = 0
             if player.info:
                 length = player.info.get("~#length", 0)
-                if length: progress = player.get_position() / (length * 1000.0)
+                if length:
+                    progress = player.get_position() / (length * 1000.0)
             strings.append("%0.3f" % progress)
             f.write(" ".join(strings) + "\n")
-            try: f.write(window.browser.status + "\n")
-            except AttributeError: pass
+            try:
+                f.write(window.browser.status + "\n")
+            except AttributeError:
+                pass
             f.close()
 
     def _song_list(self, value, library, window, player):
         if value.startswith("t"):
             value = not window.song_scroller.get_property('visible')
-        else: value = value not in ['0', 'off', 'false']
+        else:
+            value = value not in ['0', 'off', 'false']
         window.song_scroller.set_property('visible', value)
 
     def _queue(self, value, library, window, player):
         if value.startswith("t"):
             value = not window.qexpander.get_property('visible')
-        else: value = value not in ['0', 'off', 'false']
+        else:
+            value = value not in ['0', 'off', 'false']
         window.qexpander.set_property('visible', value)
 
     def _dump_playlist(self, value, library, window, player):
-        try: f = file(value, "w")
-        except EnvironmentError: pass
+        try:
+            f = file(value, "w")
+        except EnvironmentError:
+            pass
         else:
             for song in window.playlist.pl.get():
                 f.write(song("~uri") + "\n")
             f.close()
 
     def _dump_queue(self, value, library, window, player):
-        try: f = file(value, "w")
-        except EnvironmentError: pass
+        try:
+            f = file(value, "w")
+        except EnvironmentError:
+            pass
         else:
             for song in window.playlist.q.get():
                 f.write(song("~uri") + "\n")
             f.close()
 
     def _refresh(self, library, window, player):
-        paths = util.split_scan_dirs(config.get("settings", "scan"))
-        progress = window.statusbar.progress
-        copool.add(library.rebuild, paths, progress, False, funcid="library")
+        scan_libary(library, False)
