@@ -8,7 +8,7 @@
 import os
 import unicodedata
 
-import gtk
+from gi.repository import Gtk, Gdk
 
 from quodlibet import const
 from quodlibet import qltk
@@ -16,20 +16,27 @@ from quodlibet import util
 
 from quodlibet.parse import FileFromPattern
 from quodlibet.qltk._editpane import EditPane, FilterCheckButton
+from quodlibet.qltk._editpane import EditingPluginHandler
+from quodlibet.qltk.views import TreeViewColumn
 from quodlibet.qltk.wlw import WritingWindow
 
+
 class SpacesToUnderscores(FilterCheckButton):
     _label = _("Replace spaces with _underscores")
     _section = "rename"
     _key = "spaces"
     _order = 1.0
-    def filter(self, original, filename): return filename.replace(" ", "_")
+
+    def filter(self, original, filename):
+        return filename.replace(" ", "_")
+
 
 class StripWindowsIncompat(FilterCheckButton):
     _label = _("Strip _Windows-incompatible characters")
     _section = "rename"
     _key = "windows"
     _order = 1.1
+
     def __init__(self):
         super(StripWindowsIncompat, self).__init__()
         # If on Windows, force this to be inactive (and hidden)
@@ -37,53 +44,66 @@ class StripWindowsIncompat(FilterCheckButton):
             self.set_active(False)
             self.set_sensitive(False)
             self.set_no_show_all(True)
+
     def filter(self, original, filename):
         return util.strip_win32_incompat(filename)
 
+
 class StripDiacriticals(FilterCheckButton):
     _label = _("Strip _diacritical marks")
     _section = "rename"
     _key = "diacriticals"
     _order = 1.2
+
     def filter(self, original, filename):
         return filter(lambda s: not unicodedata.combining(s),
                       unicodedata.normalize('NFKD', filename))
 
+
 class StripNonASCII(FilterCheckButton):
     _label = _("Strip non-_ASCII characters")
     _section = "rename"
     _key = "ascii"
     _order = 1.3
+
     def filter(self, original, filename):
         return u"".join(map(lambda s: (s <= "~" and s) or u"_", filename))
 
+
 class Lowercase(FilterCheckButton):
     _label = _("Use only _lowercase characters")
     _section = "rename"
     _key = "lowercase"
     _order = 1.4
+
     def filter(self, original, filename):
         return filename.lower()
 
+
+class RenameFilesPluginHandler(EditingPluginHandler):
+    from quodlibet.plugins.editing import RenameFilesPlugin
+    Kind = RenameFilesPlugin
+
+
 class RenameFiles(EditPane):
     title = _("Rename Files")
     FILTERS = [SpacesToUnderscores, StripWindowsIncompat, StripDiacriticals,
                StripNonASCII, Lowercase]
+    handler = RenameFilesPluginHandler()
 
     def __init__(self, parent, library):
-        plugins = parent.plugins.RenamePlugins()
         super(RenameFiles, self).__init__(
-            const.NBP, const.NBP_EXAMPLES.split("\n"), plugins)
+            const.NBP, const.NBP_EXAMPLES.split("\n"))
 
-        column = gtk.TreeViewColumn(
-            _('File'), gtk.CellRendererText(), text=1)
-        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
+        column = TreeViewColumn(
+            _('File'), Gtk.CellRendererText(), text=1)
+        column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         self.view.append_column(column)
-        render = gtk.CellRendererText()
+        render = Gtk.CellRendererText()
         render.set_property('editable', True)
 
-        column = gtk.TreeViewColumn(_('New Name'), render, text=2)
-        column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
+        column = TreeViewColumn(_('New Name'), render, text=2)
+        column.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         self.view.append_column(column)
 
         self.preview.connect_object('clicked', self.__preview, None)
@@ -113,31 +133,34 @@ class RenameFiles(EditPane):
                 newname = util.fsnative(newname)
                 library.rename(song, newname, changed=was_changed)
             except StandardError:
-                if skip_all: continue
+                util.print_exc()
+                if skip_all:
+                    continue
                 RESPONSE_SKIP_ALL = 1
                 buttons = (_("Ignore _All Errors"), RESPONSE_SKIP_ALL,
-                           gtk.STOCK_STOP, gtk.RESPONSE_CANCEL,
-                           _("_Continue"), gtk.RESPONSE_OK)
+                           Gtk.STOCK_STOP, Gtk.ResponseType.CANCEL,
+                           _("_Continue"), Gtk.ResponseType.OK)
                 msg = qltk.Message(
-                    gtk.MESSAGE_ERROR, win, _("Unable to rename file"),
+                    Gtk.MessageType.ERROR, win, _("Unable to rename file"),
                     _("Renaming <b>%s</b> to <b>%s</b> failed. "
                       "Possibly the target file already exists, "
                       "or you do not have permission to make the "
-                      "new file or remove the old one.") %(
+                      "new file or remove the old one.") % (
                     util.escape(util.fsdecode(oldname)),
                     util.escape(util.fsdecode(newname))),
-                    buttons=gtk.BUTTONS_NONE)
+                    buttons=Gtk.ButtonsType.NONE)
                 msg.add_buttons(*buttons)
-                msg.set_default_response(gtk.RESPONSE_OK)
+                msg.set_default_response(Gtk.ResponseType.OK)
                 resp = msg.run()
                 skip_all |= (resp == RESPONSE_SKIP_ALL)
                 # Preserve old behavior: shift-click is Ignore All
-                mods = gtk.gdk.display_get_default().get_pointer()[3]
-                skip_all |= mods & gtk.gdk.SHIFT_MASK
+                mods = Gdk.Display.get_default().get_pointer()[3]
+                skip_all |= mods & Gdk.ModifierType.SHIFT_MASK
                 library.reload(song, changed=was_changed)
-                if resp != gtk.RESPONSE_OK and resp != RESPONSE_SKIP_ALL:
+                if resp != Gtk.ResponseType.OK and resp != RESPONSE_SKIP_ALL:
                     break
-            if win.step(): break
+            if win.step():
+                break
 
         self.view.thaw_child_notify()
         win.destroy()
@@ -148,7 +171,7 @@ class RenameFiles(EditPane):
         model = self.view.get_model()
         if songs is None:
             songs = [row[0] for row in model]
-        pattern = self.combo.child.get_text().decode("utf-8")
+        pattern = self.combo.get_child().get_text().decode("utf-8")
 
         try:
             pattern = FileFromPattern(pattern)
@@ -158,28 +181,30 @@ class RenameFiles(EditPane):
                 _("The pattern\n\t<b>%s</b>\ncontains / but "
                   "does not start from root. To avoid misnamed "
                   "folders, root your pattern by starting "
-                  "it with / or ~/.")%(
+                  "it with / or ~/.") % (
                 util.escape(pattern))).run()
             return
         else:
-            if self.combo.child.get_text():
-                self.combo.prepend_text(self.combo.child.get_text())
+            if self.combo.get_child().get_text():
+                self.combo.prepend_text(self.combo.get_child().get_text())
                 self.combo.write(const.NBP)
 
         orignames = [song["~filename"] for song in songs]
         newnames = [util.fsdecode(util.fsencode(pattern.format(song)))
                     for song in songs]
         for f in self.filters:
-            if f.active: newnames = f.filter_list(orignames, newnames)
+            if f.active:
+                newnames = f.filter_list(orignames, newnames)
 
         model.clear()
         for song, newname in zip(songs, newnames):
             basename = util.fsdecode(song("~basename"))
             model.append(row=[song, basename, newname])
         self.preview.set_sensitive(False)
-        self.save.set_sensitive(bool(self.combo.child.get_text()))
+        self.save.set_sensitive(bool(self.combo.get_child().get_text()))
         for song in songs:
             if not song.is_file:
                 self.set_sensitive(False)
                 break
-        else: self.set_sensitive(True)
+        else:
+            self.set_sensitive(True)
