@@ -8,7 +8,7 @@
 import os
 import re
 
-import gtk
+from gi.repository import Gtk
 
 from quodlibet import config
 from quodlibet import const
@@ -16,24 +16,26 @@ from quodlibet import qltk
 from quodlibet import util
 
 from quodlibet.qltk._editpane import EditPane, FilterCheckButton
+from quodlibet.qltk._editpane import EditingPluginHandler
 from quodlibet.qltk.wlw import WritingWindow
 
+
 class TagsFromPattern(object):
     def __init__(self, pattern):
         self.compile(pattern)
 
     def compile(self, pattern):
         self.headers = []
-        self.slashes = len(pattern) - len(pattern.replace(os.path.sep,'')) + 1
+        self.slashes = len(pattern) - len(pattern.replace(os.path.sep, '')) + 1
         self.pattern = None
         # patterns look like <tagname> non regexy stuff <tagname> ...
         pieces = re.split(r'(<[A-Za-z0-9~_]+>)', pattern)
-        override = { '<tracknumber>': r'\d\d?', '<discnumber>': r'\d\d??' }
+        override = {'<tracknumber>': r'\d\d?', '<discnumber>': r'\d\d??'}
         dummies_found = 0
         for i, piece in enumerate(pieces):
             if not piece:
                 continue
-            if piece[0]+piece[-1] == '<>':
+            if piece[0] + piece[-1] == '<>':
                 piece = piece.lower()   # canonicalize to lowercase tag names
                 if "~" in piece:
                     dummies_found += 1
@@ -64,53 +66,69 @@ class TagsFromPattern(object):
         # only match on the last n pieces of a filename, dictated by pattern
         # this means no pattern may effectively cross a /, despite .* doing so
         sep = os.path.sep
-        matchon = sep+sep.join(song.split(sep)[-self.slashes:])
+        matchon = sep + sep.join(song.split(sep)[-self.slashes:])
         match = self.pattern.search(matchon)
 
         # dicts for all!
-        if match is None: return {}
-        else: return match.groupdict()
+        if match is None:
+            return {}
+        else:
+            return match.groupdict()
+
 
 class UnderscoresToSpaces(FilterCheckButton):
     _label = _("Replace _underscores with spaces")
     _section = "tagsfrompath"
     _key = "underscores"
     _order = 1.0
-    def filter(self, tag, value): return value.replace("_", " ")
+
+    def filter(self, tag, value):
+        return value.replace("_", " ")
+
 
 class TitleCase(FilterCheckButton):
     _label = _("_Title-case tags")
     _section = "tagsfrompath"
     _key = "titlecase"
     _order = 1.1
-    def filter(self, tag, value): return util.title(value)
+
+    def filter(self, tag, value):
+        return util.title(value)
+
 
 class SplitTag(FilterCheckButton):
     _label = _("Split into multiple _values")
     _section = "tagsfrompath"
     _key = "split"
     _order = 1.2
+
     def filter(self, tag, value):
         spls = config.get("editing", "split_on").decode('utf-8', 'replace')
         spls = spls.split()
         return "\n".join(util.split_value(value, spls))
 
+
+class TagsFromPathPluginHandler(EditingPluginHandler):
+    from quodlibet.plugins.editing import TagsFromPathPlugin
+    Kind = TagsFromPathPlugin
+
+
 class TagsFromPath(EditPane):
     title = _("Tags From Path")
     FILTERS = [UnderscoresToSpaces, TitleCase, SplitTag]
-        
+    handler = TagsFromPathPluginHandler()
+
     def __init__(self, parent, library):
-        plugins = parent.plugins.TagsFromPathPlugins()
         super(TagsFromPath, self).__init__(
-            const.TBP, const.TBP_EXAMPLES.split("\n"), plugins)
+            const.TBP, const.TBP_EXAMPLES.split("\n"))
 
         vbox = self.get_children()[2]
-        addreplace = gtk.combo_box_new_text()
+        addreplace = Gtk.ComboBoxText()
         addreplace.append_text(_("Tags replace existing ones"))
         addreplace.append_text(_("Tags are added to existing ones"))
         addreplace.set_active(config.getboolean("tagsfrompath", "add"))
         addreplace.connect('changed', self.__add_changed)
-        vbox.pack_start(addreplace)
+        vbox.pack_start(addreplace, True, True, 0)
         addreplace.show()
 
         self.preview.connect_object('clicked', self.__preview, None)
@@ -126,15 +144,18 @@ class TagsFromPath(EditPane):
         if songs is None:
             songs = [row[0] for row in (self.view.get_model() or [])]
 
-        if songs: pattern_text = self.combo.child.get_text().decode("utf-8")
-        else: pattern_text = ""
-        try: pattern = TagsFromPattern(pattern_text)
+        if songs:
+            pattern_text = self.combo.get_child().get_text().decode("utf-8")
+        else:
+            pattern_text = ""
+        try:
+            pattern = TagsFromPattern(pattern_text)
         except re.error:
             qltk.ErrorMessage(
                 self, _("Invalid pattern"),
                 _("The pattern\n\t<b>%s</b>\nis invalid. "
                   "Possibly it contains the same tag twice or "
-                  "it has unbalanced brackets (&lt; / &gt;).")%(
+                  "it has unbalanced brackets (&lt; / &gt;).") % (
                 util.escape(pattern_text))).run()
             return
         else:
@@ -161,22 +182,22 @@ class TagsFromPath(EditPane):
             pattern = TagsFromPattern("")
 
         self.view.set_model(None)
-        model = gtk.ListStore(
+        model = Gtk.ListStore(
             object, str, *([str] * len(pattern.headers)))
         for col in self.view.get_columns():
             self.view.remove_column(col)
 
-        col = gtk.TreeViewColumn(_('File'), gtk.CellRendererText(),
+        col = Gtk.TreeViewColumn(_('File'), Gtk.CellRendererText(),
                                  text=1)
-        col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
+        col.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         self.view.append_column(col)
         for i, header in enumerate(pattern.headers):
-            render = gtk.CellRendererText()
+            render = Gtk.CellRendererText()
             render.set_property('editable', True)
             render.connect('edited', self.__row_edited, model, i + 2)
             escaped_title = header.replace("_", "__")
-            col = gtk.TreeViewColumn(escaped_title, render, text=i + 2)
-            col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
+            col = Gtk.TreeViewColumn(escaped_title, render, text=i + 2)
+            col.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
             self.view.append_column(col)
 
         for song in songs:
@@ -186,19 +207,21 @@ class TagsFromPath(EditPane):
             for h in pattern.headers:
                 text = match.get(h, '')
                 for f in self.filters:
-                    if f.active: text = f.filter(h, text)
+                    if f.active:
+                        text = f.filter(h, text)
                 if not song.multiple_values:
                     text = u", ".join(text.split("\n"))
                 row.append(text)
             model.append(row=row)
 
         # save for last to potentially save time
-        if songs: self.view.set_model(model)
+        if songs:
+            self.view.set_model(model)
         self.preview.set_sensitive(False)
         self.save.set_sensitive(len(pattern.headers) > 0)
 
     def __save(self, addreplace, library):
-        pattern_text = self.combo.child.get_text().decode('utf-8')
+        pattern_text = self.combo.get_child().get_text().decode('utf-8')
         pattern = TagsFromPattern(pattern_text)
         model = self.view.get_model()
         add = bool(addreplace.get_active())
@@ -214,7 +237,7 @@ class TagsFromPath(EditPane):
                 _("<b>%s</b> changed while the program was running. "
                   "Saving without refreshing your library may "
                   "overwrite other changes to the song.\n\n"
-                  "Save this song anyway?") %(
+                  "Save this song anyway?") % (
                 util.escape(util.fsdecode(song("~basename"))))
                 ).run():
                 break
@@ -232,20 +255,22 @@ class TagsFromPath(EditPane):
                                 changed = True
 
             if changed:
-                try: song.write()
+                try:
+                    song.write()
                 except:
                     qltk.ErrorMessage(
                         self, _("Unable to edit song"),
                         _("Saving <b>%s</b> failed. The file "
                           "may be read-only, corrupted, or you "
-                          "do not have permission to edit it.")%(
+                          "do not have permission to edit it.") % (
                         util.escape(util.fsdecode(song('~basename'))))
                         ).run()
                     library.reload(song, changed=was_changed)
                     break
                 was_changed.append(song)
 
-            if win.step(): break
+            if win.step():
+                break
 
         win.destroy()
         library.changed(was_changed)
