@@ -6,6 +6,7 @@
 # published by the Free Software Foundation
 
 import os
+import stat
 import sys
 import errno
 import urllib
@@ -16,11 +17,14 @@ from os.path import isdir, basename, exists, splitext
 
 from quodlibet.util import xdg_get_data_home, find_mount_point
 
+
 class TrashError(EnvironmentError):
     pass
 
+
 def is_sticky(path):
-    return bool(os.stat(path).st_mode & (1<9))
+    return bool(os.stat(path).st_mode & stat.S_ISVTX)
+
 
 def get_fd_trash_dir(path):
     """Returns the right trash directory for the given path."""
@@ -41,6 +45,7 @@ def get_fd_trash_dir(path):
             root = join(mount, ".Trash-" + uid)
         return root
 
+
 def trash_free_desktop(path):
     """Partial implementation of
     http://www.freedesktop.org/wiki/Specifications/trash-spec
@@ -59,13 +64,14 @@ def trash_free_desktop(path):
     # to make things easier
     if path.startswith(join(trash_dir, "")) or path == trash_dir:
         raise TrashError("Can't move files to the trash from within the"
-            "trash directory.")
+                         "trash directory.")
 
     files = join(trash_dir, "files")
     info = join(trash_dir, "info")
 
     for d in (files, info):
-        if not isdir(d): os.makedirs(d, 0700)
+        if not isdir(d):
+            os.makedirs(d, 0700)
 
     info_ext = ".trashinfo"
     name = basename(path)
@@ -114,10 +120,12 @@ def trash_free_desktop(path):
         os.unlink(info_path)
         raise
 
+
 def can_trash():
     """If the current platform supports moving files into a trash can."""
     return (os.name == "posix" and sys.platform != "darwin")
 
+
 def trash(path):
     if os.name == "posix" and sys.platform != "darwin":
         trash_free_desktop(path)
