@@ -1,14 +1,14 @@
-import gobject, gtk
 from tests import TestCase, add
 
-import widgets
-from player import PlaylistPlayer
-from browsers.filesystem import FileSystem
-from qltk.watcher import SongWatcher
+from quodlibet.browsers.filesystem import FileSystem
+from quodlibet.player.nullbe import NullPlayer
+from quodlibet.library import SongLibrary
+import quodlibet.config
 
 class TFileSystem(TestCase):
     def setUp(self):
-        self.bar = FileSystem(SongWatcher(), PlaylistPlayer('fakesink'))
+        quodlibet.config.init()
+        self.bar = FileSystem(SongLibrary(), NullPlayer())
 
     def test_can_filter(self):
         for key in ["foo", "title", "fake~key", "~woobar", "~#huh"]:
@@ -17,4 +17,5 @@ class TFileSystem(TestCase):
 
     def tearDown(self):
         self.bar.destroy()
+        quodlibet.config.quit()
 add(TFileSystem)
