@@ -1,8 +1,7 @@
 from tests import TestCase, add
 
-import gtk
+from gi.repository import Gtk
 
-from quodlibet import widgets
 import quodlibet.browsers.search
 import quodlibet.config
 
@@ -54,8 +53,7 @@ class TEmptyBar(TestCase):
         self.expected = None
 
     def _do(self):
-        self.bar.activate()
-        while gtk.events_pending(): gtk.main_iteration()
+        while Gtk.events_pending(): Gtk.main_iteration()
         self.failUnless(self.expected is None)
 
     def test_can_filter(self):
@@ -63,15 +61,14 @@ class TEmptyBar(TestCase):
             self.failUnless(self.bar.can_filter(key))
 
     def test_empty_is_all(self):
-        self.bar.set_text("")
+        self.bar.filter_text("")
         self.expected = list(sorted(SONGS))
         self._do()
 
     def test_dynamic(self):
         self.failUnless(self.bar.dynamic(SONGS[0]))
-        self.bar.set_text("this does not match any song")
+        self.bar.filter_text("this does not match any song")
         self.expected = []
-        self.bar.activate()
         self.failIf(self.bar.dynamic(SONGS[0]))
 
     def test_filter(self):
@@ -99,11 +96,11 @@ class TEmptyBar(TestCase):
         self.bar.filter("~#length", [0])
 
     def test_saverestore(self):
-        self.bar.set_text("title = %s" % SONGS[0]["title"])
+        self.bar.filter_text("title = %s" % SONGS[0]["title"])
         self.expected = [SONGS[0]]
         self._do()
         self.bar.save()
-        self.bar.set_text("")
+        self.bar.filter_text("")
         self.expected = list(sorted(SONGS))
         self._do()
         self.bar.restore()
