@@ -216,6 +216,47 @@ class TID3File(TestCase):
 
         self.failUnlessAlmostEqual(song.replay_gain(["track"]), 1.0, 1)
 
+    def test_foobar2k_replaygain_read_new(self):
+        # Others don't like RVA2, so we have to read/write foobar style
+        # http://code.google.com/p/quodlibet/issues/detail?id=1027
+        f = mutagen.File(self.filename)
+        f.tags.add(mutagen.id3.TXXX(encoding=3, desc="replaygain_track_gain",
+                                    text=["-6 db"]))
+        f.tags.add(mutagen.id3.TXXX(encoding=3, desc="replaygain_track_peak",
+                                    text=["0.9"]))
+        f.tags.add(mutagen.id3.TXXX(encoding=3, desc="replaygain_album_gain",
+                                    text=["3 db"]))
+        f.tags.add(mutagen.id3.TXXX(encoding=3, desc="replaygain_album_peak",
+                                    text=["0.8"]))
+        f.save()
+
+        song = MP3File(self.filename)
+        self.failUnlessEqual(song("replaygain_track_gain"), "-6 db")
+        self.failUnlessEqual(song("replaygain_track_peak"), "0.9")
+        self.failUnlessEqual(song("replaygain_album_gain"), "3 db")
+        self.failUnlessEqual(song("replaygain_album_peak"), "0.8")
+
+        # still prefer RVA2 in case there are both
+        f.tags.add(mutagen.id3.RVA2(desc="track", channel=1,
+                                    gain=-9, peak=1.0))
+        f.save()
+        song = MP3File(self.filename)
+        self.failUnlessAlmostEqual(song.replay_gain(["track"]), 0.35, 1)
+
+    def test_foobar2k_replaygain_write_new(self):
+        # Others don't like RVA2, so we have to read/write foobar style
+        # http://code.google.com/p/quodlibet/issues/detail?id=1027
+        song = MP3File(self.filename)
+        song["replaygain_track_gain"] = "-6 db"
+        song["replaygain_track_peak"] = "0.9"
+        song["replaygain_album_gain"] = "3 db"
+        song["replaygain_album_peak"] = "0.8"
+        song.write()
+
+        f = mutagen.File(self.filename)
+        for k in ["track_peak", "track_gain", "album_peak", "album_gain"]:
+            self.failUnless(f["TXXX:replaygain_" + k])
+
     def test_quodlibet_txxx_inval(self):
         # This shouldn't happen in our namespace, but check anyway since
         # we might open the whole TXXX namespace sometime
@@ -382,6 +423,15 @@ class TID3File(TestCase):
         f = mutagen.File(self.filename)
         self.failUnlessEqual(dict(f.tags["TMCL"].people)["quux"], "foo")
 
+    def test_rva_large(self):
+        song = MP3File(self.filename)
+        song["replaygain_track_peak"] = "3"
+        song["replaygain_track_gain"] = "100"
+        song.write()
+        song["replaygain_track_peak"] = "-1"
+        song["replaygain_track_gain"] = "-100"
+        song.write()
+
     def test_rva(self):
         f = mutagen.File(self.filename)
         f.tags.add(mutagen.id3.RVA2(desc="track", channel=1,
