@@ -1,17 +1,36 @@
 from tests import add, TestCase
 
 import os
+import sys
 import shutil
 import tempfile
 import base64
 
 from quodlibet import config, const, formats
-from quodlibet.formats.xiph import OggFile, FLACFile
+from quodlibet.formats.xiph import OggFile, FLACFile, OggOpusFile, OggOpus
 
 from mutagen.flac import FLAC, Picture
 from mutagen.id3 import ID3, TIT2, ID3NoHeaderError
 from mutagen.oggvorbis import OggVorbis
 
+
+class TXiphPickle(TestCase):
+    # make sure the classes are available at the old paths
+    # so unpickling old libraries works.
+
+    def test_modules_flac(self):
+        self.failUnless("formats.flac" in sys.modules)
+        mod = sys.modules["formats.flac"]
+        self.failUnless(mod.FLACFile is FLACFile)
+
+    def test_modules_vorbis(self):
+        self.failUnless("formats.oggvorbis" in sys.modules)
+        mod = sys.modules["formats.oggvorbis"]
+        self.failUnless(mod.OggFile is OggFile)
+
+add(TXiphPickle)
+
+
 class TVCFile(TestCase):
     # Mixin to test Vorbis writing features
 
@@ -79,22 +98,6 @@ class TVCFile(TestCase):
         song = type(self.song)(self.filename)
         self.failUnlessEqual(song["~#playcount"], count)
 
-    def test_totaltracks(self):
-        self.song["tracknumber"] = "1"
-        self.song["totaltracks"] = "1"
-        self.song.write()
-        song = type(self.song)(self.filename)
-        self.failUnlessEqual(song["tracknumber"], "1/1")
-        self.failIf("totaltracks" in song)
-
-    def test_tracktotal(self):
-        self.song["tracknumber"] = "1"
-        self.song["tracktotal"] = "1"
-        self.song.write()
-        song = type(self.song)(self.filename)
-        self.failUnlessEqual(song["tracknumber"], "1/1")
-        self.failIf("tracktotal" in song)
-
     def test_parameter(self):
         for bad in ["rating", "playcount", "rating:foo", "playcount:bar"]:
             self.failIf(self.song.can_change(bad))
@@ -110,6 +113,142 @@ class TVCFile(TestCase):
     def test_can_change(self):
         self.failUnless(self.song.can_change())
 
+class TTotalTagsBase(TestCase):
+    """Test conversation between the tracknumber/totaltracks/tracktotal
+    format and the tracknumber="x/y" format.
+
+    """
+
+    MAIN = None
+    FALLBACK = None
+    SINGLE = None
+
+    def setUp(self):
+        config.init()
+        self.filename = tempfile.mkstemp(".ogg")[1]
+        shutil.copy(os.path.join('tests', 'data', 'empty.ogg'), self.filename)
+
+    def tearDown(self):
+        os.unlink(self.filename)
+        config.quit()
+
+    def __load_tags(self, tags, expected):
+        m = OggVorbis(self.filename)
+        for key, value in tags.iteritems():
+            m.tags[key] = value
+        m.save()
+        song = OggFile(self.filename)
+        for key, value in expected.iteritems():
+            self.failUnlessEqual(song(key), value)
+        if self.MAIN not in expected:
+            self.failIf(self.MAIN in song)
+        if self.SINGLE not in expected:
+            self.failIf(self.SINGLE in song)
+        if self.FALLBACK not in expected:
+            self.failIf(self.FALLBACK in song)
+
+    def test_load_old_single(self):
+        self.__load_tags(
+            {self.SINGLE: "1/42"},
+            {self.SINGLE: "1/42"})
+
+    def test_load_main(self):
+        self.__load_tags(
+            {self.SINGLE: "3", self.MAIN: "10"},
+            {self.SINGLE: "3/10"})
+
+    def test_load_fallback(self):
+        self.__load_tags(
+            {self.SINGLE: "3", self.FALLBACK: "10"},
+            {self.SINGLE: "3/10"})
+
+    def test_load_all(self):
+        self.__load_tags(
+            {self.SINGLE: "3", self.FALLBACK: "10", self.MAIN: "5"},
+            {self.SINGLE: "3/5", self.FALLBACK: "10"})
+
+    def test_load_main_no_single(self):
+        self.__load_tags(
+            {self.MAIN: "5"},
+            {self.SINGLE: "/5"})
+
+    def test_load_fallback_no_single(self):
+        self.__load_tags(
+            {self.FALLBACK: "6"},
+            {self.SINGLE: "/6"})
+
+    def test_load_both_no_single(self):
+        self.__load_tags(
+            {self.FALLBACK: "6", self.MAIN: "5"},
+            {self.FALLBACK: "6", self.SINGLE: "/5"})
+
+    def __save_tags(self, tags, expected):
+        #return
+        song = OggFile(self.filename)
+        for key, value in tags.iteritems():
+            song[key] = value
+        song.write()
+        m = OggVorbis(self.filename)
+        # test if all values ended up where we wanted
+        for key, value in expected.iteritems():
+            self.failUnless(key in m.tags)
+            self.failUnlessEqual(m.tags[key], [value])
+
+        # test if not specified are not there
+        if self.MAIN not in expected:
+            self.failIf(self.MAIN in m.tags)
+        if self.FALLBACK not in expected:
+            self.failIf(self.FALLBACK in m.tags)
+        if self.SINGLE not in expected:
+            self.failIf(self.SINGLE in m.tags)
+
+    def test_save_single(self):
+        self.__save_tags(
+            {self.SINGLE: "1/2"},
+            {self.SINGLE: "1", self.MAIN: "2"})
+
+    def test_save_main(self):
+        self.__save_tags(
+            {self.MAIN: "3"},
+            {self.MAIN: "3"})
+
+    def test_save_fallback(self):
+        self.__save_tags(
+            {self.FALLBACK: "3"},
+            {self.MAIN: "3"})
+
+    def test_save_single_and_main(self):
+        # not clear what to do here...
+        self.__save_tags(
+            {self.SINGLE: "1/2", self.MAIN: "3"},
+            {self.SINGLE: "1", self.MAIN: "3"})
+
+    def test_save_single_and_fallback(self):
+        self.__save_tags(
+            {self.SINGLE: "1/2", self.FALLBACK: "3"},
+            {self.SINGLE: "1", self.MAIN: "2", self.FALLBACK: "3"})
+
+    def test_save_all(self):
+        # not clear what to do here...
+        self.__save_tags(
+            {self.SINGLE: "1/2", self.MAIN: "4", self.FALLBACK: "3"},
+            {self.SINGLE: "1", self.MAIN: "4", self.FALLBACK: "3"})
+
+
+class TTrackTotal(TTotalTagsBase):
+    MAIN = "tracktotal"
+    FALLBACK = "totaltracks"
+    SINGLE = "tracknumber"
+add(TTrackTotal)
+
+
+class TDiscTotal(TTotalTagsBase):
+    MAIN = "disctotal"
+    FALLBACK = "totaldiscs"
+    SINGLE = "discnumber"
+add(TDiscTotal)
+
+
 class TFLACFile(TVCFile):
     def setUp(self):
         TVCFile.setUp(self)
@@ -117,6 +256,9 @@ class TFLACFile(TVCFile):
         shutil.copy(os.path.join('tests', 'data', 'empty.flac'), self.filename)
         self.song = FLACFile(self.filename)
 
+    def test_mime(self):
+        self.failUnless(self.song.mimes)
+
     def test_save_empty(self):
         self.song.write()
         flac = FLAC(self.filename)
@@ -146,10 +288,10 @@ class TVCCover(TestCase):
         config.init()
 
     def __get_jpeg(self, size=5):
-        import gtk
-        pb = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, size, size)
+        from gi.repository import GdkPixbuf
+        pb = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, False, 8, size, size)
         fn = tempfile.NamedTemporaryFile()
-        pb.save(fn.name, "jpeg")
+        pb.savev(fn.name, "jpeg", [], [])
         return fn.read()
 
     def test_no_cover(self):
@@ -303,3 +445,23 @@ class TOggFile(TVCFile):
         os.unlink(self.filename)
         config.quit()
 add(TOggFile)
+
+
+class TOggOpusFile(TVCFile):
+    def setUp(self):
+        TVCFile.setUp(self)
+        self.filename = tempfile.mkstemp(".ogg")[1]
+        shutil.copy(os.path.join('tests', 'data', 'empty.opus'), self.filename)
+        self.song = OggOpusFile(self.filename)
+
+    def test_length(self):
+        self.failUnlessEqual(self.song("~#length"), 3)
+        self.failUnless("opusenc" in self.song("encoder"))
+
+    def tearDown(self):
+        os.unlink(self.filename)
+        config.quit()
+
+if OggOpus:
+    add(TOggOpusFile)
+
