@@ -1,14 +1,19 @@
-from unittest import TestCase
-from tests import registerCase
+from tests import TestCase, add
+
+import os
+
+from quodlibet import formats
+from quodlibet import config
+
 from shutil import copyfileobj
-from tempfile import mkstemp, mkdtemp
-import os, formats
+from tempfile import mkstemp
 
 class TestMetaData(TestCase):
     base = 'tests/data/silence-44-s'
 
     def setUp(self):
         """Copy the base silent file to a temp name/location and load it"""
+        config.init()
         fd, self.filename = mkstemp(suffix=self.ext, text=False)
         dst = os.fdopen(fd, 'w')
         src = open(self.base + self.ext, 'rb')
@@ -21,6 +26,7 @@ class TestMetaData(TestCase):
         os.remove(self.filename)
         del self.filename
         del self.song
+        config.quit()
 
     def test_base_data(self):
         self.failUnlessEqual(self.song['artist'], 'piman\njzig')
@@ -60,6 +66,11 @@ class TestMetaData(TestCase):
         self._test_tag('genre', [u'Pop', u'Rock\nClassical', u'Big Bird',
              u'\u30a2\u30cb\u30e1\u30b5\u30f3\u30c8\u30e9',])
 
+    def test_odd_performer(self):
+        values = [u"A Person", u"Another"]
+        self._test_tag("performer:vocals", values)
+        self._test_tag("performer:guitar", values)
+
     def test_wackjob(self): # undefined tag
         self._test_tag('wackjob', [u'Jelly\nDanish', u'Muppet',
              u'\u30cf\u30f3\u30d0\u30fc\u30ac\u30fc'])
@@ -67,7 +78,8 @@ class TestMetaData(TestCase):
 tags = ['album', 'arranger', 'artist', 'author', 'comment', 'composer',
 'conductor', 'copyright', 'discnumber', 'encodedby', 'genre', 'isrc',
 'language', 'license', 'lyricist', 'organization', 'performer', 'title',
-'tracknumber', 'version', 'xyzzy_undefined_tag', 'musicbrainz_trackid']
+'tracknumber', 'version', 'xyzzy_undefined_tag', 'musicbrainz_trackid',
+'releasecountry']
 
 for ext in formats._infos.keys():
     if os.path.exists(TestMetaData.base + ext):
@@ -82,4 +94,4 @@ for ext in formats._infos.keys():
 
         testcase = type('MetaData' + ext, (TestMetaData,), extra_tests)
         testcase.ext = ext
-        registerCase(testcase)
+        add(testcase)
