@@ -1,19 +1,29 @@
 # -*- coding: utf-8 -*-
-import pygst
-pygst.require("0.10")
-import gst
 
+import contextlib
+
+from gi.repository import Gst
+
+import unittest
 from tests import TestCase, add
 
 from quodlibet.player.gstbe import GStreamerSink as Sink
 from quodlibet.player.gstbe import parse_gstreamer_taglist
 from quodlibet.util import sanitize_tags
 
+
+@contextlib.contextmanager
+def ignore_gst_errors():
+    old = Gst.debug_get_default_threshold()
+    Gst.debug_set_default_threshold(Gst.DebugLevel.NONE)
+    yield
+    Gst.debug_set_default_threshold(old)
+
+
 class TGStreamerSink(TestCase):
     def test_simple(self):
-        import gst
         sinks = ["gconfaudiosink", "alsasink"]
-        for n in filter(gst.element_factory_find, sinks):
+        for n in filter(Gst.ElementFactory.find, sinks):
             obj, name = Sink(n)
             self.failUnless(obj)
             self.failUnlessEqual(name, n)
@@ -22,7 +32,8 @@ class TGStreamerSink(TestCase):
         import __builtin__
         pw = print_w
         __builtin__.__dict__["print_w"] = lambda *x: None
-        obj, name = Sink("notarealsink")
+        with ignore_gst_errors():
+            obj, name = Sink("notarealsink")
         __builtin__.__dict__["print_w"] = pw
         self.failUnless(obj)
         self.failUnlessEqual(name, "autoaudiosink")
@@ -36,7 +47,7 @@ add(TGStreamerSink)
 
 class TGstreamerTagList(TestCase):
     def test_parse(self):
-        # gst.TagList can't be filled using pygtk... so use a dict instead
+        # gst.TagList can't be filled using pyGtk... so use a dict instead
 
         l = {}
         l["extended-comment"] = u"foo=bar"
@@ -49,17 +60,15 @@ class TGstreamerTagList(TestCase):
 
 
         # date is abstract, so define our own
+        # (might work with pygobject now)
         class Foo(object):
-            year = 3000
-            month = 10
-            day = 2
+            def to_iso8601_string(self):
+                return "3000-10-2"
         l["date"] = Foo()
-        date = gst.Date
-        gst.Date = Foo
+        date = Gst.DateTime
+        Gst.DateTime = Foo
         self.failUnlessEqual(parse_gstreamer_taglist(l)["date"], "3000-10-2")
-        gst.Date = date
-
-        self.failIf(parse_gstreamer_taglist({"bla": ["xyz"]}))
+        Gst.DateTime = date
 
         l["foo"] = u"äöü"
         self.failUnless(isinstance(parse_gstreamer_taglist(l)["foo"], unicode))
@@ -78,7 +87,7 @@ class TGstreamerTagList(TestCase):
         l["bar"] = 9
         self.failUnlessEqual(parse_gstreamer_taglist(l)["bar"], 9)
 
-        l["bar"] = gst.TagList() # some random gst instance
+        l["bar"] = Gst.TagList() # some random gst instance
         self.failUnless(isinstance(parse_gstreamer_taglist(l)["bar"], unicode))
         self.failUnless("GstTagList" in parse_gstreamer_taglist(l)["bar"])
 
