@@ -2,11 +2,9 @@ from tests import TestCase, add
 
 import os
 
-from tempfile import mkstemp, mkdtemp
-
 from quodlibet import config
 from quodlibet.formats._audio import AudioFile
-from quodlibet.plugins import SongWrapper, ListWrapper
+from quodlibet.util.songwrapper import SongWrapper, ListWrapper
 
 class TSongWrapper(TestCase):
 
