@@ -1,5 +1,7 @@
-import os, glob
-from tests import add, TestCase
+from tests import TestCase, add
+
+import os
+import glob
 
 class PO(TestCase):
     def test_pos(self):
