@@ -36,8 +36,29 @@ class TComboBoxEntrySave(TestCase):
             self.failUnlessEqual(row1[1], row2[1])
             self.failUnlessEqual(row1[2], row2[2])
 
+    def test_text_changed_signal(self):
+        called = [0]
+
+        def cb(*args):
+            called[0] += 1
+
+        def get_count():
+            c = called[0]
+            called[0] = 0
+            return c
+
+        self.cbes.connect("text-changed", cb)
+        entry = self.cbes.get_child()
+        entry.set_text("foo")
+        self.failUnlessEqual(get_count(), 1)
+        self.cbes.prepend_text("bar")
+        # in case the model got changed but the entry is still the same
+        # the text-changed signal should not be triggered
+        self.failUnlessEqual(entry.get_text(), "foo")
+        self.failUnlessEqual(get_count(), 0)
+
     def test_shared_model(self):
-        self.cbes.append_text("a test")
+        self.cbes.prepend_text("a test")
         self.test_equivalence()
 
     def test_initial_size(self):
@@ -55,7 +76,7 @@ class TComboBoxEntrySave(TestCase):
         self.failUnlessEqual(self.saved, file(self.fname + ".saved").read())
 
     def test_set_text_then_prepend(self):
-        self.cbes.child.set_text("foobar")
+        self.cbes.get_child().set_text("foobar")
         self.cbes.prepend_text("foobar")
         self.memory = "foobar\npattern 1\n"
         self.test_save()
