@@ -1,7 +1,5 @@
 from tests import TestCase, add
 
-import os
-
 from quodlibet.qltk.chooser import FolderChooser, FileChooser
 import quodlibet.config
 
@@ -24,4 +22,8 @@ add(TFolderChooser)
 
 class TFileChooser(TFolderChooser):
     Kind = FileChooser
+
+    def test_filter(self):
+        f = lambda *x: None
+        FileChooser(None, "foo", filter=f)
 add(TFileChooser)
