@@ -1,5 +1,6 @@
-from tests import add, TestCase
-from qltk.delete import DeleteDialog
+from tests import TestCase, add
+
+from quodlibet.qltk.delete import DeleteDialog
 
 class TDeleteDialog(TestCase):
     def setUp(self):
