@@ -10,7 +10,7 @@ class TExFalsoWindow(TestCase):
         self.ef = ExFalsoWindow(SongLibrary())
 
     def test_nothing(self):
-        self.failUnless(self.ef.child)
+        self.failUnless(self.ef.get_child())
 
     def tearDown(self):
         self.ef.destroy()
