@@ -1,6 +1,9 @@
-from tests import add, TestCase
-from qltk.information import Information
-from formats._audio import AudioFile
+from tests import TestCase, add
+
+from quodlibet.formats._audio import AudioFile
+from quodlibet.library import SongLibrary
+from quodlibet.qltk.information import Information
+import quodlibet.config
 
 def AF(*args, **kwargs):
     a = AudioFile(*args, **kwargs)
@@ -9,31 +12,32 @@ def AF(*args, **kwargs):
 
 class TInformation(TestCase):
     def setUp(self):
-        from qltk.watcher import SongWatcher
-        self.watcher = SongWatcher()
+        quodlibet.config.init()
+        self.library = SongLibrary()
 
     def test_none(self):
-        Information(self.watcher, []).destroy()
+        Information(self.library, []).destroy()
 
     def test_one(self):
         f = AF({"~filename": "/dev/null"})
-        Information(self.watcher, [f]).destroy()
+        Information(self.library, [f]).destroy()
 
     def test_two(self):
         f = AF({"~filename": "/dev/null"})
         f2 = AF({"~filename": "/dev/null2"})
-        Information(self.watcher, [f, f2]).destroy()
+        Information(self.library, [f, f2]).destroy()
 
     def test_album(self):
         f = AF({"~filename": "/dev/null", "album": "woo"})
         f2 = AF({"~filename": "/dev/null2", "album": "woo"})
-        Information(self.watcher, [f, f2]).destroy()
+        Information(self.library, [f, f2]).destroy()
 
     def test_artist(self):
         f = AF({"~filename": "/dev/null", "artist": "woo"})
         f2 = AF({"~filename": "/dev/null2", "artist": "woo"})
-        Information(self.watcher, [f, f2]).destroy()
+        Information(self.library, [f, f2]).destroy()
 
     def tearDown(self):
-        self.watcher.destroy()
+        self.library.destroy()
+        quodlibet.config.quit()
 add(TInformation)
