@@ -1,5 +1,6 @@
-from tests import add, TestCase
-from qltk.msg import *
+from tests import TestCase, add
+
+from quodlibet.qltk.msg import *
 
 class _TMessage(TestCase):
     def setUp(self): self.win = self.Kind(None, "title", "description")
