@@ -2,9 +2,11 @@ from tests import TestCase, add
 
 from quodlibet.qltk.pluginwin import PluginWindow
 import quodlibet.config
+import quodlibet.plugins
 
 class TPluginWindow(TestCase):
     def setUp(self):
+        quodlibet.plugins.init()
         quodlibet.config.init()
         self.win = PluginWindow(None)
 
@@ -13,5 +15,6 @@ class TPluginWindow(TestCase):
 
     def tearDown(self):
         self.win.destroy()
+        quodlibet.plugins.quit()
         quodlibet.config.quit()
 add(TPluginWindow)
