@@ -9,7 +9,7 @@ class TSongsMenu(TestCase):
     def setUp(self):
         self.library = SongLibrary()
         backend = quodlibet.player.init("nullbe")
-        self.device = quodlibet.player.init_device(self.library)
+        self.device = backend.init(self.library)
         
         self.songs = [AudioFile({"title": x}) for x in
                       ["song1", "song2", "song3"]]
@@ -56,7 +56,11 @@ class TSongsMenu(TestCase):
         self.menu = SongsMenu(
             self.library, self.songs, plugins=False, playlists=False,
             queue=False, devices=True, remove=False, delete=False, edit=False)
-        self.failUnlessEqual(1, len(self.menu))
+        from quodlibet import browsers
+        if browsers.media.MediaDevices in browsers.browsers:
+            self.failUnlessEqual(1, len(self.menu))
+        else:
+            self.failUnlessEqual(0, len(self.menu))
 
     def test_remove(self):
         self.menu = SongsMenu(
@@ -88,7 +92,7 @@ class TSongsMenu(TestCase):
         self.failIf(self.menu.get_children()[0].props.sensitive)
 
     def tearDown(self):
-        quodlibet.player.quit(self.device)
+        self.device.destroy()
         self.library.destroy()
         try: self.menu.destroy()
         except AttributeError: pass
