@@ -1,6 +1,6 @@
 from tests import TestCase, add
 
-import gtk
+from gi.repository import Gtk
 
 from quodlibet.formats._audio import AudioFile
 from quodlibet.player.nullbe import NullPlayer
@@ -21,13 +21,14 @@ class TSongTracker(TestCase):
         self.current = None
 
     def do(self):
-        while gtk.events_pending(): gtk.main_iteration()
+        while Gtk.events_pending(): Gtk.main_iteration()
 
     def test_play(self):
         import time
-        # Allow at least 1 second to elapse to simulate playing
+        # Allow at least 2 second to elapse to simulate playing
+        self.p.song = self.s1
         self.p.paused = False
-        time.sleep(1.1)
+        time.sleep(2)
         self.do()
         self.p.emit('song-ended', self.s1, False)
         self.do()
@@ -45,7 +46,7 @@ class TSongTracker(TestCase):
 
     def test_error(self):
         self.current = self.p.song = self.s1
-        self.p.error('Test error', None)
+        self.p.error('Test error')
         self.do()
         self.assertEquals(self.s1["~#playcount"], 0)
         self.assertEquals(self.s1["~#skipcount"], 0)
