@@ -62,21 +62,54 @@ add(Tunexpand)
 
 class Tformat_rating(TestCase):
     def test_empty(self):
-        self.failUnlessEqual(util.format_rating(0), "")
+        self.failUnlessEqual(util.format_rating(0, blank=False), "")
 
     def test_full(self):
         self.failUnlessEqual(
-            len(util.format_rating(1)), int(1/util.RATING_PRECISION))
-    def test_rating(self):
+            len(util.format_rating(1, blank=False)),
+            int(1/util.RATING_PRECISION))
+
+    def test_rating_length(self):
+        util.RATING_PRECISION = 4
         for i in range(0, int(1/util.RATING_PRECISION+1)):
             self.failUnlessEqual(
-                i, len(util.format_rating(i * util.RATING_PRECISION)))
+                i, len(util.format_rating(i * util.RATING_PRECISION,
+                                          blank=False)))
 
     def test_bogus(self):
         max_length = int(1 / util.RATING_PRECISION)
-        self.failUnlessEqual(len(util.format_rating(2**32-1)), max_length)
+        self.failUnlessEqual(len(util.format_rating(2**32-1, blank=False)),
+                             max_length)
+        self.failUnlessEqual(len(util.format_rating(-4.2, blank=False)), 0)
 
-        self.failUnlessEqual(len(util.format_rating(-4.2)), 0)
+    def test_blank_lengths(self):
+        """Check that there are no unsuspected edge-cases
+        for various rating precisions"""
+        for util.RATING_PRECISION in [0.1, 0.2, 0.25, 1/3.0, 0.5]:
+            steps = int(1/util.RATING_PRECISION)
+            self.failUnlessEqual(len(util.format_rating(1)), steps)
+            self.failUnlessEqual(len(util.format_rating(0)), steps)
+            self.failUnlessEqual(len(util.format_rating(0.5)), steps)
+            self.failUnlessEqual(len(util.format_rating(1/3.0)), steps)
+
+    def test_blank_values(self):
+        util.RATING_PRECISION = 0.2
+        util.BLANK_RATING_SYMBOL = "0"
+        util.RATING_SYMBOL = "1"
+        # Easy ones first
+        self.failUnlessEqual(util.format_rating(0.0), "00000")
+        self.failUnlessEqual(util.format_rating(0.2), "10000")
+        self.failUnlessEqual(util.format_rating(0.8), "11110")
+        self.failUnlessEqual(util.format_rating(1.0), "11111")
+        # A bit arbitrary, but standard behaviour
+        self.failUnlessEqual(util.format_rating(0.5), "11100")
+        # Test rounding down...
+        self.failUnlessEqual(util.format_rating(0.6), "11100")
+        # Test rounding up...
+        self.failUnlessEqual(util.format_rating(0.9), "11111")
+        # You never know...
+        self.failUnlessEqual(util.format_rating(3.0), "11111")
+        self.failUnlessEqual(util.format_rating(-0.5), "00000")
 
 add(Tformat_rating)
 
@@ -329,6 +362,19 @@ class Tsplit_people(TestCase):
             util.split_people("foo (Ft. bar)"), ("foo", ["bar"]))
         self.failUnlessEqual(
             util.split_people("foo(feat barman)"), ("foo", ["barman"]))
+    def test_originally_by(self):
+        self.failUnlessEqual(
+            util.split_people("title (originally by artist)"),
+            ("title", ["artist"]))
+        self.failUnlessEqual(
+            util.split_people("title [originally by artist & artist2]"),
+            ("title", ["artist", "artist2"]))
+    def test_cover(self):
+        self.failUnlessEqual(
+            util.split_people("Pyscho Killer [Talking Heads Cover]"),
+            ("Pyscho Killer", ["Talking Heads"]))
+
+
 
 add(Tsplit_people)
 
@@ -445,8 +491,8 @@ class Tspawn(TestCase):
         self.failUnless(util.spawn(["ls", "."], stdout=True))
 
     def test_invalid(self):
-        import gobject
-        self.failUnlessRaises(gobject.GError, util.spawn, ["not a command"])
+        from gi.repository import GLib
+        self.failUnlessRaises(GLib.GError, util.spawn, ["not a command"])
 
     def test_types(self):
         self.failUnlessRaises(TypeError, util.spawn, [u"ls"])
