@@ -1,6 +1,7 @@
 from tests import TestCase, add
 
-import gtk
+from gi.repository import Gtk, GdkPixbuf
+
 import os
 import urllib
 try:
@@ -13,11 +14,11 @@ from quodlibet.util import mtime
 
 class TThumb(TestCase):
     def setUp(s):
-        s.wide = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 150, 10)
-        s.high = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 10, 100)
-        s.small = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, 10, 20)
+        s.wide = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 150, 10)
+        s.high = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 10, 100)
+        s.small = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB, True, 8, 10, 20)
         s.filename = os.path.join(os.getcwd(), "test_thumbnail.png")
-        s.wide.save(s.filename, "png")
+        s.wide.savev(s.filename, "png", [], [])
 
     def tearDown(s):
         os.remove(s.filename)
@@ -46,12 +47,16 @@ class TThumb(TestCase):
         uri = "file://" + urllib.pathname2url(s.filename)
         name = hash.md5(uri).hexdigest() + ".png"
         path = os.path.expanduser("~/.thumbnails")
+        # newer spec
+        new_path = os.path.expanduser("~/.cache/thumbnails")
+        if os.path.exists(new_path):
+            path = new_path
         path = os.path.join(path, "normal", name)
 
         s.failUnless(os.path.isfile(path))
 
         #check for metadata
-        thumb_pb = gtk.gdk.pixbuf_new_from_file(path)
+        thumb_pb = GdkPixbuf.Pixbuf.new_from_file(path)
         meta_mtime = thumb_pb.get_option("tEXt::Thumb::MTime")
         meta_uri = thumb_pb.get_option("tEXt::Thumb::URI")
 
